import * as React from "react";

import {loadAjax} from "../api/ajax";
import {SelectElement} from "./components/Select";
import {GraphComponent, INode,} from "yfiles/view-component";
import {
    // BalloonLayout,
    CircularLayout,
    GraphOverviewComponent,
    HierarchicLayout,
    ICommand,
    ILayoutAlgorithm,
    Insets,
    LayoutExecutor,
    MinimumNodeSizeStage,
    OrganicLayout,
    OrthogonalLayout,
    RadialLayout,
    SeriesParallelLayout,
    TimeSpan,
} from "yfiles";
import {GraphRenderer} from "./components/GraphRenderer";
import {RackProps, RackRenderer} from "./components/RackRenderer";
import {FixGroupLayoutStage} from "yfiles/router-polyline";
import {Tooltip} from "./components/Tooltip";
import {GraphEditorInputMode} from "yfiles/view-editor";
import {StructureView} from "./components/StructureView";
import {HighlightClickInputMode} from "./components/HighlightClickInputMode";
import {Defaults} from "./components/Defaults";
import {DynamicFilter, DynamicLevel, ObjectTypeElement} from "./components/DynamicFilter";
import {Preset, PresetSelector} from "./components/PresetSelector";
import PdfExport, {PaperSize} from "./components/PdfExport";
import FileSaveSupport from "./components/FileSaveSupport";
import {DisplaySettings, EDGE_COLORING_DEFAULT, Settings} from "./components/Settings";
import {ContextMenu} from "./components/ContextMenu";

export interface AnalyzerComposeProps {
    ajax_url: string,
    connection_url: string,
    image_url: string,
    object_url: string,
    session: string,
    language: {[key: string]: string},
    dynamic_levels: DynamicLevel[],
    presets?: Preset[],
    connector_types: {[key: string]: string},
}

interface AnalyzerComposeState {
    dynamic_levels: DynamicLevel[],
    object_types_next_level: ObjectTypeElement[],
    selected_preset: number,
    result: SelectElement[],
    is_loading: boolean,
    racks?: RackProps[],
    display_settings: DisplaySettings
}

export interface Layout {
    id: number,
    title: string,
    instance: ILayoutAlgorithm,
}

export class AnalyzerCompose extends React.Component<AnalyzerComposeProps, AnalyzerComposeState> {

    private readonly div: React.RefObject<HTMLDivElement>;
    private readonly overview_div: React.RefObject<HTMLDivElement>;
    private readonly structure_div: React.RefObject<HTMLDivElement>;
    private readonly graphComponent: GraphComponent;
    private readonly overviewGraphComponent: GraphOverviewComponent;
    private structureView: StructureView;
    private readonly layouts: Layout[];

    constructor(props: AnalyzerComposeProps) {
        super(props);

        this.state = {
            dynamic_levels: this.props.dynamic_levels,
            object_types_next_level: [],
            selected_preset: 1,
            result: [],
            is_loading: false,
            display_settings: {
                show_rack_view: false,
                show_capacity: false,
                show_only_connected_nodes: false,
                show_node_images: true,
                neighbor_edges: 0,
                connector_types: [],
                current_layout: 1,
                edge_coloring: EDGE_COLORING_DEFAULT,
                routing_shortest_path: true,
                routing_connection_count: 1,
            }
        };

        this.loadElementsByType = this.loadElementsByType.bind(this)
        this.filterElementsByLevel = this.filterElementsByLevel.bind(this);
        this.handleElementSelect = this.handleElementSelect.bind(this);
        this.handleAddFilter = this.handleAddFilter.bind(this)
        this.handleFilterObjectTypeSelection = this.handleFilterObjectTypeSelection.bind(this)
        this.handleRemoveLastFilter = this.handleRemoveLastFilter.bind(this)
        this.handlePresetSelect = this.handlePresetSelect.bind(this)
        this.handleUpdate = this.handleUpdate.bind(this);
        this.setLoadState = this.setLoadState.bind(this);
        this.renderGraph = this.renderGraph.bind(this);
        this.renderRackGraph = this.renderRackGraph.bind(this);
        this.handlePdfExport = this.handlePdfExport.bind(this)
        this.handleAnimateLayout = this.handleAnimateLayout.bind(this)
        this.handleLayoutChange = this.handleLayoutChange.bind(this)
        this.updateDisplaySetting = this.updateDisplaySetting.bind(this)
        this.getCurrentLayout = this.getCurrentLayout.bind(this)

        this.div = React.createRef<HTMLDivElement>();

        this.graphComponent = new GraphComponent('#yGraph');

        Defaults.setDefaults(this.graphComponent, this.state.display_settings);

        this.layouts = [
            {id: 1, title: this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__LAYOUT_HIERARCHIC, instance: new HierarchicLayout()},
            {id: 2, title: this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__LAYOUT_ORTHOGONAL, instance: new OrthogonalLayout()},
            {id: 3, title: this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__LAYOUT_CIRCULAR, instance: new CircularLayout()},
            {id: 4, title: this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__LAYOUT_ORGANIC, instance: new OrganicLayout()},
            {id: 5, title: this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__LAYOUT_RADIAL, instance: new RadialLayout()},
            {id: 6, title: this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__LAYOUT_SERIES_PARALLEL, instance: new SeriesParallelLayout()},
            // {id: 7, title: this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__LAYOUT_BALLOON, instance: new BalloonLayout()},
        ]

        // add overview
        this.overview_div = React.createRef<HTMLDivElement>();
        this.overviewGraphComponent = new GraphOverviewComponent('yGraphOverview', this.graphComponent);

        // add structure
        this.structure_div = React.createRef<HTMLDivElement>();
        this.structureView = new StructureView('#yGraphStructure');
        this.structureView.setGraph(this.graphComponent.graph);
        this.structureView.onElementClicked = (node: INode) => {
            const viewNode = this.graphComponent.graph.foldingView
                ? this.graphComponent.graph.foldingView.getViewItem(node)
                : node;
            if (viewNode) {
                this.graphComponent.currentItem = viewNode;
                this.graphComponent.selection.clear();
                this.graphComponent.selection.setSelected(viewNode, true);
                ICommand.ZOOM_TO_CURRENT_ITEM.execute(null, this.graphComponent);
            }
        }
    }

    componentDidMount() {
        // Append the GraphComponent to the DOM
        this.div.current!.appendChild(this.graphComponent.div);
        this.overview_div.current!.appendChild(this.overviewGraphComponent.div);

        // Append the structure view to the DOM
        this.structure_div.current!.appendChild(this.structureView.getContainer());

        // Initialize tooltip
        let tooltip = new Tooltip(this.graphComponent, this.props.language);
        tooltip.initializeTooltips();

        // Initialize context menu
        const contextMenu = new ContextMenu(this.graphComponent, this.props.language, this.props.object_url, this.props.connection_url)
        contextMenu.configureContextMenu()
    }

    loadElementsByType(type_id: number, obj_id?: number) {
        if (! type_id || type_id == 0) {
            return
        }

        this.setLoadState(true);
        let url = this.props.ajax_url + '&func=get-elements-by-type&type_id=' + type_id
        if (obj_id) {
            url += '&obj_id=' + obj_id
        }
        loadAjax(url, this.props.session)
            .then(res => {
                this.setLoadState(false);
                let mod_levels: DynamicLevel[] = []
                this.state.dynamic_levels.forEach(l => {
                    if (l.type_id == type_id) {
                        l.objects = res.data
                    }
                    mod_levels.push(l)
                });
                this.setState((prevState) => ({
                    ...prevState,
                    dynamic_levels: mod_levels,
                }));
            });
    }

    handleUpdate() {
        this.filterElementsByLevel();
    }

    filterElementsByLevel(obj_id?: number, level?: number) {
        if (! level) {
            level = this.state.selected_preset
        }

        if (! obj_id || obj_id < 0) {
            this.state.dynamic_levels.forEach(level => {
                // get last selected ID
                obj_id = level.id && level.id != 0 ? level.id : obj_id
            })
        }

        this.setLoadState(true);
        const preset = this.props.presets.filter(p => p.id == level)[0]
        if (preset.show_rack_view && this.state.display_settings.show_rack_view) {
            loadAjax(this.props.ajax_url + '&func=get-racks&obj_id=' + obj_id + '&level=' + level, this.props.session)
                .then(res => {
                    this.setLoadState(false);
                    this.setState((prevState) => ({
                        ...prevState,
                        racks: {...res.data},
                    }), () => {
                        this.renderRackGraph();
                    });
                })
                .catch(() => {
                    this.setLoadState(false);
                });
        } else {
            const connector_types = this.state.display_settings.connector_types.join(',')
            loadAjax(this.props.ajax_url + '&func=filter-elements-by-level&obj_id=' +
                obj_id + '&level=' + level + '&connector_types=' + connector_types, this.props.session)
                .then(res => {
                    this.setLoadState(false);
                    this.setState((prevState) => ({
                        ...prevState,
                        result: {...res.data},
                    }), () => {
                        this.renderGraph();
                    });
                })
                .catch(() => {
                    this.setLoadState(false);
                });
        }
    }

    async renderGraph() {
        let renderer = new GraphRenderer(this.graphComponent.graph, this.props.image_url, this.state.display_settings.show_node_images);
        renderer.render(this.state.result, 'device', this.state.display_settings);

        this.graphComponent.fitGraphBounds();
        this.structureView.buildStructure();

        const layoutExecutor = new LayoutExecutor(this.graphComponent, this.getCurrentLayout());
        layoutExecutor.duration = TimeSpan.fromSeconds(1);
        layoutExecutor.easedAnimation = true;
        layoutExecutor.animateViewport = true;
        await layoutExecutor.start();
    }

    async renderRackGraph() {
        this.graphComponent.graph.clear();

        let renderer = new RackRenderer(this.graphComponent.graph, this.state.display_settings);
        renderer.render(this.state.racks);
        this.graphComponent.fitGraphBounds();
        this.structureView.buildStructure();

        const layout = new HierarchicLayout();
        const stage = new FixGroupLayoutStage(layout);

        const layoutExecutor = new LayoutExecutor(this.graphComponent, stage);
        layoutExecutor.duration = TimeSpan.fromSeconds(1);
        layoutExecutor.easedAnimation = true;
        layoutExecutor.animateViewport = true;
        await layoutExecutor.start();
    }

    setLoadState(is_loading: boolean) {
        this.setState((prevState) => ({
            ...prevState,
            is_loading: is_loading,
        }));
    }

    handleElementSelect(obj_id: number, type: string) {
        // updated selected filter element, set elements underneath to null and get object type of next level
        const type_id = parseInt(type)
        let found_level = false
        let next_type: number = null

        let mod_levels: DynamicLevel[] = []
        this.state.dynamic_levels.forEach(level => {
            if (found_level) {
                level.id = null
                next_type = next_type ? next_type : level.type_id
            }
            if (level.type_id == type_id) {
                found_level = true
                level.id = obj_id > 0 ? obj_id : null
            }
            mod_levels.push(level)
        })

        this.setState((prevState) => ({
            ...prevState,
            dynamic_levels: mod_levels,
        }), () => {
            this.loadElementsByType(next_type, obj_id)
        });
    }

    handleAddFilter() {
        let obj_id: number = null
        this.state.dynamic_levels.forEach(level => {
            // get last selected ID
            obj_id = level.id && level.id != 0 ? level.id : obj_id
        })

        // load all elements underneath this object
        this.setLoadState(true);
        loadAjax(this.props.ajax_url + '&func=get-elements-underneath_object&obj_id=' + obj_id, this.props.session)
            .then(res => {
                this.setLoadState(false);

                const object_types: ObjectTypeElement[] = res.data
                this.setState((prevState) => ({
                    ...prevState,
                    object_types_next_level: object_types,
                }), () => {
                    if (object_types.length == 1) {
                        // load object type directly if there is only one
                        const object_type = object_types[0]
                        this.handleFilterObjectTypeSelection(object_type.id)
                    }
                });
            })
            .catch(() => {
                this.setLoadState(false);
            });
    }

    handleFilterObjectTypeSelection(type_id: number) {
        if (! this.state.object_types_next_level || this.state.object_types_next_level.length == 0) {
            return
        }

        // add selected object type from next level to dynamic filter
        const object_type = this.state.object_types_next_level.filter(o => o.id == type_id)[0]
        let levels = this.state.dynamic_levels
        levels.push({
            id: null,
            title: null,
            type_id: object_type.id,
            type_const: object_type.const,
            type_title: object_type.title,
            type_color: object_type.color,
            objects: object_type.objects
        })

        this.setState((prevState) => ({
            ...prevState,
            dynamic_levels: levels,
            object_types_next_level: [],
        }));
    }

    handleRemoveLastFilter(type_id: number) {
        const levels = this.state.dynamic_levels.filter(l => l.type_id != type_id)
        this.setState((prevState) => ({
            ...prevState,
            dynamic_levels: levels,
            object_types_next_level: [],
        }));
    }

    handlePresetSelect(id: number) {
        this.setState((prevState) => ({
            ...prevState,
            selected_preset: id,
        }));
    }

    async handlePdfExport() {
        const scale = 1
        const margin = 5
        const rectangle = this.graphComponent.contentRect

        // configure export, export the PDF and show a dialog to save the PDF file
        let clientSidePdfExport = new PdfExport()
        clientSidePdfExport.scale = scale
        clientSidePdfExport.margins = new Insets(margin)
        clientSidePdfExport.paperSize = PaperSize.A4
        const {raw} = await clientSidePdfExport.exportPdf(this.graphComponent.graph, rectangle)
        FileSaveSupport.save(raw, 'graph.pdf').catch(() => {
            alert(
                'Saving directly to the filesystem is not supported by this browser. Please use the server based export instead.'
            )
        })
    }

    handleLayoutChange(layout: string) {
        this.updateDisplaySetting('current_layout', parseInt(layout), this.handleAnimateLayout)
    }

    updateDisplaySetting(key: string, value: string|number|number[]|boolean, callback?: Function) {
        let settings = this.state.display_settings
        settings[key] = value
        this.setState(() => ({
            display_settings: settings,
        }), () => {
            // update click input mode
            let inputMode = this.graphComponent.inputMode as GraphEditorInputMode
            let mode = inputMode.clickInputMode as HighlightClickInputMode
            mode.display_settings = this.state.display_settings

            if (callback) {
                callback()
            }
        });
    }

    async handleAnimateLayout() {
        try {
            await this.graphComponent.morphLayout({
                layout: new MinimumNodeSizeStage(this.getCurrentLayout()),
                morphDuration: '1s',
                easedAnimation: true
            })
        } catch (error) {
        }
    }

    getCurrentLayout(): ILayoutAlgorithm {
        const l = this.layouts.filter(l => l.id == this.state.display_settings.current_layout)[0]
        return l.instance;
    }

    render() {
        let spinner_url = this.props.image_url + 'ajax-loading.gif';
        let spinner;
        if (this.state.is_loading) {
            spinner = <img src={spinner_url} alt="..." />;
        }

        let layout_opts: {[key: number]: string} = []
        this.layouts.forEach(l => {
            layout_opts[l.id] = l.title
        })

        return <div>
            <div className="analyzer-container">
                <div className="settings_left">
                    <div className="overview" ref={this.overview_div} />
                    <DynamicFilter
                        levels={this.state.dynamic_levels}
                        object_types_next_level={this.state.object_types_next_level}
                        language={this.props.language}
                        is_loading={this.state.is_loading}
                        onChange={this.handleElementSelect}
                        onAddFilter={this.handleAddFilter}
                        onRemoveLastFilter={this.handleRemoveLastFilter}
                        onObjectTypeSelected={this.handleFilterObjectTypeSelection} />
                    <PresetSelector
                        onChange={this.handlePresetSelect}
                        language={this.props.language}
                        is_loading={this.state.is_loading}
                        selected_preset={this.state.selected_preset}
                        presets={this.props.presets} />
                    <button type="button"
                            className="btn"
                            disabled={this.state.is_loading}
                            onClick={this.handleUpdate}>
                        {this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_UPDATE']}
                        {spinner}
                    </button>
                    <div className="structure" ref={this.structure_div} />
                </div>
                <div className="graph">
                    <div className="graph-component-container" ref={this.div} />
                </div>
            </div>
            <Settings display_settings={this.state.display_settings}
                      connector_types={this.props.connector_types}
                      layouts={this.layouts}
                      language={this.props.language}
                      is_loading={this.state.is_loading}
                      onDisplaySettingChange={this.updateDisplaySetting}
                      onHandleLayoutChange={this.handleLayoutChange}
                      onHandlePdfExport={this.handlePdfExport} />
        </div>;
    }
}
