import {GraphComponent, GraphEditorInputMode, IModelItem, INode, PopulateItemContextMenuEventArgs} from "yfiles/view-component";
import {Point, TimeSpan} from "yfiles";
import {SelectElement} from "./Select";

export class ContextMenu {

    private readonly graphComponent: GraphComponent;
    private readonly language: {[key: string]: string}
    private readonly object_url: string
    private readonly connection_url: string

    element: HTMLElement
    focusOutListener: (e: FocusEvent) => void
    focusInListener: (e: FocusEvent) => void
    closeListener: (e: Event) => void
    closeOnEscListener: (e: KeyboardEvent) => void
    blurredTimeout: number | null
    isOpen: boolean
    private onClosedCallbackField: () => void

    /**
     * Creates a new empty menu.
     *
     * @param {GraphComponent} graphComponent The graph component of this context menu.
     * @param {[key: string]: string} language
     * @param {string} object_url
     * @param {string} connection_url
     */
    constructor(graphComponent: GraphComponent, language: {[key: string]: string}, object_url: string, connection_url: string) {
        this.graphComponent = graphComponent;
        this.language = language
        this.object_url = object_url
        this.connection_url = connection_url

        const contextMenu = document.createElement('div')
        contextMenu.setAttribute('class', 'analyzer-ctx-menu')
        this.element = contextMenu
        this.blurredTimeout = null
        this.isOpen = false
        this.onClosedCallbackField = null!

        // Listeners for focus events since this menu closes itself if it loses the focus.
        this.focusOutListener = (evt: FocusEvent): void => {
            this.onFocusOut(evt.relatedTarget as HTMLElement)
        }

        this.focusInListener = (): void => {
            if (this.blurredTimeout) {
                clearTimeout(this.blurredTimeout)
                this.blurredTimeout = null
            }
        }

        // A click listener that closes the menu and calls the onCloseCallback.
        // This way, the individual menu items do not need to handle this by themselves.
        this.closeListener = (evt): void => {
            evt.stopPropagation()
            this.close()
            // Set the focus to the graph component
            graphComponent.focus()
            this.onClosedCallback()
        }

        // A ESC key press listener that closes the menu and calls the callback.
        this.closeOnEscListener = (evt): void => {
            if (evt.keyCode === 27 && this.element.parentNode) {
                this.closeListener(evt)
            }
        }

        // We lower the default long-press duration to 100ms to avoid conflicts between the context menu and
        // other gestures on long-press, e.g. edge creation.
        graphComponent.longPressTime = TimeSpan.fromMilliseconds(100)

        this.configureContextMenu = this.configureContextMenu.bind(this);
        this.populateContextMenu = this.populateContextMenu.bind(this);
        this.updateSelection = this.updateSelection.bind(this);
    }

    /**
     * Initializes the context menu.
     */
    configureContextMenu(): void {
        const graphComponent = this.graphComponent
        const inputMode = graphComponent.inputMode as GraphEditorInputMode

        // Create a context menu. In this demo, we use our sample context menu implementation but you can use any other
        // context menu widget as well. See the Context Menu demo for more details about working with context menus.
        const contextMenu = this; //new ContextMenu(graphComponent, this.language)

        // Add event listeners to the various events that open the context menu. These listeners then
        // call the provided callback function which in turn asks the current ContextMenuInputMode if a
        // context menu should be shown at the current location.
        contextMenu.addOpeningEventListeners(graphComponent, location => {
            if (inputMode.contextMenuInputMode.shouldOpenMenu(graphComponent.toWorldFromPage(location))) {
                contextMenu.show(location)
            }
        })

        // Add an event listener that populates the context menu according to the hit elements, or cancels showing a menu.
        // This PopulateItemContextMenu is fired when calling the ContextMenuInputMode.shouldOpenMenu method above.
        inputMode.addPopulateItemContextMenuListener((sender, args) =>
            this.populateContextMenu(contextMenu, args)
        )

        // Add a listener that closes the menu when the input mode requests this
        inputMode.contextMenuInputMode.addCloseMenuListener(() => {
            contextMenu.close()
        })

        // If the context menu closes itself, for example because a menu item was clicked, we must inform the input mode
        contextMenu.onClosedCallback = () => {
            inputMode.contextMenuInputMode.menuClosed()
        }
    }

    /**
     * Populates the context menu based on the item the mouse hovers over.
     *
     * @param contextMenu The context menu.
     * @param args The event args.
     */
    populateContextMenu(contextMenu: ContextMenu, args: PopulateItemContextMenuEventArgs<IModelItem>): void {
        // The 'showMenu' property is set to true to inform the input mode that we actually want to show a context menu
        // for this item (or more generally, the location provided by the event args).
        // If you don't want to show a context menu for some locations, set 'false' in this cases.
        args.showMenu = true

        const graphComponent = this.graphComponent
        contextMenu.clearItems()

        const node = args.item instanceof INode ? args.item : null
        // If the cursor is over a node select it
        this.updateSelection(node)

        // Create the context menu items
        if (graphComponent.selection.selectedNodes.size > 0) {
            // a node has been hit
            const data = node.tag as SelectElement

            // replace placeholders ('-1')
            const obj_url = this.object_url.replace('-1', data.id.toString())
            const conn_url = this.connection_url.replace('-1', data.id.toString())

            contextMenu.addMenuItem(this.language['LC__CMDB__CATG__SMVIEW_PATCH__CTX_OPEN_OBJECT'], () => window.open(obj_url, '_blank').focus())
            contextMenu.addMenuItem(this.language['LC__CMDB__CATG__SMVIEW_PATCH__CTX_OPEN_CONNECTIONS'], () => window.open(conn_url, '_blank').focus())
        } else {
            // no node has been hit
        }
    }

    /**
     * Helper function that updates the node selection state when the context menu is opened on a node.
     * @param node The node or <code>null</code>.
     */
    updateSelection(node: INode | null): void {
        const graphComponent = this.graphComponent
        if (node === null) {
            // clear the whole selection
            graphComponent.selection.clear()
        } else if (!graphComponent.selection.selectedNodes.isSelected(node)) {
            // no - clear the remaining selection
            graphComponent.selection.clear()
            // and select the node
            graphComponent.selection.selectedNodes.setSelected(node, true)
            // also update the current item
            graphComponent.currentItem = node
        }
    }

    /**
     * Adds a new separator to this menu.
     */
    addSeparator(): void {
        const separator = document.createElement('div')
        separator.setAttribute('class', 'ctx-separator')
        this.element.appendChild(separator)
    }

    /**
     * Adds a new menu entry with the given text and click-listener to this menu.
     */
    addMenuItem(label: string, clickListener: ((e: MouseEvent) => void) | null): HTMLElement {
        const menuItem = document.createElement('button')
        menuItem.setAttribute('class', 'ctx-menu-item')
        menuItem.innerHTML = label
        if (clickListener !== null) {
            menuItem.addEventListener('click', clickListener, false)
        }
        this.element.appendChild(menuItem)
        return menuItem
    }

    /**
     * Removes all menu entries and separators from this menu.
     */
    clearItems(): void {
        const element = this.element
        while (element.firstChild) {
            element.removeChild(element.firstChild)
        }
    }

    /**
     * Shows this menu at the given location.
     *
     * This menu only shows if it has at least one menu item.
     *
     * @param {Point} location The location of the menu relative to the left edge of the entire
     *   document. This are typically the pageX and pageY coordinates of the contextmenu event.
     */
    show(location: Point): void {
        if (this.element.childElementCount <= 0) {
            return
        }
        this.element.addEventListener('focusout', this.focusOutListener)
        this.element.addEventListener('focusin', this.focusInListener)
        this.element.addEventListener('click', this.closeListener, false)
        document.addEventListener('keydown', this.closeOnEscListener, false)

        // Set the location of this menu and append it to the body
        const style = this.element.style
        style.setProperty('position', 'absolute', '')
        style.setProperty('left', `${location.x}px`, '')
        style.setProperty('top', `${location.y}px`, '')
        if (document.fullscreenElement && !document.fullscreenElement.contains(document.body)) {
            document.fullscreenElement.appendChild(this.element)
        } else {
            document.body.appendChild(this.element)
        }

        // trigger enter animation
        setTimeout(() => {
            this.element.setAttribute('class', `${this.element.getAttribute('class')} visible`)
        }, 0)
        ;(this.element.firstElementChild! as HTMLElement).focus()
        this.isOpen = true
    }

    /**
     * Closes this menu.
     */
    close(): void {
        this.element.removeEventListener('focusout', this.focusOutListener)
        this.element.removeEventListener('focusin', this.focusInListener)
        this.element.removeEventListener('click', this.closeListener, false)
        document.removeEventListener('keydown', this.closeOnEscListener, false)

        const parentNode = this.element.parentNode
        if (parentNode) {
            // trigger fade-out animation on a clone
            const contextMenuClone = this.element.cloneNode(true) as HTMLElement
            contextMenuClone.setAttribute(
                'class',
                `${contextMenuClone.getAttribute('class')} analyzer-ctx-menu-clone`
            )
            parentNode.appendChild(contextMenuClone)
            // fade the clone out, then remove it from the DOM. Both actions need to be timed.
            setTimeout(() => {
                contextMenuClone.setAttribute(
                    'class',
                    contextMenuClone.getAttribute('class')!.replace(/\s?visible/, '')
                )

                setTimeout(() => {
                    parentNode.removeChild(contextMenuClone)
                }, 300)
            }, 0)

            this.element.setAttribute(
                'class',
                this.element.getAttribute('class')!.replace(/\s?visible/, '')
            )
            parentNode.removeChild(this.element)
        }

        this.isOpen = false
    }

    /**
     * Sets a callback function that is invoked if the context menu closed itself, for example because a
     * menu item was clicked.
     *
     * Typically, the provided callback informs the <code>ContextMenuInputMode</code> that this menu is
     * closed.
     */
    get onClosedCallback() {
        if (this.onClosedCallbackField == null) {
            alert('For this context menu, the onClosedCallback property must be set.')
        }
        return this.onClosedCallbackField
    }

    set onClosedCallback(callback: () => void) {
        this.onClosedCallbackField = callback
    }

    /**
     * Adds event listeners for events that should show the context menu. These listeners then call the provided
     * openingCallback function.
     *
     * Besides the obvious <code>contextmenu</code> event, we listen for long presses and the Context Menu key.
     *
     * A long touch press doesn't trigger a <code>contextmenu</code> event on all platforms therefore we listen to the
     * GraphComponent's TouchLongPress event
     *
     * The Context Menu key is not handled correctly in Chrome. In other browsers, when the Context Menu key is
     * pressed, the correct <code>contextmenu</code> event is fired but the event location is not meaningful.
     * In this case, we set a better location, centered on the given element.
     *
     * @param {GraphComponent} graphComponent The graph component of this context menu.
     * @param {function(Point)} openingCallback This function is called when an event that should
     *   open the context menu occurred. It gets the location of the event.
     */
    addOpeningEventListeners(graphComponent: GraphComponent, openingCallback: (p: Point) => void): void {
        const componentDiv = graphComponent.div
        const contextMenuListener = (evt: MouseEvent & { mozInputSource?: number }): void => {
            evt.preventDefault()
            if (this.isOpen) {
                // might be open already because of the longpress listener
                return
            }
            const me = evt
            if (evt.mozInputSource === 1 && me.button === 0) {
                // This event was triggered by the context menu key in Firefox.
                // Thus, the coordinates of the event point to the lower left corner of the element and should be corrected.
                openingCallback(ContextMenu.getCenterInPage(componentDiv))
            } else if (me.pageX === 0 && me.pageY === 0) {
                // Most likely, this event was triggered by the context menu key in IE.
                // Thus, the coordinates are meaningless and should be corrected.
                openingCallback(ContextMenu.getCenterInPage(componentDiv))
            } else {
                openingCallback(new Point(me.pageX, me.pageY))
            }
        }

        // Listen for the contextmenu event
        // Note: On Linux based systems (e.g. Ubuntu), the contextmenu event is fired on mouse down
        // which triggers the ContextMenuInputMode before the ClickInputMode. Therefore handling the
        // event, will prevent the ItemRightClicked event from firing.
        // For more information, see https://docs.yworks.com/yfileshtml/#/kb/article/780/
        componentDiv.addEventListener('contextmenu', contextMenuListener, false)

        // Listen to the context menu key to make it work in Chrome
        componentDiv.addEventListener('keyup', evt => {
            if (evt.keyCode === 93) {
                evt.preventDefault()
                openingCallback(ContextMenu.getCenterInPage(componentDiv))
            }
        })
    }

    /**
     * Closes the context menu when it lost the focus.
     *
     * @param {HTMLElement} relatedTarget The related target of the focus event.
     *
     * @private
     */
    onFocusOut(relatedTarget: HTMLElement): void {
        // focusout can also occur when the focus shifts between the buttons in this context menu.
        // We have to find out if none of the buttons has the focus and focusout is real
        if (relatedTarget) {
            if (relatedTarget.parentElement && relatedTarget.parentElement !== this.element) {
                this.close()
            }
        } else if (!this.blurredTimeout) {
            // If the browser doesn't provide a related target, we wait a little bit to see whether the focus is given to
            // another button in this context menu
            this.element.addEventListener('focusin', this.focusInListener)
            this.blurredTimeout = setTimeout(() => {
                this.close()
            }, 350) as any as number
        }
    }

    /**
     * Calculates the location of the center of the given element in absolute coordinates relative to the body element.
     *
     * @param {HTMLElement} element
     * @return {Point}
     *
     * @private
     */
    static getCenterInPage(element: HTMLElement): Point {
        let left = element.clientWidth / 2.0
        let top = element.clientHeight / 2.0
        while (element.offsetParent) {
            left += element.offsetLeft
            top += element.offsetTop
            element = element.offsetParent as HTMLElement
        }
        return new Point(left, top)
    }
}
