import {
    DefaultLabelStyle,
    ExteriorLabelModel,
    ExteriorLabelModelPosition,
    GraphComponent,
    GraphItemTypes,
    IArrow,
    IEdge,
    NinePositionsEdgeLabelModel,
    PolylineEdgeStyle,
    ShapeNodeStyle,
    Size,
} from "yfiles/view-component";
import {NodeStyleDecorationInstaller} from "yfiles";
import {GraphEditorInputMode} from "yfiles/view-editor";
import {HighlightClickInputMode} from "./HighlightClickInputMode";
import {EdgeTooltip} from "./Tooltip";
import {DisplaySettings, EDGE_COLORING_CABLE_COLOR, EDGE_COLORING_CAPACITY} from "./Settings";

export class Defaults {

    static setDefaults(graphComponent: GraphComponent, display_settings: DisplaySettings) {
        graphComponent.inputMode = new GraphEditorInputMode({
            clickableItems: GraphItemTypes.NODE,
            clickSelectableItems: GraphItemTypes.NODE,
            clickInputMode: new HighlightClickInputMode(graphComponent, display_settings),
            selectableItems: GraphItemTypes.NODE,
            marqueeSelectableItems: GraphItemTypes.NODE,
            movableItems: GraphItemTypes.NODE,
            deletableItems: GraphItemTypes.NONE,
            labelEditableItems: GraphItemTypes.NONE,
            allowClipboardOperations: false,
            allowGroupingOperations: false,
            allowDuplicate: false,
            allowEditLabel: false,
            allowCreateNode: false,
            allowCreateEdge: false,
            allowCreateBend: false,
        });

        let graph = graphComponent.graph;

        const labelModel = new ExteriorLabelModel({ insets: 5 });
        graph.nodeDefaults.labels.layoutParameter = labelModel.createParameter(ExteriorLabelModelPosition.SOUTH);
        graph.nodeDefaults.labels.style = Defaults.getDefaultNodeLabelStyle();
        graph.nodeDefaults.size = new Size(60, 40);
        graph.nodeDefaults.style = Defaults.getDefaultNodeStyle();

        graph.edgeDefaults.labels.layoutParameter = NinePositionsEdgeLabelModel.CENTER_BELOW;
        graph.edgeDefaults.labels.style = Defaults.getDefaultEdgeLabelStyle();
        graph.edgeDefaults.style = Defaults.getDefaultEdgeStyle();

        const selectionInstaller = new NodeStyleDecorationInstaller({
            nodeStyle: Defaults.getSelectedNodeStyle()
        });

        const focusIndicatorInstaller = new NodeStyleDecorationInstaller({
            nodeStyle: Defaults.getFocusedNodeStyle()
        });

        const decorator = graph.decorator;
        decorator.nodeDecorator.selectionDecorator.setImplementation(selectionInstaller);
        decorator.nodeDecorator.focusIndicatorDecorator.setImplementation(focusIndicatorInstaller);
    }

    static getDefaultNodeStyle(color?: string) {
        let is_hex = color ? /(^[0-9A-F]{6}$)|(^[0-9A-F]{3}$)/i.test(color) : false;
        let fill = is_hex ? '#' + color : '#00AA21';
        let stroke = is_hex ? '#444444' : '#00600E';
        return new ShapeNodeStyle({
            fill: fill,
            stroke: stroke,
            shape: 'round-rectangle'
        });
    }

    static getSelectedNodeStyle() {
        return new ShapeNodeStyle({
            shape: 'rectangle',
            stroke: '2px #6BC9E5',
            fill: null
        });
    }

    static getFocusedNodeStyle() {
        return new ShapeNodeStyle({
            shape: 'rectangle',
            stroke: '0px dashed #6BC9E5',
            fill: null
        });
    }

    static getDefaultNodeLabelStyle() {
        return new DefaultLabelStyle({
            backgroundStroke: '#AAAAAA',
            backgroundFill: '#EEEEEE',
            autoFlip: false,
            insets: [3, 4, 3, 4]
        });
    }

    static getRackNodeLabelStyle() {
        return new DefaultLabelStyle({
            backgroundStroke: 'transparent',
            backgroundFill: 'transparent',
            autoFlip: false,
            insets: [3, 4, 3, 4]
        });
    }

    static getDefaultEdgeStyle(edge?: IEdge) {
        let color = '#444444'
        if (edge) {
            const data = edge.tag as EdgeTooltip
            if (data.edge_coloring == EDGE_COLORING_CABLE_COLOR) {
                color = data.cable_color != '' ? data.cable_color : '#444444'
            }
            if (data.edge_coloring == EDGE_COLORING_CAPACITY) {
                color = (data.count - data.patched) > 0 ? '#00FF00' : '#FF0000'
            }
        }
        return new PolylineEdgeStyle({
            sourceArrow: IArrow.NONE,
            targetArrow: IArrow.NONE,
            stroke: '1px solid ' + color
        });
    }

    static getFocusedEdgeStyle() {
        return new PolylineEdgeStyle({
            sourceArrow: IArrow.NONE,
            targetArrow: IArrow.NONE,
            stroke: '3px dashed #6BC9E5'
        });
    }

    static getDefaultEdgeLabelStyle() {
        return new DefaultLabelStyle({
            backgroundStroke: '#6BC9E5',
            backgroundFill: '#D0E9ED',
            autoFlip: false,
            insets: [3, 4, 3, 4]
        });
    }
}
