import {IEdge, IGraph, INode, INodeStyle, Rect} from "yfiles/view-component";
import {SelectElement} from "./Select";
import {CustomImageNodeStyle} from "./ImageNode";
import {Defaults} from "./Defaults";
import {DisplaySettings} from "./Settings";

export class GraphRenderer {

    private readonly graph: IGraph;
    private readonly image_url: string;
    private readonly show_node_images: boolean;

    constructor(graph: IGraph, image_url: string, show_node_images: boolean) {
        this.graph = graph;
        this.image_url = image_url;
        this.show_node_images = show_node_images;
        this.render = this.render.bind(this);
        this.renderNodes = this.renderNodes.bind(this);
    }

    render(nodes: SelectElement[], level: string, display_settings: DisplaySettings) {
        // clear graph
        this.graph.clear();

        let current_nodes = this.renderNodes(nodes, level);
        GraphRenderer.renderEdges(this.graph, nodes, current_nodes, display_settings);
    }

    renderNodes(results: SelectElement[], level: string) {
        let current_nodes:{[key: string]: INode} = {};

        // add nodes
        let y = 10;
        for (let k in results) {
            if (! results.hasOwnProperty(k) ) {
                continue;
            }

            let item: SelectElement = results[k];
            let nodeStyle: INodeStyle;
            if (this.show_node_images) {
                nodeStyle = new CustomImageNodeStyle(this.image_url, level);
            } else {
                nodeStyle = Defaults.getDefaultNodeStyle(item.type_color);
            }
            let node = this.graph.createNode(new Rect(10, y, 20, 20), nodeStyle);
            node.tag = item;
            this.graph.addLabel(node, item.title);
            current_nodes[item.id] = node;
            y += 25;
        }
        return current_nodes;
    }

    static renderEdges(graph: IGraph, results: SelectElement[], current_nodes: {[key: string]: INode}, display_settings: DisplaySettings) {
        let connected_nodes:{[key: string]: INode} = {};
        let current_edges:{[key: string]: IEdge} = {};
        for (let itemId in results) {
            if (! results.hasOwnProperty(itemId) ) {
                continue;
            }

            let item = results[itemId];
            for (let sibItemId in item['connections']) {
                if (! item['connections'].hasOwnProperty(sibItemId) ) {
                    continue;
                }

                let sibling = item['connections'][sibItemId];
                let sib_node_id = sibling.sibling;

                let first = item.id > sib_node_id ? sib_node_id : item.id;
                let second = item.id > sib_node_id ? item.id : sib_node_id;
                let key = first + '-' + second;

                if (current_edges[key]) {
                    continue;
                }

                let node = current_nodes[item.id];
                let sib_node = current_nodes[sib_node_id];
                connected_nodes[item.id] = node;
                connected_nodes[sib_node_id] = sib_node;

                if (! node || ! sib_node) {
                    continue;
                }

                let edge = graph.createEdge(node, sib_node);
                edge.tag = {
                    key: key,
                    patched: sibling.used,
                    count: sibling.count,
                    cable_color: sibling.cable_color,
                    edge_coloring: display_settings.edge_coloring,
                }
                current_edges[key] = edge;

                const style = Defaults.getDefaultEdgeStyle(edge)
                graph.setStyle(edge, style)

                if (display_settings.show_capacity) {
                    graph.addLabel(edge, sibling.used +'/' + sibling.count);
                }
            }
        }

        if (display_settings.show_only_connected_nodes) {
            GraphRenderer.removeUnconnectedNodes(graph, current_nodes, connected_nodes);
        }
    }

    static removeUnconnectedNodes(graph: IGraph, nodeMap: {[key: string]: INode}, connected_nodes: {[key: string]: INode}) {
        let removable_nodes:INode[] = [];
        for (let n in nodeMap) {
            if (nodeMap.hasOwnProperty(n) && ! connected_nodes[n]) {
                let group_node = nodeMap[n];
                if (graph.isGroupNode(group_node)) {
                    graph.nodes.forEach((node: INode) => {
                        if (graph.getParent(node) == group_node) {
                            removable_nodes.push(node);
                        }
                    });
                }
                removable_nodes.push(group_node);
            }
        }

        for (let n in removable_nodes) {
            let node = removable_nodes[n];
            graph.remove(node);
        }
    }
}
