import {
    ClickEventArgs,
    ClickInputMode,
    IModelItem,
    IEdge,
    INode,
    Neighborhood,
    ShortestPath
} from 'yfiles'
import {GraphComponent} from "yfiles/view-component";
import {Defaults} from "./Defaults";
import {DisplaySettings} from "./Settings";
import {EdgeTooltip} from "./Tooltip";

export class HighlightClickInputMode extends ClickInputMode {

    public display_settings: DisplaySettings
    private readonly graphComponent: GraphComponent

    constructor(graphComponent: GraphComponent, display_settings: DisplaySettings) {
        super();
        this.display_settings = display_settings
        this.graphComponent = graphComponent;
    }

    onLeftClicked(clickEventArgs: ClickEventArgs) {
        super.onLeftClicked(clickEventArgs);

        // reset edge styles
        const graph = this.inputModeContext.graph;
        graph.edges.forEach(edge => {
            graph.setStyle(edge, Defaults.getDefaultEdgeStyle(edge));
        });

        let nodes = this.graphComponent.selection.selectedNodes;
        if (nodes.size == 2) {
            let n = []
            nodes.forEach((node: INode) => {
                n.push(node)
            })
            this.markPath(n)
        } else if (this.display_settings.neighbor_edges > 0) {
            // select neighbor edges
            nodes.forEach((node: INode) => {
                this.markNeighborEdges(node);
            });
        }
    }

    markNeighborEdges(item: IModelItem) {
        if (! (item instanceof INode)) {
            return;
        }

        const graph = this.inputModeContext.graph;
        let node: INode = item as INode;
        const neighborResult = new Neighborhood({
            maximumDistance: this.display_settings.neighbor_edges,
            startNodes: [node]
        }).run(graph);

        neighborResult.neighbors.forEach(target_node => {
            let path = new ShortestPath({
                source: node,
                sink: target_node,
                directed: false
            }).run(graph);

            let style = Defaults.getFocusedEdgeStyle()
            path.edges.forEach(edge => {
                graph.setStyle(edge, style);
            });
        });
    }

    markPath(nodes: INode[]) {
        // configure and run the single-source single-sink algorithm on the graph
        const graph = this.inputModeContext.graph;
        const shortestPathResult = new ShortestPath({
            source: nodes[0],
            sink: nodes[1],
            directed: false,
            costs: edge => this.getEdgeCost(edge)
        }).run(graph)

        // because costs can not be infinite, we have to check if all edges have really a valid available connection
        let hasAvailableConnections = true
        shortestPathResult.edges.forEach(edge => {
            if (this.getEdgeCost(edge) > 1) {
                hasAvailableConnections = false
            }
        });

        if (hasAvailableConnections) {
            // only mark path if each edge has at least minimum count of available connections
            const pathEdgeStyle = Defaults.getFocusedEdgeStyle()
            shortestPathResult.edges.forEach(edge => {
                graph.setStyle(edge, pathEdgeStyle)
            })
        }
    }

    getEdgeCost(edge: IEdge) {
        if (this.display_settings.routing_shortest_path) {
            return 1
        }

        const data = edge.tag as EdgeTooltip
        const unPatchedConnections = data.count - data.patched
        const hasMinAvailableConnections = unPatchedConnections >= this.display_settings.routing_connection_count
        return hasMinAvailableConnections ? 1 : 9999
    }
}
