import { GraphComponent, IGraph, Insets, Rect, Size, SvgExport } from 'yfiles'
import * as jsPDF from "jspdf"
import 'svg2pdf.js'

export enum PaperSize {
    A3 = 'A3',
    A4 = 'A4',
    A5 = 'A5',
    A6 = 'A6',
    LETTER = 'Letter',
    AUTO = 'Auto'
}

/**
 * Provides PDF export in the client's browser.
 *
 * yFiles' SvgExport is used to export the contents of a GraphComponent into an SVG document
 * which is subsequently converted into a PDF document by jsPDF.
 *
 * See also: https://github.com/yWorks/yfiles-for-html-demos/tree/master/demos/view/pdfexport
 */
export default class PdfExport {

    scale = 1
    margins = new Insets(5)
    paperSize = PaperSize.AUTO

    setPaperSize(paperSize: PaperSize) {
        this.paperSize = paperSize
    }

    async exportPdf(graph: IGraph, exportRect: Rect | null): Promise<{ raw: string; uri: string }> {
        // Create a new graph component for exporting the original SVG content
        const exportComponent = new GraphComponent()
        // ... and assign it the same graph.
        exportComponent.graph = graph
        exportComponent.updateContentRect()

        // Determine the bounds of the exported area
        const targetRect = exportRect || exportComponent.contentRect

        exportComponent.zoomTo(targetRect)

        // Create the exporter class
        const exporter = new SvgExport(targetRect, this.scale)
        exporter.margins = this.margins

        if (window.btoa != null) {
            // Don't use base 64 encoding if btoa is not available and don't inline images as-well.
            exporter.encodeImagesBase64 = true
            exporter.inlineSvgImages = true
        }

        // export the component to svg
        const svgElement = await exporter.exportSvgAsync(exportComponent)

        const size = getExportSize(this.paperSize, exporter)
        return convertSvgToPdf(svgElement as SVGElement, size)
    }
}

function convertSvgToPdf(svgElement: SVGElement, size: Size): Promise<{ raw: string; uri: string }> {
    svgElement = svgElement.cloneNode(true) as SVGElement

    const sizeArray = [size.width, size.height]
    // eslint-disable-next-line no-undef,new-cap
    const jsPdf = new jsPDF.jsPDF({
        orientation: sizeArray[0] > sizeArray[1] ? 'l' : 'p',
        unit: 'pt',
        format: sizeArray,
        compress: true,
    })

    const options = {
        width:  sizeArray[0],
        height: sizeArray[1]
    }

    return jsPdf
        .svg(svgElement, options)
        .then(() => ({ raw: jsPdf.output(), uri: jsPdf.output('datauristring') }))
}

function getExportSize(paperSize: PaperSize, exporter: SvgExport): Size {
    switch (paperSize) {
        case PaperSize.A3:
            return new Size(842, 1191)
        case PaperSize.A4:
            return new Size(595, 842)
        case PaperSize.A5:
            return new Size(420, 595)
        case PaperSize.A6:
            return new Size(298, 420)
        case PaperSize.LETTER:
            return new Size(612, 792)
        case PaperSize.AUTO:
            return new Size(exporter.viewWidth, exporter.viewHeight)
    }
}