import {
    ExteriorLabelModel, ExteriorLabelModelPosition, InteriorLabelModel, InteriorLabelModelPosition, IGraph,
    Rect, ShapeNodeStyle, INode
} from "yfiles/view-component";
import {ImageNodeStyle} from "yfiles/styles-other";
import {Connection, SelectElement} from "./Select";
import {GraphRenderer} from "./GraphRenderer";
import {Defaults} from "./Defaults";
import {DisplaySettings} from "./Settings";


export interface RackProps {
    id: number,
    title: string,
    units: number,
    has_half_he: boolean,
    sorting: string,
    devices_front: RackDeviceProps[],
    devices_back: RackDeviceProps[],
    vertical_slots_front: number,
    vertical_slots_back: number,
    angle: string,
    selected_device_id?: number,
    connections: Connection[],
}

interface RackDeviceProps {
    id: number,
    title: string,
    type_id: number,
    type_title: string,
    position: number,
    he: number,
    has_half_he: boolean,
    image_url: string,
    is_chassis: boolean,
    angle: string,
    chassis_width?: number,
    chassis_height?: number,
    slots?: SlotProps[],
    selected_port_id?: number,
    comment?: string,
}

interface SlotProps {
    id: number,
    title: string,
    x_from: number,
    x_to: number,
    y_from: number,
    y_to: number,
    devices: SlotDeviceProps[],
}

interface SlotDeviceProps {
    id: number,
    title: string,
    image_url: string,
}

export class RackRenderer {

    DEFAULT_WIDTH = 240;
    DEFAULT_HEIGHT = 24;

    private readonly graph: IGraph
    private readonly displaySettings: DisplaySettings

    constructor(graph: IGraph, displaySettings: DisplaySettings) {
        this.graph = graph;
        this.displaySettings = displaySettings
        this.render = this.render.bind(this);
        this.renderRack = this.renderRack.bind(this);
    }

    render(racks: RackProps[]) {
        // add nodes
        let x = 1;
        let y = 1;
        let nodeMap:{[key: string]: INode} = {};
        let results: SelectElement[] = [];
        for (let r in racks) {
            if (! racks.hasOwnProperty(r) ) {
                continue;
            }

            let rackData = racks[r];
            nodeMap[rackData.id] = this.renderRack(rackData, x, y);
            x += this.DEFAULT_WIDTH + 20;

            // convert type
            results.push({
                id: rackData.id,
                title: rackData.title,
                type_id: 0,
                type_const: '',
                type_title: '',
                type_color: '',
                connections: rackData.connections,
            });

            if (x > 1600) {
                x = 1;
                y = 500; // todo: get largest rack by its units
            }
        }

        GraphRenderer.renderEdges(this.graph, results, nodeMap, this.displaySettings);
    }

    renderRack(rack: RackProps, pos_x: number = 0, pos_y: number = 0) {
        let graph = this.graph;
        graph.groupNodeDefaults.style = new ShapeNodeStyle({
            fill: '#EEEEEE',
            stroke: '#D3D3D3',
            shape: 'rectangle'
        });

        // render group node
        const interiorLabelModel = new InteriorLabelModel({ insets: 4 });
        const exteriorLabelModel = new ExteriorLabelModel({ insets: 4 });
        const folder = graph.createGroupNode(null, new Rect(pos_x, pos_y, this.DEFAULT_WIDTH, this.DEFAULT_HEIGHT));
        graph.addLabel(folder, rack.title, interiorLabelModel.createParameter(InteriorLabelModelPosition.NORTH));

        let y = pos_y + 28;
        for (let i:number = 1; i <= rack.units; i++) {
            let position = rack.sorting == 'asc' ? i : rack.units - i + 1;
            position = rack.has_half_he ? position / 2 : position;
            let he = 1;

            // todo: handle angle
            //let devices = rack.angle == 'F' ? rack.devices_front : rack.devices_back;
            let devices = rack.devices_front;
            let filtered_devices = devices.filter(d => d.position == i);
            let device:RackDeviceProps = null;
            if (filtered_devices.length > 0) {
                let dev = filtered_devices[0];
                device = dev;

                he = dev.he;
                if (dev.he == 0 || dev.he == null) {
                    // fix invalid HE
                    he = 1;
                }
                // cast value to be integer
                he = parseInt(he.toString());
                if (rack.has_half_he && !dev.has_half_he) {
                    // duplicate HE
                    he = he * 2;
                }

                if (! rack.has_half_he && dev.has_half_he) {
                    // halve HE
                    he = he % 2 == 0 ? (he / 2) : ((he + 1) / 2);
                }

                if (he == 0) {
                    // set to 1 if it is 0
                    he = 1;
                }
            }

            // add node
            let height = this.DEFAULT_HEIGHT * he;
            let x = pos_x + 32;
            let node = graph.createNode(folder, new Rect(x, y, this.DEFAULT_WIDTH, height));
            let nodeStyle;
            if (device) {
                if (device.image_url) {
                    nodeStyle = new ImageNodeStyle(device.image_url);
                } else {
                    nodeStyle = new ShapeNodeStyle({
                        fill: '#BFDEBC',
                        stroke: '#AAAAAA'
                    });

                    // add device name if no image exists
                    graph.addLabel(node, device.title, interiorLabelModel.createParameter(InteriorLabelModelPosition.CENTER));
                }
                node.tag = device;
            } else {
                nodeStyle = new ShapeNodeStyle({
                    fill: '#DDDDDD',
                    stroke: '#AAAAAA',
                    shape: 'rectangle'
                });
            }
            graph.setStyle(node, nodeStyle);

            // add position label
            let position_title = position.toString();
            if (he > 1) {
                i += (he - 1);
                position_title += '-' + i.toString();
            }
            let layoutParameter = exteriorLabelModel.createParameter(ExteriorLabelModelPosition.WEST);
            graph.addLabel(node, position_title, layoutParameter, Defaults.getRackNodeLabelStyle());

            y += height;
        }

        graph.setNodeLayout(folder, new Rect(pos_x, pos_y, this.DEFAULT_WIDTH + 40, (y + 4) - pos_y));
        return folder;
    }
}
