import * as React from "react";
import {SimpleSelect} from "./SimpleSelect";
import {Layout} from "../AnalyzerCompose";

export const EDGE_COLORING_DEFAULT = 'default'
export const EDGE_COLORING_CAPACITY = 'capacity'
export const EDGE_COLORING_CABLE_COLOR = 'cable_color'

export interface DisplaySettings {
    show_rack_view: boolean,
    show_capacity: boolean,
    show_only_connected_nodes: boolean,
    show_node_images: boolean,
    neighbor_edges: number,
    connector_types: number[],
    current_layout: number,
    edge_coloring: string,
    routing_shortest_path: boolean,
    routing_connection_count: number,
}

export interface SettingsProps {
    display_settings: DisplaySettings,
    connection_types?: {[key: string]: string},
    connector_types: {[key: string]: string},
    layouts: Layout[],
    language: {[key: string]: string},
    is_loading: boolean,
    onDisplaySettingChange: Function,
    onHandleLayoutChange: Function,
    onHandlePdfExport: Function,
}

export class Settings extends React.Component<SettingsProps, {}> {
    constructor(props: SettingsProps) {
        super(props);
        this.handleShowRackView = this.handleShowRackView.bind(this);
        this.handleShowCapacity = this.handleShowCapacity.bind(this);
        this.handleShowOnlyConnectedNodes = this.handleShowOnlyConnectedNodes.bind(this);
        this.handleShowNodeImage = this.handleShowNodeImage.bind(this);
        this.handleSetNeighborEdges = this.handleSetNeighborEdges.bind(this)
        this.handleFilterConnectionType = this.handleFilterConnectionType.bind(this);
        this.addFilterConnectorType = this.addFilterConnectorType.bind(this);
        this.removeFilterConnectorType = this.removeFilterConnectorType.bind(this)
        this.handleEdgeColoring = this.handleEdgeColoring.bind(this)
        this.handleRouting = this.handleRouting.bind(this)
        this.handleSetRoutingConnectionCount = this.handleSetRoutingConnectionCount.bind(this)
    }

    handleShowRackView(event: React.ChangeEvent<HTMLInputElement>) {
        this.props.onDisplaySettingChange('show_rack_view', event.currentTarget.checked);
    }

    handleShowCapacity(event: React.ChangeEvent<HTMLInputElement>) {
        this.props.onDisplaySettingChange('show_capacity', event.currentTarget.checked);
    }

    handleShowOnlyConnectedNodes(event: React.ChangeEvent<HTMLInputElement>) {
        this.props.onDisplaySettingChange('show_only_connected_nodes', event.currentTarget.checked);
    }

    handleShowNodeImage(event: React.ChangeEvent<HTMLInputElement>) {
        let show_images = event.currentTarget.value == 'images'
        this.props.onDisplaySettingChange('show_node_images', show_images);
    }

    handleSetNeighborEdges(event: React.ChangeEvent<HTMLInputElement>) {
        let value = parseInt(event.target.value);
        value = value < 0 ? 0 : value;
        value = value > 10 ? 10 : value;
        this.props.onDisplaySettingChange('neighbor_edges', value)
    }

    handleFilterConnectionType(type: number) {
        this.props.onDisplaySettingChange('connection_type', type)
    }

    addFilterConnectorType(type_selection: string) {
        const type = parseInt(type_selection)
        let types = this.props.display_settings.connector_types
        const index = types.indexOf(type);
        if (index == -1) {
            types.push(type)
        }
        this.props.onDisplaySettingChange('connector_types', types)
    }

    removeFilterConnectorType(event: React.MouseEvent<HTMLSpanElement>) {
        if (this.props.is_loading) {
            return;
        }

        const key = event.currentTarget.id != '' ? event.currentTarget.id : event.currentTarget.parentElement.id;
        const type = parseInt(key.substr(10))
        let types = this.props.display_settings.connector_types
        const index = types.indexOf(type);
        if (index > -1) {
            types.splice(index, 1);
        }
        this.props.onDisplaySettingChange('connector_types', types)
    }

    handleEdgeColoring(event: React.ChangeEvent<HTMLInputElement>) {
        this.props.onDisplaySettingChange('edge_coloring', event.currentTarget.value)
    }

    handleRouting(event: React.ChangeEvent<HTMLInputElement>) {
        let use_shortest_path = event.currentTarget.value == 'shortest_path'
        this.props.onDisplaySettingChange('routing_shortest_path', use_shortest_path);
    }

    handleSetRoutingConnectionCount(event: React.ChangeEvent<HTMLInputElement>) {
        let value = parseInt(event.target.value);
        value = value < 0 ? 0 : value;
        value = value > 100 ? 100 : value;
        this.props.onDisplaySettingChange('routing_connection_count', value);
    }

    render() {
        let layout_opts: {[key: number]: string} = []
        this.props.layouts.forEach(l => {
            layout_opts[l.id] = l.title
        })

        let selected_connector_types: JSX.Element[] = []
        for (let key in this.props.connector_types) {
            if (! this.props.connector_types.hasOwnProperty(key)) {
                continue;
            }

            const connector_id = parseInt(key)
            if (this.props.display_settings.connector_types.indexOf(connector_id) < 0) {
                continue;
            }

            const opt = this.props.connector_types[connector_id]
            selected_connector_types.push(
                <div id={'connector_' + connector_id}
                     key={connector_id}
                     className="filter-item"
                     style={{cursor: this.props.is_loading ? 'not-allowed' :  'pointer'}}
                     onClick={this.removeFilterConnectorType}>{opt} <span>X</span></div>
            )
        }

        return <div className="settings_bottom">

            <div>
                {this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__DISPLAY_DETAILS']}
                <ul>
                    <li>
                        <input type="checkbox"
                               checked={this.props.display_settings.show_only_connected_nodes}
                               onChange={this.handleShowOnlyConnectedNodes}
                               disabled={this.props.is_loading} />
                        <label>{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__SHOW_ONLY_CONNECTED_NODES']}</label>
                    </li>
                    <li>
                        <label>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__LAYOUT}:</label>
                        <SimpleSelect opts={layout_opts}
                                      onChange={this.props.onHandleLayoutChange}
                                      disabled={this.props.is_loading} />
                    </li>
                    <li>
                        <label>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__FILTER_CONNECTOR_TYPE}:</label>
                        <SimpleSelect opts={this.props.connector_types}
                                      has_empty_option={true}
                                      onChange={this.addFilterConnectorType}
                                      disabled={this.props.is_loading} />
                        <div className="filter-item-list">
                            {selected_connector_types}
                        </div>
                    </li>
                    <li>
                        <button type="button"
                                className="btn"
                                onClick={() => this.props.onHandlePdfExport()}
                                disabled={this.props.is_loading}>
                            {this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__EXPORT_GRAPH}
                        </button>
                    </li>
                </ul>
            </div>

            <div>
                {this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__NODE_REPRESENTATION']}
                <ul>
                    <li>
                        <input type="radio"
                               checked={!this.props.display_settings.show_node_images}
                               onChange={this.handleShowNodeImage}
                               disabled={this.props.is_loading}
                               name="show_node_image[]" value="symbols" />
                        <label>{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__NODE_SYMBOLS']}</label>
                    </li>
                    <li>
                        <input type="radio"
                               checked={this.props.display_settings.show_node_images}
                               onChange={this.handleShowNodeImage}
                               disabled={this.props.is_loading}
                               name="show_node_image[]" value="images" />
                        <label>{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__NODE_IMAGES']}</label>
                    </li>
                    <li>
                        <input type="checkbox"
                               checked={this.props.display_settings.show_rack_view}
                               onChange={this.handleShowRackView}
                               disabled={this.props.is_loading} />
                        <label>{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__SHOW_RACK_VIEW']}</label>
                    </li>
                </ul>
            </div>

            <div>
                {this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__EDGE_REPRESENTATION']}
                <ul>
                    <li>
                        <input type="checkbox"
                               checked={this.props.display_settings.show_capacity}
                               onChange={this.handleShowCapacity}
                               disabled={this.props.is_loading} />
                        <label>{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__SHOW_CAPACITY']}</label>
                    </li>
                    <li>
                        <label>{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__MARK_NEIGHBOR_EDGES']}:</label>
                        <input type="text"
                               className="input input-mini"
                               style={{width: '24px'}}
                               value={this.props.display_settings.neighbor_edges}
                               onChange={this.handleSetNeighborEdges}
                               disabled={this.props.is_loading} />
                    </li>
                </ul>
            </div>

            <div>
                {this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__EDGE_COLORING']}
                <ul>
                    <li>
                        <input type="radio"
                               checked={this.props.display_settings.edge_coloring == EDGE_COLORING_DEFAULT}
                               onChange={this.handleEdgeColoring}
                               disabled={this.props.is_loading}
                               name="edge_coloring[]" value={EDGE_COLORING_DEFAULT} />
                        <label>{this.props.language['LC__UNIVERSAL__DEFAULT']}</label>
                    </li>
                    <li>
                        <input type="radio"
                               checked={this.props.display_settings.edge_coloring == EDGE_COLORING_CAPACITY}
                               onChange={this.handleEdgeColoring}
                               disabled={this.props.is_loading}
                               name="edge_coloring[]" value={EDGE_COLORING_CAPACITY} />
                        <label>{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__CAPACITY']}</label>
                    </li>
                    <li>
                        <input type="radio"
                               checked={this.props.display_settings.edge_coloring == EDGE_COLORING_CABLE_COLOR}
                               onChange={this.handleEdgeColoring}
                               disabled={this.props.is_loading}
                               name="edge_coloring[]" value={EDGE_COLORING_CABLE_COLOR} />
                        <label>{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__CABLE_COLOR']}</label>
                    </li>
                </ul>
            </div>

            <div>
                {this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__ROUTING']}
                <ul>
                    <li>
                        <input type="radio"
                               checked={this.props.display_settings.routing_shortest_path}
                               onChange={this.handleRouting}
                               disabled={this.props.is_loading}
                               name="routing[]" value="shortest_path" />
                        <label>{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__ROUTING_SHORTEST_PATH']}</label>
                    </li>
                    <li>
                        <input type="radio"
                               checked={!this.props.display_settings.routing_shortest_path}
                               onChange={this.handleRouting}
                               disabled={this.props.is_loading}
                               name="routing[]" value="optimal_path" />
                        <label>{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__ROUTING_OPTIMAL_PATH']}</label>
                    </li>
                    <li>
                        <label>{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__ROUTING_FREE_CONNECTION_COUNT']}:</label>
                        <input type="text"
                               className="input input-mini"
                               style={{width: '24px'}}
                               value={this.props.display_settings.routing_connection_count}
                               onChange={this.handleSetRoutingConnectionCount}
                               disabled={this.props.is_loading} />
                    </li>
                </ul>
            </div>
        </div>;
    }
}
