import {HashMap, IGraph, INode} from 'yfiles'
import {IFoldingView} from "yfiles/view-component";

/**
 * stolen by: https://github.com/yWorks/yfiles-for-html-demos/blob/master/demos/view/structureview/StructureView.js
 */
export class StructureView {

    private readonly labelPlaceholder: string;
    private syncFoldingState: boolean;
    private element2node: Map<Element, INode>;
    private node2element: HashMap<INode, Element>;
    private groupElementCounter: number;
    public onElementClicked: Function;
    private readonly rootListElement: Element;
    private graph: IGraph;
    private foldingView: IFoldingView;
    private masterGraph: IGraph;
    private readonly container: Element;

    constructor(selector: string) {

        this.element2node = new Map();
        this.node2element = new HashMap();
        this.groupElementCounter = 0;
        this.labelPlaceholder = '< node >';

        this.container = document.querySelector(selector);
        this.container.setAttribute('class', 'structure-view-container');
        const graphList = document.createElement('ol');
        graphList.setAttribute('class', 'structure-view-list');
        this.container.appendChild(graphList);

        // Create the 'Graph' root folder
        const li = document.createElement('li');
        li.setAttribute('class', 'structure-view-group');
        const label = document.createElement('label');
        label.setAttribute('class', 'structure-view-expanded');
        label.setAttribute('for', 'graph-root');
        label.textContent = 'Graph';
        const collapseBox = document.createElement('input');
        collapseBox.id = 'graph-root';
        collapseBox.type = 'checkbox';
        collapseBox.setAttribute('checked', 'true');
        collapseBox.addEventListener('change', e => {
            let target = e.target as HTMLInputElement;
            label.setAttribute(
                'class',
                target.checked ? 'structure-view-expanded' : 'structure-view-collapsed'
            )
        });

        li.appendChild(label);
        li.appendChild(collapseBox);

        this.rootListElement = document.createElement('ol');
        li.appendChild(this.rootListElement);

        // Append the 'Graph' root folder
        graphList.appendChild(li);
    }

    getContainer() {
        return this.container;
    }

    setGraph(value: IGraph) {
        if (this.graph === value) {
            return;
        }

        const previousSyncFoldingState = this.syncFoldingState;
        if (this.graph) {
            // reset the StructureView
            this.syncFoldingState = false;
            this.clearStructure();
            this.foldingView = null;
            this.masterGraph = null;
        }

        this.graph = value;
        if (this.graph) {
            // re-initialize the StructureView with the new graph
            this.foldingView = value.foldingView;
            if (this.foldingView === null) {
                this.masterGraph = value;
            } else {
                this.masterGraph = this.foldingView.manager.masterGraph;
            }
            this.buildStructure();
        }
        this.syncFoldingState = previousSyncFoldingState;
    }

    buildStructure() {
        this.clearStructure();
        this.masterGraph.getChildren(null).forEach(node => {
            // traverse the top-level nodes and recursively build the structure view
            this.buildRecursiveTreeStructure(this.rootListElement, node);
        })
    }

    buildRecursiveTreeStructure(rootElement: Element, node: INode) {
        if (this.masterGraph.getChildren(node).size > 0) {
            // add a new group element with its children
            this.appendGroupElement(rootElement, node);
        } else {
            // add a leaf
            this.appendNodeElement(rootElement, node);
        }
    }

    appendGroupElement(parentElement: Element, groupNode: INode) {
        // create the group
        const li = document.createElement('li');
        li.setAttribute('class', 'structure-view-group');

        // create the label
        const label = document.createElement('label');
        label.setAttribute('data-groupelement', 'group#' + this.groupElementCounter);
        label.setAttribute('class', 'structure-view-expanded');
        label.textContent =
            groupNode.labels.size > 0 ? groupNode.labels.first().text : this.labelPlaceholder;
        label.addEventListener('click', () => {
            if (typeof this.onElementClicked === 'function') {
                this.onElementClicked(groupNode)
            }
        });

        // the collapse/expand control is done via checkbox
        const collapseBox = document.createElement('input') as HTMLInputElement;
        collapseBox.id = 'group#' + this.groupElementCounter;
        collapseBox.type = 'checkbox';
        collapseBox.setAttribute('checked', 'true');

        // change folder icon if collapsed/expanded
        collapseBox.addEventListener('change', e => {
            let target = e.target as HTMLInputElement;
            label.setAttribute(
                'class',
                target.checked ? 'structure-view-expanded' : 'structure-view-collapsed'
            );
            if (this.syncFoldingState) {
                const masterGroupNode:INode = this.element2node.get(collapseBox.parentNode as Element);
                const viewGroupNode = this.foldingView.getViewItem(masterGroupNode);
                if (viewGroupNode === null) {
                    return
                }
                if (!target.checked) {
                    this.foldingView.collapse(viewGroupNode)
                } else {
                    this.foldingView.expand(viewGroupNode)
                }
            } else {
            }
        });

        // append the actual DOM elements
        li.appendChild(label);
        li.appendChild(collapseBox);

        // append the group element to the root list
        const root = document.createElement('ol');
        li.appendChild(root);
        parentElement.appendChild(li);

        // keep track of the elements
        this.node2element.set(groupNode, li);
        this.element2node.set(li, groupNode);

        // increase unique IDs for the groups
        this.groupElementCounter++;

        // recursively traverse the children
        const children = this.masterGraph.getChildren(groupNode);
        children.forEach(child => this.buildRecursiveTreeStructure(root, child))
    }

    appendNodeElement(parentElement: Element, node: INode) {
        // create the element
        const nodeElement = document.createElement('li');
        nodeElement.setAttribute('class', 'structure-view-node');
        nodeElement.textContent =
            node.labels.size > 0 ? node.labels.first().text : this.labelPlaceholder;
        // register the click callback
        nodeElement.addEventListener('click', () => {
            if (typeof this.onElementClicked === 'function') {
                this.onElementClicked(node)
            }
        });
        // append the element
        parentElement.appendChild(nodeElement);
        // keep track of the elements
        this.node2element.set(node, nodeElement);
        this.element2node.set(nodeElement, node)
    }

    clearStructure() {
        while (this.rootListElement.hasChildNodes()) {
            this.rootListElement.removeChild(this.rootListElement.firstChild)
        }
        this.element2node.clear();
        this.node2element.clear();
        this.groupElementCounter = 0
    }

    getMasterNode(node: INode) {
        return this.foldingView !== null ? this.foldingView.getMasterItem(node) : node
    }
}
