import {GraphComponent, IEdge, IModelItem, INode, MouseHoverInputMode, Point} from "yfiles/view-component";
import {QueryItemToolTipEventArgs, TimeSpan} from "yfiles";
import {GraphEditorInputMode} from "yfiles/view-editor";
import {SelectElement} from "./Select";

export interface EdgeTooltip {
    key: string,
    patched: number,
    count: number,
    cable_color: string,
    edge_coloring: string,
}

export class Tooltip {

    private graphComponent: GraphComponent;
    private readonly language: {[key: string]: string}

    constructor(graphComponent: GraphComponent, language: {[key: string]: string}) {
        this.graphComponent = graphComponent;
        this.language = language
        this.initializeTooltips = this.initializeTooltips.bind(this);
        this.createTooltipContent = this.createTooltipContent.bind(this)
    }

    initializeTooltips() {
        const inputMode = this.graphComponent.inputMode as GraphEditorInputMode;

        const mouseHoverInputMode: MouseHoverInputMode = inputMode.mouseHoverInputMode;
        mouseHoverInputMode.toolTipLocationOffset = new Point(15, 15);
        mouseHoverInputMode.delay = new TimeSpan(400);
        mouseHoverInputMode.duration = new TimeSpan(5000);

        // Register a listener for when a tooltip should be shown.
        inputMode.addQueryItemToolTipListener((src: GraphEditorInputMode, eventArgs: QueryItemToolTipEventArgs<IModelItem>) => {
            if (eventArgs.handled) {
                // Tooltip content has already been assigned -> nothing to do.
                return
            }

            // Use a rich HTML element as tooltip content. Alternatively, a plain string would do as well.
            eventArgs.toolTip = this.createTooltipContent(eventArgs.item);

            // Indicate that the tooltip content has been set.
            eventArgs.handled = true;
        })
    }

    createTooltipContent(item: IModelItem) {
        const tooltip = document.createElement('div');
        if (INode.isInstance(item) && item.tag) {
            let data = item.tag as SelectElement;
            const title = document.createElement('h4');
            title.innerHTML = data.title;
            tooltip.appendChild(title);

            let text = document.createElement('p');
            text.innerHTML = `ID: ${data.id}`;
            tooltip.appendChild(text);

            text = document.createElement('p');
            text.innerHTML = this.language['LC_UNIVERSAL__OBJECT_TYPE'] + `: ${data.type_title}`;
            tooltip.appendChild(text);
        } else if (IEdge.isInstance(item) && item.tag) {
            let data = item.tag as EdgeTooltip;
            let text = document.createElement('p');
            text.innerHTML = this.language['LC__CMDB__CATG__SMVIEW_PATCH__PATCHED_COUNT'] + `: ${data.patched}`;
            tooltip.appendChild(text);

            text = document.createElement('p');
            text.innerHTML = this.language['LC__CMDB__CATG__SMVIEW_PATCH__TOTAL_COUNT'] + `: ${data.count}`;
            tooltip.appendChild(text);
        } else {
            tooltip.innerHTML = '-';
        }
        return tooltip;
    }
}
