import * as React from 'react'

import { loadAjax } from '../api/ajax'
import { Rack, RackDeviceProps, SlotProps } from './components/Rack'
import { Device } from './components/Device/Device'
import { DeviceConnectorProps, DeviceObjectProps } from './components/Device/DeviceProps'
import {CommonPortProps, ConnectorStatus, Port, PortConnectorProps} from './components/Port/Port'
import { RackSelector } from './components/RackSelector'
import { PortConnectionSetup } from './components/Port/PortConnectionSetup'
import { Socket } from './components/Socket'

export interface Permissions {
    has_edit_permission: boolean,
    has_planner_edit_permission: boolean,
    has_planner_activate_permission: boolean,
    has_end_device_edit_permission: boolean,
}

export interface EditorComposeProps {
    ajax_url: string,
    image_url: string,
    spinner_url: string,
    session: string,
    rack_obj_id: number,
    device_obj_id: number,
    lang: string,
    language: {[key: string]: string},
    racks: {[key: number]: string},
    size: string,
    colors: {[name: string]: string},
    cable_colors: {[id: number]: string},
    cable_types: {[id: number]: string},
    connector_types: {[id: number]: string},
    status_list: {[id: number]: ConnectorStatus},
    use_cable_color: boolean,
    current_menu_width: number,
    right_align_connector_titles: boolean,
    permissions: Permissions,
    trace_right_port_directly: boolean,
}

export interface SocketDialog {
    is_visible: boolean,
    port_id?: number, // selected ID from port dialog
    side?: string, // port from left or right side
    type?: string, // socket of "link" or "patch"
}

interface EditorComposeState {
    selected_angle: string,
    left_rack: RackProps,
    left_device: DeviceContainerProps,
    left_port: AjaxPortProps,
    selected_left_port: number,

    right_rack: RackProps,
    right_device: DeviceContainerProps,
    right_port: AjaxPortProps,
    selected_right_port: number,

    comment: CommentProps,
    menu_width: number,

    selected_socket_port: number, // ID of selected port from socket dialog
    socket_dialog: SocketDialog // selected socket from port dialog
    is_planner_mode: boolean,
}

interface CommentProps {
    device_id: number,
    comment: string,
}

interface RackProps {
    id: number,
    title: string,
    units: number,
    has_half_he: boolean,
    sorting: string,
    devices_front: RackDeviceProps[],
    devices_front_vertical: RackDeviceProps[],
    devices_back: RackDeviceProps[],
    devices_back_vertical: RackDeviceProps[],
    vertical_slots_front: number,
    vertical_slots_back: number,
    angle: string,
    selected_device_id?: number,
    print_url: string,
}

interface DeviceContainerProps {
    device: DeviceObjectProps,
    connectors: DeviceConnectorProps[],
    angle: string,
    selected_port_id?: number,
    has_connectors?: boolean,
}

export interface AjaxPortProps extends CommonPortProps{
    id: number,
    status: string,
    number?: number,
    title?: string,
    url?: string,
}

export class EditorCompose extends React.Component<EditorComposeProps, EditorComposeState> {
  constructor (props: EditorComposeProps) {
    super(props)
    this.state = {
      selected_angle: 'F',
      left_rack: EditorCompose.getEmptyRack(),
      left_device: EditorCompose.getEmptyDevice(),
      left_port: EditorCompose.getEmptyPort(),
      selected_left_port: null,

      right_rack: EditorCompose.getEmptyRack(),
      right_device: EditorCompose.getEmptyDevice(),
      right_port: EditorCompose.getEmptyPort(),
      selected_right_port: null,

      comment: { device_id: null, comment: '' },
      menu_width: this.props.current_menu_width,

      selected_socket_port: null,
      socket_dialog: {
        is_visible: false
      },
      is_planner_mode: false,
    }

    this.loadRack = this.loadRack.bind(this)
    this.loadDevice = this.loadDevice.bind(this)
    this.loadPort = this.loadPort.bind(this)
    this.handleRackAngleChange = this.handleRackAngleChange.bind(this)
    this.handleDeviceAngleChange = this.handleDeviceAngleChange.bind(this)
    this.handlePortTrace = this.handlePortTrace.bind(this)
    this.handleSocketClick = this.handleSocketClick.bind(this)
    this.handlePortConnect = this.handlePortConnect.bind(this)
    this.handlePortDisconnect = this.handlePortDisconnect.bind(this)
    this.handleCableSave = this.handleCableSave.bind(this)
    this.handleCablePropertyChange = this.handleCablePropertyChange.bind(this)
    this.handleConnectorTypeChange = this.handleConnectorTypeChange.bind(this)
    this.handleConnectorStatusChange = this.handleConnectorStatusChange.bind(this)
    this.handleSocketSelectionChange = this.handleSocketSelectionChange.bind(this)
    this.handleMenuCollapse = this.handleMenuCollapse.bind(this)
    this.handleSideSwitch = this.handleSideSwitch.bind(this)
    this.handleDuplicateRack = this.handleDuplicateRack.bind(this)
    this.handleTogglePlannerMode = this.handleTogglePlannerMode.bind(this)
    this.handlePlannedConnection = this.handlePlannedConnection.bind(this)
    this.updateComment = this.updateComment.bind(this)
  }

  componentDidMount () {
    this.loadRack('left', this.props.rack_obj_id, this.props.device_obj_id)
    this.loadDevice(this.props.device_obj_id, 'left')
  }

  loadRack (side: string, rack_id?: number, device_id?: number) {
    let url = this.props.ajax_url + '&func=get-rack'
    if (rack_id) {
      url += '&rack_id=' + rack_id
    }
    if (device_id) {
      url += '&device_id=' + device_id
    }
    loadAjax(url, this.props.session)
      .then(res => {
        // update rack
        let rack = side == 'left' ? this.state.left_rack : this.state.right_rack
        rack = { ...rack, ...res.data }
        rack.selected_device_id = device_id

        let angle = 'F'
        if (device_id) {
            if (this._isDeviceInList(device_id, rack.devices_back)) {
              angle = 'B'
            }
            if (this._isDeviceInList(device_id, rack.devices_front_vertical)) {
              angle = 'FV'
            }
            if (this._isDeviceInList(device_id, rack.devices_back_vertical)) {
              angle = 'BV'
            }
            // fix: also check front to open devices assigned on both sides with front-angle first
            if (this._isDeviceInList(device_id, rack.devices_front)) {
                angle = 'F'
            }
        }
        rack.angle = angle
        this.updateRackState(rack, side)

        // reset device and port if rack has no selected device
        if (!device_id) {
          const device = EditorCompose.getEmptyDevice()
          this.updateDeviceState(device, side)

          const port = EditorCompose.getEmptyPort()
          this.updatePortState(port, side)

          this.updateSocketVisibility(side != 'left', false)
        }
      })
  }

  _isDeviceInList (device_id, devices) {
    const filteredDevices = devices.filter(d => d.id == device_id)
    if (filteredDevices.length > 0) {
      return true
    }

    // check if device is in a chassis
    let found = false
    for (const dev_key in devices) {
      if (!devices.hasOwnProperty(dev_key)) {
        continue
      }
      const dev = devices[dev_key]
      found = EditorCompose.isDeviceInChassis(dev.slots, device_id) ? true : found
    }
    return found
  }

  handleRackAngleChange (angle: string, side: string) {
    const rack = side == 'left' ? this.state.left_rack : this.state.right_rack
    rack.angle = angle
    this.updateRackState(rack, side)
  }

  updateRackState (rack: RackProps, side: string) {
    const rack_side = side == 'left' ? 'left_rack' : 'right_rack'
    this.setState((prevState) => ({
      ...prevState,
      [rack_side]: rack
    }))
  }

  loadDevice (obj_id: number, side: string, selected_port?: number) {
    loadAjax(this.props.ajax_url + '&func=get-grid&obj_id=' + obj_id, this.props.session)
      .then(res => {
        // update selected device
        const rack = side == 'left' ? this.state.left_rack : this.state.right_rack
        rack.selected_device_id = obj_id
        this.updateRackState(rack, side)

        // update device
        let device = side == 'left' ? this.state.left_device : this.state.right_device
        device = { ...device, ...res.data }
        device.selected_port_id = selected_port || null

        const front_slots = device.device.chassis_front ? device.device.chassis_front.slots : null
        let angle = EditorCompose.isDeviceInChassis(front_slots, device.device.id) ? 'F' : null
        if (!angle) {
          const back_slots = device.device.chassis_back ? device.device.chassis_back.slots : null
          angle = EditorCompose.isDeviceInChassis(back_slots, device.device.id) ? 'B' : null
        }
        if (!angle) {
          const ports = device.connectors.filter(c => c.connector_id == selected_port)
          if (ports.length > 0) {
            const port = ports[0]
            device.angle = port.angle
          }
        } else {
          device.angle = angle
        }
        this.updateDeviceState(device, side)

        // reset port if device has no connectors
        if (!selected_port || device.connectors.length == 0) {
          const port = EditorCompose.getEmptyPort()
          this.updatePortState(port, side)

          this.updateSocketVisibility(side != 'left', false)
        }
      })
  }

  static isDeviceInChassis (slots: SlotProps[], device_id: number) {
    if (slots) {
      for (const slot_key in slots) {
        const slot = slots[slot_key]
        for (const slot_dev_key in slot.devices) {
          const slot_dev = slot.devices[slot_dev_key]
          if (slot_dev.id == device_id) {
            return true
          }
        }
      }
    }
    return false
  }

  handleDeviceAngleChange (angle: string, side: string) {
    const device = side == 'left' ? this.state.left_device : this.state.right_device
    device.angle = angle
    this.updateDeviceState(device, side)
  }

  handleTogglePlannerMode() {
      this.setState((prevState) => ({
          ...prevState,
          is_planner_mode: ! this.state.is_planner_mode
      }))
  }

  updateDeviceState (device: DeviceContainerProps, side: string) {
    const device_side = side == 'left' ? 'left_device' : 'right_device'
    this.setState((prevState) => ({
      ...prevState,
      [device_side]: device
    }))
  }

  loadPort (id: number, side: string) {
    this.updateSocketVisibility(side == 'left', false)

    loadAjax(this.props.ajax_url + '&func=get-port&connector_id=' + id, this.props.session)
      .then(res => {
        // update selected port
        const device = side == 'left' ? this.state.left_device : this.state.right_device
        device.selected_port_id = id
        this.updateDeviceState(device, side)

        const port = { ...EditorCompose.getEmptyPort(), ...res.data }

        // reset save icon
        port.link.has_changed = false
        port.patch.has_changed = false

        this.updatePortState(port, side)

        this.updateSocketVisibility(side != 'left', false)

        if (this.props.trace_right_port_directly && side == 'left' && ! this.state.is_planner_mode) {
          // trace patch if possible
          const patch = port.patch
          if (patch && patch.id && patch.id > 0 && ! patch.is_socket) {
            this.loadRack('right', null, patch.obj_id)
            this.loadDevice(patch.obj_id, 'right', patch.id)
            this.loadPort(patch.id, 'right')
          }
        }
      })
  }

  handlePortTrace (side: string, device_id: number, port_id: number) {
    side = side == 'left' ? 'right' : 'left'
    this.loadRack(side, null, device_id)
    this.loadDevice(device_id, side, port_id)
    this.loadPort(port_id, side)

    this.updateSocketVisibility(side == 'right', false)
  }

  // someone clicked on the "socket"- or "manual end device"-button in the port dialog
  handleSocketClick (side: string, type: string, port_id: number) {
    // open socket dialog on the other side
    const sib_side = side == 'left' ? 'right' : 'left'
    const rack = EditorCompose.getEmptyRack()
    this.updateRackState(rack, sib_side)

    const device = EditorCompose.getEmptyDevice()
    this.updateDeviceState(device, sib_side)

    this.updateSocketVisibility(side == 'right', true, type, port_id)
  }

  // display or hide socket dialog
  updateSocketVisibility (on_left_side: boolean, visible: boolean, type?: string, port_id?: number) {
    const socket_dialog: SocketDialog = {
      is_visible: visible,
      side: on_left_side ? 'left' : 'right',
      type: type,
      port_id: port_id
    }
    this.setState((prevState) => ({
      ...prevState,
      selected_socket_port: null,
      socket_dialog: socket_dialog
    }))

    if (visible) {
      const port = on_left_side ? 'left_port' : 'right_port'
      this.setState((prevState) => ({
        ...prevState,
        [port]: EditorCompose.getEmptyPort()
      }))
    }
  }

  updatePortState (port: AjaxPortProps, side: string) {
    const port_side = side == 'left' ? 'left_port' : 'right_port'
    this.setState((prevState) => ({
      ...prevState,
      [port_side]: port
    }))
  }

  handlePortConnect (connector_a: number, connector_b: number, cable_name: string, add_to_exiting_cable: boolean) {
    const func = this.state.is_planner_mode ? '&func=connect-virtual-ports' : '&func=connect-ports'
    let url = this.props.ajax_url + func + '&connector_a=' + connector_a + '&connector_b=' + connector_b +
        '&cable_name=' + cable_name + '&add_to_exiting_cable=' + add_to_exiting_cable
    url = encodeURI(url)
    this.handlePortConnection(url)
  }

  handlePortDisconnect (connector_id: number) {
    const func = this.state.is_planner_mode ? '&func=disconnect-virtual-ports' : '&func=disconnect-ports'
    const url = this.props.ajax_url + func + '&connector_id=' + connector_id
    this.handlePortConnection(url)
  }

  handlePlannedConnection (action: string, connector_a: number, connector_b?: number) {
      if (! this.state.is_planner_mode) {
          return;
      }

      let url = this.props.ajax_url
      switch (action) {
          case 'active':
              url += '&func=activate-virtual-connection&connector_a=' + connector_a + '&connector_b=' + connector_b
              break;
          case 'active_all':
              url += '&func=activate-virtual-connections&device_id=' + connector_a
              break;
          case 'remove':
              url += '&func=remove-virtual-connection&connector_a=' + connector_a + '&connector_b=' + connector_b
              break;
      }

      url = encodeURI(url)
      this.handlePortConnection(url)
  }

  handlePortConnection (url: string) {
    // hide socket dialog
    this.updateSocketVisibility(true, false)

    loadAjax(url, this.props.session)
      .then(() => {
        this.loadDevice(this.state.left_rack.selected_device_id, 'left', this.state.left_device.selected_port_id)
        this.loadDevice(this.state.right_rack.selected_device_id, 'right', this.state.right_device.selected_port_id)
        this.loadPort(this.state.left_device.selected_port_id, 'left')
        this.loadPort(this.state.right_device.selected_port_id, 'right')
      })
  }

    handleCableSave (side: string, type: string) {
        const port = side == 'left' ? this.state.left_port : this.state.right_port
        const portProps = type == 'patch' ? port.patch : port.link

        // save via AJAX
        let url = this.props.ajax_url + '&func=change-cable&cable_id=' + portProps.cable_id
        url += `&length=${portProps.cable_length}&title=${portProps.cable_title}`
        url += `&color_id=${portProps.cable_color_id}&type_id=${portProps.cable_type_id}`
        url += `&obj_id=${portProps.obj_id}`
        if (this.state.is_planner_mode) {
            url += `&planned_end_device=${portProps.planned_manual_end_device_name}`
        } else {
            url += `&end_device=${portProps.manual_end_device_name}`
        }

        loadAjax(url, this.props.session).then(() => {})

        // reset save icon
        portProps.has_changed = false
        port[type] = portProps

        const port_side = side == 'left' ? 'left_port' : 'right_port'
        this.setState((prevState) => ({
            ...prevState,
            [port_side]: port,
        }))
    }

  handleCablePropertyChange (side: string, type: string, property: string, value?: number, value_str?: string) {
    const port = side == 'left' ? this.state.left_port : this.state.right_port
    const portProps = type == 'patch' ? port.patch : port.link
    portProps[property] = value || value_str
    portProps.has_changed = true

    port[type] = portProps
    this.updatePortState(port, side)

    const sib_port = side == 'left' ? this.state.right_port : this.state.left_port
    const sib_portProps = type == 'patch' ? sib_port.patch : sib_port.link
    if (portProps.cable_id == sib_portProps.cable_id) {
          // sync port values if cables are identically
          sib_portProps[property] = value || value_str
          sib_port[type] = sib_portProps
          this.updatePortState(sib_port, side == 'left' ? 'right' : 'left')
    }
  }

  handleConnectorTypeChange (side: string, connector_id: number, type: number) {
    // save via AJAX
    const url = this.props.ajax_url + '&func=change-connector-type&connector_id=' + connector_id + '&type=' + type
    loadAjax(url, this.props.session)
      .then(() => {})

    const port = side == 'left' ? this.state.left_port : this.state.right_port
    port.connector_type = type
    this.updatePortState(port, side)
  }

  handleConnectorStatusChange (side: string, connector_id: number, status: number) {
    // save via AJAX
    const url = this.props.ajax_url + '&func=change-connector-status&connector_id=' + connector_id + '&status=' + status
    loadAjax(url, this.props.session)
      .then(() => {})

    // update port
    const port = side == 'left' ? this.state.left_port : this.state.right_port
    port.connector_status = status
    this.updatePortState(port, side)

    // update device
    const device = side == 'left' ? this.state.left_device : this.state.right_device
    const connectors = []
    for (const key in device.connectors) {
      if (!device.connectors.hasOwnProperty(key)) {
        continue
      }

      const connector = device.connectors[key]
      if (connector.connector_id == connector_id) {
        connector.connector_status = status
      }
      connectors.push(connector)
    }
    device.connectors = connectors
    this.updateDeviceState(device, side)
  }

  // someone selected a port in socket dialog
  handleSocketSelectionChange (side: string, socket_id: number) {
    this.setState((prevState) => ({
      ...prevState,
      selected_socket_port: socket_id
    }))
  }

  handleMenuCollapse () {
    const menuWidth = this.state.menu_width > 50 ? 0 : this.props.current_menu_width
    const iconPosition = this.state.menu_width > 50 ? 1400 : 25;
    document.getElementById('contentArea').style.left = menuWidth + 'px'
    document.getElementById('nodeVisibilityIcon').style.right = iconPosition + 'px'
    this.setState((prevState) => ({
      ...prevState,
      menu_width: menuWidth
    }))
  }

  handleSideSwitch () {
    this.setState((prevState) => ({
      ...prevState,
      left_rack: this.state.right_rack,
      left_device: this.state.right_device,
      left_port: this.state.right_port,
      selected_left_port: this.state.selected_right_port,

      right_rack: this.state.left_rack,
      right_device: this.state.left_device,
      right_port: this.state.left_port,
      selected_right_port: this.state.selected_left_port
    }))
  }

  handleDuplicateRack (event: React.MouseEvent<HTMLInputElement>) {
    const side = event.currentTarget.name
    const rack_side = side == 'left' ? 'right_rack' : 'left_rack'
    const device_side = side == 'left' ? 'right_device' : 'left_device'
    const port_side = side == 'left' ? 'right_port' : 'left_port'

    const rack = side == 'left' ? this.state.left_rack : this.state.right_rack
    const device = side == 'left' ? this.state.left_device : this.state.right_device
    const port = side == 'left' ? this.state.left_port : this.state.right_port

    // create deep copies
    this.setState((prevState) => ({
      ...prevState,
      [rack_side]: JSON.parse(JSON.stringify(rack)),
      [device_side]: JSON.parse(JSON.stringify(device)),
      [port_side]: JSON.parse(JSON.stringify(port))
    }))
  }

  updateComment (side: string, angle: string, device_id: number, text: string) {
    const rack = side == 'left' ? this.state.left_rack : this.state.right_rack
    let devices = []
    switch (angle) {
      case 'F': devices = rack.devices_front; break
      case 'B': devices = rack.devices_back; break
      case 'FV': devices = rack.devices_front_vertical; break
      case 'BV': devices = rack.devices_back_vertical; break
    }
    for (const key in devices) {
      const device = devices[key]
      if (device.id == device_id) {
        devices[key].comment = text
      }
    }

    switch (angle) {
      case 'F': rack.devices_front = devices; break
      case 'B': rack.devices_back = devices; break
      case 'FV': rack.devices_front_vertical = devices; break
      case 'BV': rack.devices_back_vertical = devices; break
    }

    const rack_side = side == 'left' ? 'left_rack' : 'right_rack'
    this.setState((prevState) => ({
      ...prevState,
      [rack_side]: rack
    }))

    // save via AJAX
    const url = this.props.ajax_url + '&func=add-device-comment&obj_id=' + device_id + '&comment=' + encodeURI(text)
    loadAjax(url, this.props.session)
      .then(() => {})
  }

  getVirtualConnectionCount () {
      const vc_left = this.state.left_device.connectors.filter(c => c.virtual_connection != null && c.virtual_connection.id != null)
      const vc_right = this.state.right_device.connectors.filter(c => c.virtual_connection != null && c.virtual_connection.id != null)
      const keys_left = vc_left.map(c => c.connector_id > c.virtual_connection.id ? c.connector_id + '_' + c.virtual_connection.id : c.virtual_connection.id + '_' + c.connector_id)
      const keys_right = vc_right.map(c => c.connector_id > c.virtual_connection.id ? c.connector_id + '_' + c.virtual_connection.id : c.virtual_connection.id + '_' + c.connector_id)
      let virtualConnectionCount = keys_left.filter(key => !keys_right.includes(key)).length
      virtualConnectionCount += keys_right.length
      return virtualConnectionCount
  }

  static getEmptyRack () {
    const rack: RackProps = {
      id: -1,
      title: '',
      units: 0,
      has_half_he: false,
      sorting: 'asc',
      devices_front: [],
      devices_front_vertical: [],
      devices_back: [],
      devices_back_vertical: [],
      vertical_slots_front: 0,
      vertical_slots_back: 0,
      angle: 'F',
      print_url: ''
    }
    // create deep copy
    return JSON.parse(JSON.stringify(rack))
  }

  static getEmptyDevice () {
    const dev: DeviceContainerProps = {
      device: {
        title: '',
        type_title: '',
        url: '',
        print_url: '',
        grid_title: '',
        column_count: 0,
        row_count: 0,
        is_template: false
      },
      connectors: [],
      angle: 'F'
    }
    // create deep copy
    return JSON.parse(JSON.stringify(dev))
  }

  static getEmptyPort () {
    const dev: AjaxPortProps = {
      id: -1,
      connector_id: -1,
      connector_title: '',
      connector_type: -1,
      connector_type_id: -1,
      connector_type_title: '',
      connector_status: -1,
      status: '',
      patch: EditorCompose.getEmptyPortConnector(),
      link: EditorCompose.getEmptyPortConnector()
    }
    // create deep copy
    return JSON.parse(JSON.stringify(dev))
  }

  static getEmptyPortConnector () {
    const conn: PortConnectorProps = {
      id: -1,
      obj_id: -1,
      cable_id: null,
      cable_title: '',
      cable_length: ''
    }
    // create deep copy
    return JSON.parse(JSON.stringify(conn))
  }

  render () {
    const { language, permissions } = this.props
    const collapse_image = this.props.image_url + (this.state.menu_width > 50 ? 'arrow_out.png' : 'arrow_in_longer.png')
    const switch_image = this.props.image_url + 'arrow_refresh.png'
    const to_left_image = this.props.image_url + 'arrow_left.png'
    const to_right_image = this.props.image_url + 'arrow_right.png'

    let port_left
    if (this.state.socket_dialog.is_visible && this.state.socket_dialog.side == 'left') {
      port_left = <Socket ajax_url={this.props.ajax_url}
        spinner_url={this.props.spinner_url}
        session={this.props.session}
        language={language}
        side="left"
        socket_dialog={this.state.socket_dialog}
        cableTitle={this.state.right_port.link.cable_title}
        onPortSelectionChange={this.handleSocketSelectionChange} />
    } else {
      port_left = <Port {...this.state.left_port}
        side="left"
        is_planner_mode={this.state.is_planner_mode}
        permissions={permissions}
        onTraceClick={this.handlePortTrace}
        onSocketClick={this.handleSocketClick}
        onCablePropertyChange={this.handleCablePropertyChange}
        onCableSave={this.handleCableSave}
        onConnectorTypeChange={this.handleConnectorTypeChange}
        onConnectorStatusChange={this.handleConnectorStatusChange}
        language={language}
        cable_colors={this.props.cable_colors}
        cable_types={this.props.cable_types}
        connector_types={this.props.connector_types}
        status_list={this.props.status_list}
        image_url={this.props.image_url} />
    }

    let port_right
    if (this.state.socket_dialog.is_visible && this.state.socket_dialog.side == 'right') {
      port_right = <Socket ajax_url={this.props.ajax_url}
        spinner_url={this.props.spinner_url}
        session={this.props.session}
        language={language}
        side="right"
        socket_dialog={this.state.socket_dialog}
        onPortSelectionChange={this.handleSocketSelectionChange} />
    } else {
      port_right = <Port {...this.state.right_port}
        side="right"
        is_planner_mode={this.state.is_planner_mode}
        permissions={permissions}
        onTraceClick={this.handlePortTrace}
        onSocketClick={this.handleSocketClick}
        onCablePropertyChange={this.handleCablePropertyChange}
        onCableSave={this.handleCableSave}
        onConnectorTypeChange={this.handleConnectorTypeChange}
        onConnectorStatusChange={this.handleConnectorStatusChange}
        language={language}
        cable_colors={this.props.cable_colors}
        cable_types={this.props.cable_types}
        connector_types={this.props.connector_types}
        status_list={this.props.status_list}
        image_url={this.props.image_url} />
    }

    const style = { filter: 'hue-rotate(90deg)' }
    const virtualConnectionCount = this.getVirtualConnectionCount()
    const planner_image = this.props.image_url + (this.state.is_planner_mode || virtualConnectionCount > 0 ? 'wrench_orange.png' : 'wrench.png')
    let planner_title = ''
    if (this.state.is_planner_mode) {
      planner_title = language.LC__CMDB__CATG__SMVIEW_PATCH__PLANNER_MODE + (virtualConnectionCount > 0 ? ' (' + virtualConnectionCount + ')' : '')
    }

    return <div>
      <div id="rack_selector">
        <div className="rack_left">
          <button type="button" className="icon-button" onClick={this.handleMenuCollapse}>
            <img src={collapse_image} style={style} alt="<>" title={language.LC__CMDB__CATG__SMVIEW_PATCH__COLLAPSE_MENU} />
          </button>
          <button type="button" className="icon-button" onClick={this.handleSideSwitch}>
            <img src={switch_image} alt="<>" title={language.LC__CMDB__CATG__SMVIEW_PATCH__SWITCH_SIDES} />
          </button>
          <button type="button" className="icon-button" onClick={this.handleDuplicateRack} name="left">
            <img src={to_right_image} alt="->" title={language.LC__CMDB__CATG__SMVIEW_PATCH__OPEN_ON_OTHER_SIDE} />
          </button>
          <RackSelector racks={this.props.racks}
            selected_id={this.state.left_rack.id}
            side="left"
            onSelectionChange={this.loadRack} />
        </div>
        <div className="device_container">
            <button type="button"
                    className={this.state.is_planner_mode ? 'planner' : virtualConnectionCount > 0 ? 'planner-blink' : ''}
                    disabled={! permissions.has_planner_edit_permission}
                    onClick={() => this.handleTogglePlannerMode()}>
                {planner_title}&nbsp;
                <img src={planner_image} alt="[]" title={language.LC__CMDB__CATG__SMVIEW_PATCH__TOGGLE_PLANNER_MODE} />
            </button>
        </div>
        <div className="rack_right">
          <button type="button" className="icon-button" onClick={this.handleDuplicateRack} name="right">
            <img src={to_left_image} alt="<-" title={language.LC__CMDB__CATG__SMVIEW_PATCH__OPEN_ON_OTHER_SIDE} />
          </button>
          <RackSelector racks={this.props.racks}
            selected_id={this.state.right_rack.id}
            side="right"
            onSelectionChange={this.loadRack} />
        </div>
      </div>
      <div id="patch_editor">
        <div className="rack_left">
          <Rack {...this.state.left_rack}
            image_url={this.props.image_url}
            language={language}
            side="left"
            onUpdateComment={this.updateComment}
            onDeviceSelected={this.loadDevice}
            onAngleChange={this.handleRackAngleChange} />
        </div>
        <div className="device_container">
          <Device {...this.state.left_device}
            side="left"
            size={this.props.size}
            colors={this.props.colors}
            status_list={this.props.status_list}
            use_cable_color={this.props.use_cable_color}
            is_planner_mode={this.state.is_planner_mode}
            has_planner_activate_permission={permissions.has_planner_activate_permission}
            onSlotSelected={this.loadDevice}
            onPortSelectionChange={this.loadPort}
            onAngleChange={this.handleDeviceAngleChange}
            onActivatePlannedConnections={this.handlePlannedConnection}
            language={language}
            image_url={this.props.image_url}
            right_align_connector_titles={this.props.right_align_connector_titles} />
          <div className="device">
            {port_left}
            <PortConnectionSetup left_port={this.state.left_port}
              right_port={this.state.right_port}
              selected_socket_port={this.state.selected_socket_port}
              socket_dialog={this.state.socket_dialog}
              is_planner_mode={this.state.is_planner_mode}
              permissions={permissions}
              onConnectClicked={this.handlePortConnect}
              onDisconnectClicked={this.handlePortDisconnect}
              onHandlePlannedConnection={this.handlePlannedConnection}
              language={language}
              image_url={this.props.image_url}
              spinner_url={this.props.spinner_url}
              ajax_url={this.props.ajax_url}
              session={this.props.session} />
            {port_right}
          </div>
          <Device {...this.state.right_device}
            side="right"
            size={this.props.size}
            colors={this.props.colors}
            status_list={this.props.status_list}
            use_cable_color={this.props.use_cable_color}
            is_planner_mode={this.state.is_planner_mode}
            has_planner_activate_permission={permissions.has_planner_activate_permission}
            onSlotSelected={this.loadDevice}
            onPortSelectionChange={this.loadPort}
            onActivatePlannedConnections={this.handlePlannedConnection}
            onAngleChange={this.handleDeviceAngleChange}
            language={language}
            image_url={this.props.image_url}
            right_align_connector_titles={this.props.right_align_connector_titles} />
        </div>
        <div className="rack_right">
          <Rack {...this.state.right_rack}
            image_url={this.props.image_url}
            language={language}
            side="right"
            onUpdateComment={this.updateComment}
            onDeviceSelected={this.loadDevice}
            onAngleChange={this.handleRackAngleChange} />
        </div>
      </div>
    </div>
  }
}
