import * as React from 'react'
import { ChassisProps, DeviceConnectorProps, DeviceObjectProps } from './DeviceProps'
import { AngleSelector } from '../AngleSelector'
import { SlotTabs } from './SlotTab'
import { ConnectorStatus } from '../Port/Port'
import {DevicePortIcon} from "./DevicePortIcon";
import {DeviceListItem} from "./DeviceListItem";

interface DeviceProps {
    side: string,
    angle: string,
    size: string,
    colors: {[name: string]: string},
    status_list: {[id: number]: ConnectorStatus},
    use_cable_color: boolean,
    is_planner_mode: boolean,
    has_planner_activate_permission: boolean,
    device: DeviceObjectProps,
    connectors: DeviceConnectorProps[],
    onSlotSelected: Function,
    onPortSelectionChange: Function,
    onAngleChange: Function,
    onActivatePlannedConnections: Function,
    language: {[key: string]: string},
    selected_port_id?: number,
    image_url: string,
    has_connectors?: boolean,
    right_align_connector_titles: boolean,
}

interface DeviceState {
    is_image_view: boolean,
}

export class Device extends React.Component<DeviceProps, DeviceState> {
  constructor (props: DeviceProps) {
    super(props)
    this.state = {
      is_image_view: true
    }
    this.handleAngleChange = this.handleAngleChange.bind(this)
    this.handleSelectedIdChange = this.handleSelectedIdChange.bind(this)
    this.handleSlotSelected = this.handleSlotSelected.bind(this)
    this.toggleViewMode = this.toggleViewMode.bind(this)
  }

  handleAngleChange (angle: string) {
    this.props.onAngleChange(angle, this.props.side)
  }

  handleSelectedIdChange (id: number) {
    this.props.onPortSelectionChange(id, this.props.side)
  }

  handleSlotSelected (id: number) {
    this.props.onSlotSelected(id, this.props.side)
  }

  toggleViewMode () {
    this.setState((prevState) => ({
      ...prevState,
      is_image_view: !this.state.is_image_view
    }))
  }

  render () {
    const { device, image_url } = this.props
    const chassis:ChassisProps = this.props.angle == 'F' ? device.chassis_front : device.chassis_back
    let slotTab
    if (chassis) {
      // add slot tabs if device is in a chassis
      slotTab = <SlotTabs chassis={chassis}
                          onTabSelected={this.handleSlotSelected}
                          is_image_view={this.state.is_image_view}
                          selected_id={device.id} />
    }

    const activator = this.props.is_planner_mode ? <button type="button" className="icon-button" disabled={! this.props.has_planner_activate_permission} onClick={() => this.props.onActivatePlannedConnections('active_all', this.props.device.id)}>
        <img src={image_url + 'accept.png'} alt="[]" title={this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__ACTIVATE_PLANNED_CONNECTIONS} />
      </button> : null
    const angleSelector = <div>
      <AngleSelector
        angle={this.props.angle}
        onAngleChange={this.handleAngleChange}
        language={this.props.language} />
      <button type="button" className="icon-button" onClick={this.toggleViewMode}>
        <img src={image_url + 'table.png'} alt="[]" title={this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__TOGGLE_LIST_VIEW} />
      </button>
      <a className="icon-button" href={device.print_url} target="_blank">
        <img src={image_url + 'printer.png'} alt="P" title={this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__PRINT_DEVICE_DATA} />
      </a>
        {activator}
      {slotTab}
    </div>

    if (device.is_chassis) {
      // return only slot tabs if device is a chassis
      return <div className="device">
        <h3 className="gradient text-center">
          {device.title} ({device.type_title})
        </h3>
        {angleSelector}
      </div>
    }

    if (!device.grid_title && !device.title) {
      // no connection available
      return <div>&nbsp;</div>
    }

    if (!device.grid_title && device.title) {
      // no grid defined
      let div = <div className="no-ports">&nbsp;</div>
      if (this.props.has_connectors) {
        div = <div className="no-grid">
          <div>
            {this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__NO_GRID_DEFINED1}
                        &nbsp; '{device.title}' &nbsp;
            {this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__NO_GRID_DEFINED2}<br />
            <a href={device.url} target="_blank" className="button">
              {this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__OPEN_OBJECT}
            </a>
          </div>
        </div>
      }
      return <div className="device">
        <h3 className="gradient text-center">{device.title} ({device.type_title})</h3>
        {div}
      </div>
    }

    const rows = []
    // const grid = device
    if (!this.state.is_image_view) {
      rows.push(<tr key={this.props.side + '-head'}>
        <th>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__PORT}</th>
        <th>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__LINK_CONNECTION}</th>
        <th>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__PATCH_CONNECTION}</th>
      </tr>)
    }

    const angle = this.props.angle
    if (this.state.is_image_view) {
        let rowCount = device.row_count
        if (angle == 'B' && device.row_count_back) {
            rowCount = device.row_count_back
        }
        let columnCount = device.column_count
        if (angle == 'B' && device.column_count_back) {
            columnCount = device.column_count_back
        }
      for (let row = 1; row <= rowCount; row++) {
        const cells = []
        for (let column = 1; column <= columnCount; column++) {
          const res = this.props.connectors.filter(c => c.pos_x == column && c.pos_y == row && c.angle == angle)
          const c = res.length == 0 ? null : res[0]
          let is_selected = false
          if (c && this.props.selected_port_id != null) {
            if (c.connector_id == this.props.selected_port_id || c.sibling_id == this.props.selected_port_id) {
              is_selected = true
            }
          }

          cells.push(<DevicePortIcon connector={c}
            key={this.props.side + '-' + row + '-' + column}
            index={column}
            is_selected={is_selected}
            onSelectedIdChange={this.handleSelectedIdChange}
            language={this.props.language}
            colors={this.props.colors}
            status_list={this.props.status_list}
            use_cable_color={this.props.use_cable_color}
            right_align_connector_titles={this.props.right_align_connector_titles}
            is_planner_mode={this.props.is_planner_mode} />)
        }
          rows.push(<tr key={this.props.side + '-' + row}>{cells}</tr>)
      }
    } else {
      const res = this.props.connectors.filter(c => c.angle == angle)
      res.sort(function (a, b) {
        return (a.connector_title.toString() > b.connector_title.toString()) ? 1 : -1
      })

      res.forEach(c => {
        let is_selected = false
        if (this.props.selected_port_id != null) {
          if (c.connector_id == this.props.selected_port_id || c.sibling_id == this.props.selected_port_id) {
            is_selected = true
          }
        }
        rows.push(
          <DeviceListItem connector={c}
            key={this.props.side + '-' + c.connector_id}
            is_selected={is_selected}
            onSelectedIdChange={this.handleSelectedIdChange} />
        )
      })
    }

    let sizeClass = 'ports '
    switch (this.props.size) {
      case 'xs': sizeClass += 'layout-xs'; break
      case 's': sizeClass += 'layout-sm'; break
      case 'l': sizeClass += 'layout-lg'; break
      case 'xl': sizeClass += 'layout-xl'; break
    }

    if (!this.state.is_image_view) {
      sizeClass = 'listing'
    }

    let description = device.description ? '(' + device.description + ')' : ''
    description += this.props.is_planner_mode ? '  |  ' + this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__PLANNER_MODE : ''
    return <div className={! this.props.is_planner_mode ? "device" : "device planner"}>
      <h3 className="gradient text-center">
        {device.title} {description}
      </h3>
      {angleSelector}
      <table className={sizeClass}>
        <tbody>
          {rows}
        </tbody>
      </table>
    </div>
  }
}
