import * as React from 'react'
import {PortConnectionDetail} from './PortConnectionDetail'
import {CablePinList, CablePinProps} from "./CablePinList";
import {Select} from "../Select";
import {PortVirtualConnectionDetail} from "./PortVirtualConnectionDetail";
import {Permissions} from "../../EditorCompose";

export interface CommonPortProps {
    connector_id: number,
    connector_title: string,
    connector_type: number,
    connector_type_id: number,
    connector_type_title: string,
    connector_status: number,
    sibling_id?: number,
    patch?: PortConnectorProps,
    link?: PortConnectorProps,
    cable_pins?: CablePinProps[],
    virtual_connection?: VirtualConnection,
}

interface PortProps extends CommonPortProps{
    side: string,
    is_planner_mode: boolean,
    permissions: Permissions,
    onTraceClick: Function,
    onSocketClick: Function,
    onCablePropertyChange: Function,
    onCableSave: Function,
    onConnectorTypeChange: Function,
    onConnectorStatusChange: Function,
    language: {[key: string]: string},
    cable_colors: {[id: number]: string},
    cable_types: {[id: number]: string},
    connector_types: {[id: number]: string},
    status_list: {[id: number]: ConnectorStatus},
    image_url: string,
}

export interface PortConnectorProps {
    id?: number,
    title?: string,
    obj_id?: number,
    obj_title?: string,
    obj_url?: string,
    cable_id?: number,
    cable_title?: string,
    cable_length?: string,
    cable_color_id?: number,
    cable_color?: string,
    cable_type_id?: number,
    is_socket?: boolean,
    manual_end_device_name?: string,
    planned_manual_end_device_name?: string,
    feed_hosts?: string[],
    chassis_title?: string,
    slot_title?: string,
    has_changed?: boolean,
}

export interface VirtualConnection {
    id: number,
    title: string,
    obj_id: number,
    obj_title: string,
    obj_url: string,
    cable_title: string,
    status: string,
    has_changed?: boolean,
}

export interface ConnectorStatus {
    title: string,
    color: string,
}

export class Port extends React.Component<PortProps, {}> {
  constructor (props: PortProps) {
    super(props)
    this.handleTraceClick = this.handleTraceClick.bind(this)
    this.handleSocketClick = this.handleSocketClick.bind(this)
    this.handleConnectorTypeChange = this.handleConnectorTypeChange.bind(this)
    this.handleConnectorStatusChange = this.handleConnectorStatusChange.bind(this)
    this.getPreviewSpan = this.getPreviewSpan.bind(this)
  }

  handleTraceClick (connector_obj_id: number, connector_id: number) {
    this.props.onTraceClick(this.props.side, connector_obj_id, connector_id)
  }

  handleSocketClick (type: string, port_id: number) {
    this.props.onSocketClick(this.props.side, type, port_id)
  }

  handleConnectorTypeChange (type: number) {
    this.props.onConnectorTypeChange(this.props.side, this.props.connector_id, type)
  }

  handleConnectorStatusChange (status: number) {
    this.props.onConnectorStatusChange(this.props.side, this.props.connector_id, status)
  }

  getPreviewSpan (color_id: number) {
    const color = this.props.status_list[color_id]
    if (!color) {
      return
    }
    const is_hex = /(^#[0-9A-F]{6}$)|(^#[0-9A-F]{3}$)/i.test(color.color)
    let span
    if (color && is_hex) {
      const style = { backgroundColor: color.color }
      span = <span className="color-preview" style={style}> </span>
    }
    return span
  }

  render () {
    if (this.props.connector_id <= 0) {
      return <div className="port">&nbsp;</div>
    }

    let first_component
      if (this.props.is_planner_mode) {
          first_component = <PortVirtualConnectionDetail
              type="patch"
              port_connector={this.props.patch}
              virtual_port_connector={this.props.virtual_connection}
              cable_colors={this.props.cable_colors}
              cable_types={this.props.cable_types}
              image_url={this.props.image_url}
              side={this.props.side}
              onTraceClick={this.handleTraceClick}
              onSocketClick={this.handleSocketClick}
              onCablePropertyChange={this.props.onCablePropertyChange}
              onCableSave={this.props.onCableSave}
              language={this.props.language}
              disabled={true}
              permissions={this.props.permissions} />
      } else {
          first_component = <PortConnectionDetail
              type="patch"
              port_connector={this.props.patch}
              cable_colors={this.props.cable_colors}
              cable_types={this.props.cable_types}
              image_url={this.props.image_url}
              side={this.props.side}
              is_planner_mode={this.props.is_planner_mode}
              permissions={this.props.permissions}
              onTraceClick={this.handleTraceClick}
              onSocketClick={this.handleSocketClick}
              onCablePropertyChange={this.props.onCablePropertyChange}
              onCableSave={this.props.onCableSave}
              language={this.props.language} />
      }

    let second_component
    if (this.props.cable_pins && this.props.cable_pins.length > 0) {
      second_component = <CablePinList
          cable_pins={this.props.cable_pins}
          language={this.props.language} />
    } else {
      if (this.props.sibling_id) {
        // display second component only if a corresponding input is assigned
        second_component = <PortConnectionDetail
            type="link"
            port_connector={this.props.link}
            cable_colors={this.props.cable_colors}
            cable_types={this.props.cable_types}
            image_url={this.props.image_url}
            side={this.props.side}
            is_planner_mode={this.props.is_planner_mode}
            permissions={this.props.permissions}
            onTraceClick={this.handleTraceClick}
            onSocketClick={this.handleSocketClick}
            onCablePropertyChange={this.props.onCablePropertyChange}
            onCableSave={this.props.onCableSave}
            disabled={this.props.is_planner_mode}
            language={this.props.language} />
      }
    }

    const span = this.getPreviewSpan(this.props.connector_status)
    return <div className={! this.props.is_planner_mode ? "port" : "port planner"}>
      <h4 className="gradient text-center border">{this.props.connector_title}</h4>
      <table>
        <tbody>
          <tr>
            <td>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__TYPE}:</td>
            <td>
              <Select id="connector_type"
                options={this.props.connector_types}
                title={this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__TYPE}
                selected_option={this.props.connector_type_id}
                disabled={this.props.is_planner_mode}
                onChange={this.handleConnectorTypeChange} />
              <Select id="status"
                options={[]}
                title={this.props.language.LC__UNIVERSAL__STATUS}
                status_options={this.props.status_list}
                selected_option={this.props.connector_status}
                disabled={this.props.is_planner_mode}
                onChange={this.handleConnectorStatusChange} />
              {span}
            </td>
          </tr>
        </tbody>
      </table>

      {first_component}
      {second_component}
    </div>
  }
}
