import * as React from 'react'
import {AjaxPortProps, Permissions, SocketDialog} from '../../EditorCompose'
import { loadAjax } from '../../../api/ajax'

interface PortConnectionSetupProps {
    left_port: AjaxPortProps,
    right_port: AjaxPortProps,
    selected_socket_port: number,
    socket_dialog: SocketDialog,
    is_planner_mode: boolean,
    permissions: Permissions,
    onConnectClicked: Function,
    onDisconnectClicked: Function,
    onHandlePlannedConnection: Function,
    language: {[key: string]: string},
    image_url: string,
    spinner_url: string,
    ajax_url: string,
    session: string,
}

interface PortConnectionSetupState {
    cable_name: string,
    cables: string[],
    add_to_exiting_cable: boolean,
    is_loading: boolean,
}

export class PortConnectionSetup extends React.Component<PortConnectionSetupProps, PortConnectionSetupState> {
  constructor (props: PortConnectionSetupProps) {
    super(props)
    this.handlePatchConnect = this.handlePatchConnect.bind(this)
    this.handlePatchDisconnect = this.handlePatchDisconnect.bind(this)
    this.handleActivateVirtualConnection = this.handleActivateVirtualConnection.bind(this)
    this.handleRemoveVirtualConnection = this.handleRemoveVirtualConnection.bind(this)
    this.handleLinkConnect = this.handleLinkConnect.bind(this)
    this.handleLinkDisconnect = this.handleLinkDisconnect.bind(this)
    this.handleSocketConnect = this.handleSocketConnect.bind(this)
    this.handleSocketDisconnect = this.handleSocketDisconnect.bind(this)
    this.handleCableNameChange = this.handleCableNameChange.bind(this)
    this.handleCableNameOptionChange = this.handleCableNameOptionChange.bind(this)

    this.state = {
      cable_name: '',
      cables: [],
      add_to_exiting_cable: true,
      is_loading: false,
    }
  }

  handlePatchConnect () {
    this.props.onConnectClicked(this.props.left_port.connector_id, this.props.right_port.connector_id, this.state.cable_name, this.state.add_to_exiting_cable)
    this.setState((prevState) => ({
      ...prevState,
      cable_name: '',
      cables: [],
      add_to_exiting_cable: true
    }))
  }

  handlePatchDisconnect () {
    this.props.onDisconnectClicked(this.props.left_port.connector_id)
  }

  handleActivateVirtualConnection () {
    this.props.onHandlePlannedConnection('active', this.props.left_port.connector_id, this.props.right_port.connector_id)
  }

  handleRemoveVirtualConnection () {
    this.props.onHandlePlannedConnection('remove', this.props.left_port.connector_id, this.props.right_port.connector_id)
  }

  handleLinkConnect () {
    this.props.onConnectClicked(this.props.left_port.sibling_id, this.props.right_port.sibling_id, this.state.cable_name, this.state.add_to_exiting_cable)
    this.setState((prevState) => ({
      ...prevState,
      cable_name: '',
      cables: [],
      add_to_exiting_cable: true
    }))
  }

  handleLinkDisconnect () {
    this.props.onDisconnectClicked(this.props.left_port.sibling_id)
  }

  handleSocketConnect () {
    const sib_port = this.props.socket_dialog.side == 'right' ? this.props.left_port : this.props.right_port
    const sib_port_id = this.props.socket_dialog.type == 'link' ? sib_port.sibling_id : sib_port.connector_id
    if (!sib_port_id || !this.props.selected_socket_port) {
      return
    }
    this.props.onConnectClicked(sib_port_id, this.props.selected_socket_port, this.state.cable_name, this.state.add_to_exiting_cable)
  }

  handleSocketDisconnect () {
    const sib_port = this.props.socket_dialog.side == 'left' ? this.props.right_port : this.props.left_port
    const id = this.props.socket_dialog.type == 'link' ? sib_port.link.id : sib_port.patch.id
    this.props.onDisconnectClicked(id)
  }

  handleCableNameChange (event: React.ChangeEvent<HTMLInputElement>) {
    const cable_name = event.target.value

    // get existing cables by name
    let url = this.props.ajax_url + '&func=get-cables-by-name&cable_name=' + cable_name
    url = encodeURI(url)
    loadAjax(url, this.props.session)
      .then(res => {
        const cables = []
        const cableData = res.data
        cableData.forEach(c => {
          cables.push(c.title)
        })

        this.setState((prevState) => ({
          ...prevState,
          cables: cables
        }))
      })
        .finally(() => {
            this.setState((prevState) => ({
                ...prevState,
                is_loading: false
            }))
        })

    this.setState((prevState) => ({
      ...prevState,
      cable_name: cable_name,
      cables: [],
      is_loading: true,
    }))
  }

  handleCableNameOptionChange (event: React.ChangeEvent<HTMLInputElement>) {
    const option = event.target.value
    this.setState((prevState) => ({
      ...prevState,
      add_to_exiting_cable: option == 'merge'
    }))
  }

  render () {
    const { image_url, language, left_port, right_port, permissions} = this.props
    const elements = []
    const activate_image = image_url + 'accept.png'
    const connect_image = image_url + 'connect.png'
    const delete_image = image_url + 'delete.png'
    const disconnect_image = image_url + 'disconnect.png'
    let can_add_new_connection = false

      if (this.props.is_planner_mode) {
          // virtual patch connection
          if (left_port.connector_id > 0 && right_port.connector_id > 0) {
              if (left_port.virtual_connection.id && left_port.virtual_connection.id == right_port.connector_id && right_port.virtual_connection.id && right_port.virtual_connection.id == left_port.connector_id) {
                  // activate or remove existing virtual connection
                  let activateTitle = language.LC__CMDB__CATG__SMVIEW_PATCH__ACTIVATE_PLANNED_CONNECTION
                  let removeTitle = language.LC__CMDB__CATG__SMVIEW_PATCH__REMOVE_PLANNED_CONNECTION
                  if (left_port.virtual_connection.status == 'remove') {
                      activateTitle = language.LC__CMDB__CATG__SMVIEW_PATCH__ACTIVATE_PLANNED_CONNECTION_REMOVAL
                      removeTitle = language.LC__CMDB__CATG__SMVIEW_PATCH__REMOVE_PLANNED_CONNECTION_REMOVAL
                  }
                  elements.push(<button key="virtual_patch_activate"
                                        className="btn planner"
                                        type="button"
                                        disabled={! permissions.has_planner_activate_permission}
                                        onClick={this.handleActivateVirtualConnection}>
                      {activateTitle} <img src={activate_image} alt="" />
                  </button>)
                  elements.push(<button key="virtual_patch_remove"
                                        className="btn planner"
                                        type="button"
                                        disabled={! permissions.has_planner_edit_permission}
                                        onClick={this.handleRemoveVirtualConnection}>
                      {removeTitle} <img src={delete_image} alt="" />
                  </button>)
              } else {
                  if (left_port.patch && left_port.patch.id && left_port.patch.id == right_port.connector_id) {
                      // disconnect both connectors virtually
                      elements.push(<button key="virtual_patch_disconnect"
                                            className="btn planner"
                                            type="button"
                                            disabled={! permissions.has_planner_edit_permission}
                                            onClick={this.handlePatchDisconnect}>
                          {language.LC__CMDB__CATG__SMVIEW_PATCH__DISCONNECT_VIRTUAL_PATCH} <img src={disconnect_image} alt="" />
                      </button>)
                  } else {
                      // connect or re-connect both connectors virtually
                      let title = language.LC__CMDB__CATG__SMVIEW_PATCH__CONNECT_VIRTUAL_PATCH
                      if (left_port.virtual_connection && left_port.virtual_connection.id != null) {
                          title = language.LC__CMDB__CATG__SMVIEW_PATCH__RECONNECT_VIRTUAL_PATCH
                      } else {
                          can_add_new_connection = true
                      }
                      elements.push(<button key="virtual_patch_connect"
                                            className="btn planner"
                                            type="button"
                                            disabled={! permissions.has_planner_edit_permission || this.state.is_loading}
                                            onClick={this.handlePatchConnect}>
                          {title} <img src={connect_image} alt="" />
                      </button>)
                  }
              }
          }
      } else {
          // patch connection
          if (left_port.connector_id > 0 && right_port.connector_id > 0) {
              if (left_port.patch && left_port.patch.id && left_port.patch.id == right_port.connector_id) {
                  elements.push(<button key="patch"
                                        className="btn"
                                        type="button"
                                        disabled={! permissions.has_edit_permission}
                                        onClick={this.handlePatchDisconnect}>
                      {language.LC__CMDB__CATG__SMVIEW_PATCH__DISCONNECT_PATCH} <img src={disconnect_image} alt="" />
                  </button>)
              } else {
                  let title = language.LC__CMDB__CATG__SMVIEW_PATCH__CONNECT_PATCH
                  if (left_port.patch && left_port.patch.id != null) {
                      title = language.LC__CMDB__CATG__SMVIEW_PATCH__RECONNECT_PATCH
                  } else {
                      can_add_new_connection = true
                  }
                  elements.push(<button key="patch"
                                        className="btn"
                                        type="button"
                                        disabled={! permissions.has_edit_permission || this.state.is_loading}
                                        onClick={this.handlePatchConnect}>
                      {title} <img src={connect_image} alt="" />
                  </button>)
              }
          }
      }

    // link connection
    if (left_port.sibling_id > 0 && right_port.sibling_id > 0) {
      const is_connected = (left_port.link && left_port.link.id == right_port.sibling_id) ||
                               (right_port.link && right_port.link.id == left_port.sibling_id)
      if (is_connected) {
        elements.push(<button key="link"
                              className="btn"
                              type="button"
                              disabled={! permissions.has_edit_permission}
                              onClick={this.handleLinkDisconnect}>
          {language.LC__CMDB__CATG__SMVIEW_PATCH__DISCONNECT_LINK} <img src={disconnect_image} alt="" />
        </button>)
      } else {
        const is_socket = (left_port.link && left_port.link.is_socket) ||
                                (right_port.link && right_port.link.is_socket)
        if (!is_socket) {
          // connect or re-connect only normal link connections, not sockets
          let title = language.LC__CMDB__CATG__SMVIEW_PATCH__CONNECT_LINK
          const is_linked = (left_port.link && left_port.link.id != null) ||
                                    (right_port.link && right_port.link.id != null)
          if (is_linked) {
            title = language.LC__CMDB__CATG__SMVIEW_PATCH__RECONNECT_LINK
          } else {
            can_add_new_connection = true
          }
          elements.push(<button key="link"
                                className="btn"
                                type="button"
                                disabled={! permissions.has_edit_permission || this.state.is_loading}
                                onClick={this.handleLinkConnect}>
            {title} <img src={connect_image} alt=""/>
          </button>)
        }
      }
    }

    // socket connection
    if (this.props.socket_dialog.is_visible) {
      const can_disconnect_link = left_port.link.id > 0 || right_port.link.id > 0
      const can_disconnect_patch = left_port.patch.id > 0 || right_port.patch.id > 0
      const can_disconnect = this.props.socket_dialog.type == 'link' ? can_disconnect_link : can_disconnect_patch

      if (can_disconnect && permissions.has_edit_permission) {
        elements.push(<button key="socket"
                              className="btn"
                              type="button"
                              onClick={this.handleSocketDisconnect}>
          {language.LC__CMDB__CATG__SMVIEW_PATCH__DISCONNECT_SOCKET} <img src={disconnect_image} alt="" />
        </button>)
      } else {
        if (this.props.selected_socket_port) {
          can_add_new_connection = true
          let title = language.LC__CMDB__CATG__SMVIEW_PATCH__CONNECT_SOCKET
          if (this.props.socket_dialog.type == 'patch') {
            title = language.LC__CMDB__CATG__SMVIEW_PATCH__CONNECT_END_DEVICE
          }
          elements.push(<button key="socket"
                                className="btn"
                                type="button"
                                disabled={! permissions.has_edit_permission || this.state.is_loading}
                                onClick={this.handleSocketConnect}>
            {title} <img src={connect_image} alt=""/>
          </button>)
        }
      }
    }

    if (can_add_new_connection) {
      const name_options = []
      if (this.state.cables.length > 0) {
        // display cable name behaviour options
        name_options.push(<div className="cable_name_options">
          <span>{language.LC__CMDB__CATG__SMVIEW_PATCH__CABLE_NAME_ALREADY_EXISTS}:</span>
          <div>
            <input type="radio"
              name="name_option[]"
              value="add"
              defaultChecked={this.state.add_to_exiting_cable === false}
              onChange={this.handleCableNameOptionChange} />
            <label htmlFor="name_option">{language.LC__CMDB__CATG__SMVIEW_PATCH__ADD_CABLE_WITH_SAME_NAME}</label>
          </div>
          <div>
            <input type="radio"
              name="name_option[]"
              value="merge"
              defaultChecked={this.state.add_to_exiting_cable === true}
              onChange={this.handleCableNameOptionChange} />
            <label htmlFor="name_option">{language.LC__CMDB__CATG__SMVIEW_PATCH__ADD_TO_EXISTING_CABLE}</label>
          </div>
        </div>)
      }

      let spinner = null
      if (this.state.is_loading) {
        const spinner_url = this.props.spinner_url
        spinner = <span>
            <img src={spinner_url} alt="..." />&nbsp;
            {language.LC__CMDB__CATG__SMVIEW_PATCH__VERIFY_CABLE_NAME}...
        </span>
      }

      elements.push(<div key="new_connection_name">
        <label>{language.LC__CMDB__CATG__SMVIEW_PATCH__CABLE}:</label>
        <input key="new_cable_name"
          type="text"
          className="input"
          onChange={this.handleCableNameChange}
          value={this.state.cable_name} />
        {spinner}
        {name_options}
      </div>)
    }

    return <div className="port">
      {elements}
    </div>
  }
}
