import * as React from 'react'
import 'element-closest-polyfill'

import { AngleSelector } from './AngleSelector'
import { Comment } from './Comment'
import { EditorCompose } from '../EditorCompose'

interface RackTableProps {
    image_url: string,
    print_url: string,
    language: {[key: string]: string},
    onDeviceSelected: Function,
    onAngleChange: Function,
    side: string,
    angle: string,
    id: number,
    title: string,
    units: number,
    has_half_he: boolean,
    sorting: string,
    devices_front: RackDeviceProps[],
    devices_front_vertical: RackDeviceProps[],
    devices_back: RackDeviceProps[],
    devices_back_vertical: RackDeviceProps[],
    vertical_slots_front: number,
    vertical_slots_back: number,
    selected_device_id?: number,
    onUpdateComment: Function
}

interface RackTableState {
    is_image_view: boolean,
    show_comments: boolean,
}

export interface RackDeviceProps {
    id: number,
    title: string,
    status_color?: string,
    type_id: number,
    type_title: string,
    type_color: string,
    position: number,
    he: number,
    has_half_he: boolean,
    image_url: string,
    is_chassis: boolean,
    angle: string,
    chassis_width?: number,
    chassis_height?: number,
    slots?: SlotProps[],
    selected_port_id?: number,
    comment?: string,
}

export interface SlotProps {
    id: number,
    title: string,
    x_from: number,
    x_to: number,
    y_from: number,
    y_to: number,
    devices: SlotDeviceProps[],
}

interface SlotDeviceProps {
    id: number,
    title: string,
    image_url: string,
}

export class Rack extends React.Component<RackTableProps, RackTableState> {
  constructor (props: RackTableProps) {
    super(props)
    this.state = {
      is_image_view: true,
      show_comments: false
    }
    this.handleDeviceSelected = this.handleDeviceSelected.bind(this)
    this.handleAngleChange = this.handleAngleChange.bind(this)
    this.toggleViewMode = this.toggleViewMode.bind(this)
    this.toggleCommentDialog = this.toggleCommentDialog.bind(this)
  }

  handleDeviceSelected (id: number) {
    this.props.onDeviceSelected(id, this.props.side)
  }

  handleAngleChange (angle: string) {
    this.props.onAngleChange(angle, this.props.side)
  }

  toggleViewMode () {
    this.setState((prevState) => ({
      ...prevState,
      is_image_view: !this.state.is_image_view
    }))
  }

  toggleCommentDialog () {
    this.setState((prevState) => ({
      ...prevState,
      show_comments: !this.state.show_comments
    }))
  }

  render () {
    let units = this.props.units
    switch (this.props.angle) {
      case 'FV': units = this.props.vertical_slots_front; break
      case 'BV': units = this.props.vertical_slots_back; break
    }
    if (units <= 0) {
      return <div>&nbsp;</div>
    }

    const rows = []
    let device_he = 1
    for (let i:number = 1; i <= units; i++) {
      let position = this.props.sorting == 'asc' ? i : units - i + 1
      position = this.props.has_half_he ? position / 2 : position
      let he = 1
      let row = {
        position: position,
        he: 1
      }

      const row_span = device_he > 1
      device_he--

      let devices = []
      switch (this.props.angle) {
        case 'F': devices = this.props.devices_front; break
        case 'B': devices = this.props.devices_back; break
        case 'FV': devices = this.props.devices_front_vertical; break
        case 'BV': devices = this.props.devices_back_vertical; break
      }

      const filtered_devices = devices.filter(d => d.position == i)
      if (filtered_devices.length > 0) {
        const dev = filtered_devices[0]
        row = dev

        he = row.he
        if (row.he == 0 || row.he == null) {
          // fix invalid HE
          he = 1
        }
        // cast value to be integer
        he = parseInt(he.toString())

        if (this.props.has_half_he && !dev.has_half_he) {
          // duplicate HE
          he = he * 2
        }

        if (!this.props.has_half_he && dev.has_half_he) {
          // halve HE
          he = he % 2 == 0 ? (he / 2) : ((he + 1) / 2)
        }

        if (he == 0) {
          // set to 1 if it is 0
          he = 1
        }
      }
      rows.push(<Row {...row}
        key={this.props.side + '-' + position}
        he={he}
        position={position}
        row_span={row_span}
        selected_id={this.props.selected_device_id}
        onDeviceSelected={this.handleDeviceSelected}
        is_image_view={this.state.is_image_view}
        onUpdateComment={this.props.onUpdateComment}
        show_comment={this.state.show_comments}
        has_half_he={this.props.has_half_he}
        side={this.props.side}
        angle={this.props.angle}
        language={this.props.language} />)
      if (he > 1) {
        device_he = he
      }
    }

    const btn_image = this.props.image_url + (this.state.is_image_view ? 'table.png' : 'monitor.png')
    const print_image = this.props.image_url + 'printer.png'
    const comment_image = this.props.image_url + 'comments.png'
    const comment_class = 'icon-button ' + (!this.state.show_comments ? 'icon-button-disabled' : '')
    const class_name = 'rack ' + (this.props.has_half_he ? 'rack-half-he' : '')
    return <div>
      <h3 className="p5 gradient text-center border">{this.props.title}</h3>
      <div className={class_name}>
        <div>
          <AngleSelector
            angle={this.props.angle}
            onAngleChange={this.handleAngleChange}
            language={this.props.language}
            addFrontVerticalOption={this.props.vertical_slots_front > 0}
            addBackVerticalOption={this.props.vertical_slots_back > 0} />
          <button type="button" className="icon-button" onClick={this.toggleViewMode}>
            <img src={btn_image} alt="[]" title={this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__TOGGLE_LIST_VIEW} />
          </button>
          <button type="button" className={comment_class} onClick={this.toggleCommentDialog}>
            <img src={comment_image} alt="[]" title={this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__TOGGLE_COMMENT_DIALOG} />
          </button>
          <a className="icon-button" href={this.props.print_url} target="_blank">
            <img src={print_image} alt="P" title={this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__PRINT_RACK_DATA} />
          </a>
        </div>
        <table>
          <tbody>
            {rows}
          </tbody>
        </table>
      </div>
    </div>
  }
}

interface RowProps {
    side: string,
    angle: string,
    language: {[key: string]: string},
    onDeviceSelected: Function,
    position: number,
    he: number,
    row_span?: boolean,
    selected_id?: number,
    id?: number,
    title?: string,
    status_color?: string,
    type_title?: string,
    type_color?: string,
    image_url?: string,
    is_chassis?: boolean,
    chassis_width?: number,
    chassis_height?: number,
    slots?: SlotProps[],
    is_image_view: boolean,
    onUpdateComment: Function,
    comment?: string,
    show_comment: boolean,
    has_half_he: boolean,
}

class Row extends React.Component<RowProps, {}> {
  constructor (props: RowProps) {
    super(props)
    this.handleDeviceSelected = this.handleDeviceSelected.bind(this)
    this.updateCommentPosition = this.updateCommentPosition.bind(this)
  }

  handleDeviceSelected (event: React.MouseEvent<HTMLElement>) {
    const id = event.currentTarget.id
    this.props.onDeviceSelected(id)
  }

  updateCommentPosition (event: React.MouseEvent<HTMLElement>) {
    const parent_rect = event.currentTarget.closest('#contentArea').getBoundingClientRect()
    const rect = event.currentTarget.getBoundingClientRect()
    const comments = event.currentTarget.closest('td').getElementsByClassName('editable_comment')
    const comment = comments[0]
    if (!comment) {
      return
    }

    const comment_height = comment.getBoundingClientRect().height + 20
    const pos_x = this.props.side == 'left' ? (rect.width - 40) : -40
    let top = 0
    if (parent_rect.height > comment_height) {
      top = (rect.top + comment_height) > parent_rect.height ? (parent_rect.height - (rect.top + comment_height)) : 0
    }
    comment.setAttribute('style', this.props.side + ': ' + pos_x + 'px; top: ' + top + 'px')
  }

  render () {
    let position = Number.isInteger(this.props.position) ? this.props.position.toString() : ''
    if (this.props.row_span) {
      // create a spanning row over multiple rows
      return <tr>
        <td>{position}</td>
      </tr>
    }

    let element = null
    let he_factor = 27
    if (this.props.title) {
      he_factor = this.props.has_half_he ? 15 : 27
      const style = { height: (this.props.he * he_factor) + 'px', background: 'inherit', width: this.props.status_color ? '97%' : '100%' }
      if (this.props.image_url && this.props.is_image_view) {
        // render image of device
        position = this.props.position.toString() // also display half HE if a device is assigned
        element = <img src={this.props.image_url}
          alt={this.props.title}
          title={this.props.title}
          id={this.props.id.toString()}
          style={style}
          onClick={this.handleDeviceSelected}/>
      } else {
        // render device without image
        if (this.props.type_color) {
          style.background = '#' + this.props.type_color
        }
        element = <span className="no-image"
          id={this.props.id.toString()}
          style={style}
          onClick={this.handleDeviceSelected}>{this.props.title} ({this.props.type_title})</span>
      }
    }

    // handle selected device
    let is_selected = false
    if (this.props.selected_id) {
      if (this.props.selected_id == this.props.id) {
        is_selected = true
      } else {
        is_selected = EditorCompose.isDeviceInChassis(this.props.slots, this.props.selected_id)
      }
    }

    let comment = <div></div>
    if (this.props.show_comment) {
      comment = <div className="popup-comment">
        <Comment side={this.props.side}
          angle={this.props.angle}
          device_id={this.props.id}
          title={this.props.title}
          comment={this.props.comment}
          pos_x={150}
          pos_y={0}
          language={this.props.language}
          onEnterComment={this.props.onUpdateComment} />
      </div>
    }

    if (this.props.status_color) {
      element = <div>
        {element}
        <span className="device-status-color" style={{backgroundColor: this.props.status_color, height: (this.props.he * he_factor) + 'px'}}>&nbsp;</span>
      </div>
    }

    const className = is_selected ? 'slot selected' : 'slot'
    return <tr>
      <td>{position}</td>
      <td rowSpan={this.props.he} className={className} onMouseOver={this.updateCommentPosition}>{element}{comment}</td>
    </tr>
  }
}
