import * as React from 'react'
import { loadAjax } from '../../api/ajax'
import { Select } from './Select'
import { SocketDialog } from '../EditorCompose'

interface SocketProps {
    ajax_url: string,
    spinner_url: string,
    session: string,
    language: {[key: string]: string},
    side: string,
    socket_dialog: SocketDialog,
    cableTitle?: string,
    onPortSelectionChange: Function,
}

interface SocketState {
    object_types: {[id: number]: string},
    selected_object_type_id: number,
    objects: {[id: number]: string},
    selected_object_id: number,
    search_term_object: string,
    ports: {[id: number]: PortData},
    selected_port_id: number,
    search_term_port: string,
    original_port_id: number, // hack to detect changed properties while component is still mounted
    is_loading: boolean,
}

interface PortData {
    object_type_id: number,
    object_id: number,
    id: number,
    title: string,
    category: string,
    type: number,
    status: number,
}

export class Socket extends React.Component<SocketProps, SocketState> {
  constructor (props: SocketProps) {
    super(props)

    this.state = {
      object_types: {},
      selected_object_type_id: -1,
      objects: {},
      selected_object_id: -1,
      search_term_object: '',
      ports: {},
      selected_port_id: this.props.socket_dialog.port_id,
      search_term_port: '',
      original_port_id: -1,
      is_loading: false
    }

    this.handleObjectTypeChange = this.handleObjectTypeChange.bind(this)
    this.handleObjectChange = this.handleObjectChange.bind(this)
    this.handlePortClick = this.handlePortClick.bind(this)
    this.handleSearchTermObject = this.handleSearchTermObject.bind(this)
    this.handleSearchTermPort = this.handleSearchTermPort.bind(this)
    this.loadObjectTypes = this.loadObjectTypes.bind(this)
    this.loadObjects = this.loadObjects.bind(this)
    this.loadPorts = this.loadPorts.bind(this)
    this.loadPortData = this.loadPortData.bind(this)
  }

  componentDidMount () {
    this.componentDidUpdate(this.props, this.state)
  }

  componentDidUpdate (prevProps: Readonly<SocketProps>, prevState: Readonly<SocketState>, snapshot?: any) {
    if (this.props.socket_dialog.port_id != this.state.original_port_id) {
      if (this.props.socket_dialog.port_id) {
        this.loadPortData(this.props.socket_dialog.port_id)
      } else {
        this.loadObjectTypes()
      }
      this.setState((prevState) => ({
        ...prevState,
        original_port_id: this.props.socket_dialog.port_id
      }))
    }
  }

  async loadObjectTypes (object_type_id?: number) {
    this.setState((prevState) => ({
      ...prevState,
      is_loading: true
    }))

    let url = this.props.ajax_url + '&func=get-object-types'
    url += object_type_id ? '&object_type_id=' + object_type_id : ''
    url += '&connection_type=' + this.props.socket_dialog.type
    await loadAjax(url, this.props.session).then(res => {
      this.setState((prevState) => ({
        ...prevState,
        object_types: { ...res.data.object_types },
        selected_object_type_id: res.data.default_socket_type ? res.data.default_socket_type : -1,
        objects: {},
        selected_object_id: -1,
        ports: {},
        selected_port_id: -1,
        is_loading: false
      }), () => {
        if (res.data.default_socket_type) {
          this.loadObjects(res.data.default_socket_type)
        } else {
          this.props.onPortSelectionChange(this.props.side, null)
        }
      })
    })
  }

  async loadObjects (object_type_id: number, object_id?: number) {
    this.setState((prevState) => ({
      ...prevState,
      is_loading: true
    }))

    let url = this.props.ajax_url + '&func=get-objects&object_type_id=' + object_type_id
    url += object_id ? '&object_id=' + object_id : ''
    url += '&connection_type=' + this.props.socket_dialog.type
    await loadAjax(url, this.props.session).then(res => {
      this.setState((prevState) => ({
        ...prevState,
        objects: { ...res.data },
        selected_object_id: -1,
        ports: {},
        selected_port_id: -1,
        is_loading: false
      }), () => {
        this.props.onPortSelectionChange(this.props.side, null)
      })
    })
  }

  async loadPorts (object_id: number, port_id?: number) {
    this.setState((prevState) => ({
      ...prevState,
      is_loading: true
    }))

    let url = this.props.ajax_url + '&func=get-ports&object_id=' + object_id
    url += port_id ? '&port_id=' + port_id : ''
    url += '&connection_type=' + this.props.socket_dialog.type
    await loadAjax(url, this.props.session).then(res => {
      this.setState((prevState) => ({
        ...prevState,
        ports: { ...res.data },
        selected_port_id: -1,
        is_loading: false
      }), () => {
        this.props.onPortSelectionChange(this.props.side, null)
      })
    })
  }

  loadPortData (port_id: number) {
    this.setState((prevState) => ({
      ...prevState,
      is_loading: true
    }))

    let url = this.props.ajax_url + '&func=get-port-data&port_id=' + port_id
    url += '&connection_type=' + this.props.socket_dialog.type
    loadAjax(url, this.props.session).then(res => {
      const port_data = { ...res.data } as PortData
      this.loadObjectTypes(port_data.object_type_id).then(
        () => this.loadObjects(port_data.object_type_id, port_data.object_id).then(
          () => {
            this.loadPorts(port_data.object_id, port_data.id).then(
              () => {
                this.setState((prevState) => ({
                  ...prevState,
                  selected_object_type_id: port_data.object_type_id,
                  selected_object_id: port_data.object_id,
                  selected_port_id: port_data.id,
                  is_loading: false
                }), () => {
                  this.props.onPortSelectionChange(this.props.side, port_id)
                })
              }
            )
          }
        )
      )
    })
  }

  handleObjectTypeChange (type_id: number) {
    this.setState((prevState) => ({
      ...prevState,
      selected_object_type_id: type_id
    }), () => {
      this.loadObjects(type_id)
    })
  }

  handleObjectChange (object_id: number) {
    this.setState((prevState) => ({
      ...prevState,
      selected_object_id: object_id
    }), () => {
      this.loadPorts(object_id)
    })
  }

  handlePortClick (event: React.MouseEvent<HTMLInputElement>) {
    const port_id = parseInt(event.currentTarget.value)
    this.setState((prevState) => ({
      ...prevState,
      selected_port_id: port_id
    }), () => {
      this.props.onPortSelectionChange(this.props.side, port_id)
    })
  }

  handleSearchTermObject (event: React.ChangeEvent<HTMLInputElement>) {
    const search = event.currentTarget.value
    this.setState((prevState) => ({
      ...prevState,
      search_term_object: search
    }))
  }

  handleSearchTermPort (event: React.ChangeEvent<HTMLInputElement>) {
    const search = event.currentTarget.value
    this.setState((prevState) => ({
      ...prevState,
      search_term_port: search
    }))
  }

  render () {
    const lis = []
    for (const key in this.state.ports) {
      if (!this.state.ports.hasOwnProperty(key)) {
        continue
      }

      const port = this.state.ports[key]
      const name = port.title ? port.title : 'NO_NAME'
      const type = port.type ? port.type : '?'
      const title = `${name} (${type})`
      if (this.state.search_term_port != '' && !title.toLowerCase().includes(this.state.search_term_port.toLowerCase())) {
        continue
      }

      lis.push(<li key={this.props.side + '-' + port.id}>
        <input type="radio"
          name={this.props.side + '-sockets[]'}
          defaultChecked={this.state.selected_port_id == port.id}
          value={port.id}
          onClick={this.handlePortClick} />
        {title}
      </li>)
    }

    let ports
    if (Object.keys(this.state.ports).length > 0) {
      ports = <tr>
        <td>
          {this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__PORTS}:
        </td>
        <td>
          <input type='text'
                 value={this.state.search_term_port}
                 onChange={this.handleSearchTermPort}
                 placeholder={this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__SEARCH}
                 className="input input-small" />
          <ul>
            {lis}
          </ul>
        </td>
      </tr>
    }

    let title = this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__SOCKET_SELECTOR
    if (this.props.socket_dialog.type == 'patch') {
      title = this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__END_DEVICE_SELECTOR
    }

    let objects = {}
    if (this.state.search_term_object != '') {
      for (const key in this.state.objects) {
        if (this.state.objects.hasOwnProperty(key)) {
          const row = this.state.objects[key]
          if (row.toLowerCase().includes(this.state.search_term_object.toLowerCase())) {
            objects[key] = row
          }
        }
      }
    } else {
      objects = this.state.objects
    }

    let objectSearch
    if (Object.keys(this.state.objects).length > 0) {
      objectSearch = <input type='text' value={this.state.search_term_object} onChange={this.handleSearchTermObject} placeholder={this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__SEARCH} />
    }

    if (this.state.is_loading === true) {
      const spinner_url = this.props.spinner_url
      ports = <tr className="spinner">
        <td>&nbsp;</td>
        <td>
          <img src={spinner_url} alt="..." />
        </td>
      </tr>
    }

    return <div className="port socket">
      <h4 className="gradient text-center border">
        {title}
      </h4>
      <table>
        <tbody>
          <tr>
            <td>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__OBJECT_TYPES}:</td>
            <td>
              <Select id="object_types"
                options={this.state.object_types}
                selected_option={this.state.selected_object_type_id}
                onChange={this.handleObjectTypeChange} />
            </td>
          </tr>
          <tr>
            <td>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__OBJECTS}:</td>
            <td>
              {objectSearch}
              <Select id="objects"
                options={objects}
                selected_option={this.state.selected_object_id}
                onChange={this.handleObjectChange} />
            </td>
          </tr>
          {ports}
        </tbody>
      </table>
    </div>
  }
}
