import * as React from 'react'
import { SetupTable } from './components/SetupTable'
import { PortFrame } from './components/PortFrame'
import { loadAjax } from '../api/ajax'
import { SizeSelector } from './components/SizeSelector'
import { AvailablePortTable } from './components/AvailablePortTable'
import { SizeWidget } from "./components/SizeWidget"

interface Usage {
  title: string,
  type_title: string,
  url: string,
}

export interface Frame {
  id: number,
  title: string,
  column_count: number,
  row_count: number,
  column_count_back?: number,
  row_count_back?: number,
  is_template: boolean,
  modify_template: boolean,
  create_copy: boolean,
  used_by: Usage[],
}

export interface PortProps {
  id: number,
  title: string,
  number: number,
  pos_x: number,
  pos_y: number,
  rotation: number,
  label_orientation: string,
  status: string,
  angle: string,
  connector_id?: number,
  connector_title?: string,
  connector_type?: string,
  connector_type_id?: number,

  connector_type_title?: string,
  url: string,
  merged?: boolean,
  is_new?: boolean,
}

interface Grid {
  frame: Frame,
  ports: PortProps[],
}

export interface Connector {
  title?: string,
  connector_id: number,
  connector_title: string,
  connector_type: string,
  connector_type_id: number,
  connector_type_title: string,
  url: string,
}

export interface Size {
  label: string,
  cssClass: string,
}

export interface GridEditorComposeProps {
  is_editable: boolean,
  language: {[key: string]: string},
  ajax_url: string
  session: string,
  image_url: string,
  grid: Grid,
  connectors: Connector[],
  templates: {[key: number]: string},
  size: string,
  sizes: {[key: string]: Size},
  right_align_connector_titles: boolean,
}

interface GridEditorComposeState {
  current_angle: string,
  current_size: string,
  frame: Frame,
  ports: PortProps[],
  is_edit_template_mode: boolean,
  is_initial_mode: boolean,
  visible_frame_buttons?: string,
}

export class GridEditorCompose extends React.Component<GridEditorComposeProps, GridEditorComposeState> {
  constructor (props: GridEditorComposeProps) {
    super(props)
    this.state = {
      current_angle: 'F',
      current_size: this.props.size,
      frame: this.props.grid.frame,
      ports: this.props.grid.ports,
      is_edit_template_mode: false,
      is_initial_mode: !this.props.grid.frame.column_count
    }

    this.handleFramePropertyChange = this.handleFramePropertyChange.bind(this)
    this.handleSelectedTemplateChanged = this.handleSelectedTemplateChanged.bind(this)
    this.mergeGridTemplate = this.mergeGridTemplate.bind(this)
    this.handleAutoGeneration = this.handleAutoGeneration.bind(this)
    this.assignConnectorToGrid = this.assignConnectorToGrid.bind(this)
    this.handleEditTemplate = this.handleEditTemplate.bind(this)
    this.handleCopyTemplate = this.handleCopyTemplate.bind(this)
    this.handleAngleChange = this.handleAngleChange.bind(this)
    this.handleSizeChange = this.handleSizeChange.bind(this)
    this.handleFrameDrop = this.handleFrameDrop.bind(this)
    this.handleShowFrameButtons = this.handleShowFrameButtons.bind(this)
    this.handleRotatePort = this.handleRotatePort.bind(this)
    this.handleDeletePort = this.handleDeletePort.bind(this)
    this.handleMergePort = this.handleMergePort.bind(this)
    this.handleInsertEmptyLane = this.handleInsertEmptyLane.bind(this)
    this.handleRemoveLane = this.handleRemoveLane.bind(this)
    this._findMatchingConnectorByTemplate = this._findMatchingConnectorByTemplate.bind(this)
    this.updatePortState = this.updatePortState.bind(this)
    this.saveToDom = this.saveToDom.bind(this)
  }

  componentDidMount () {
    if (!this.props.grid.frame.column_count) {
      // set initial values on a new grid
      const port_count = this.props.connectors.length
      let col_count = port_count
      let row_count = 1
      const max_cols = 8 // default column count
      if (port_count / max_cols > 1) {
        col_count = max_cols
        row_count = Math.ceil(port_count / max_cols)
      }
      this.handleFramePropertyChange('column_count', null, col_count)
      this.handleFramePropertyChange('row_count', null, row_count)
      this.handleFramePropertyChange('column_count_back', null, col_count)
      this.handleFramePropertyChange('row_count_back', null, row_count)
    }
  }

  handleFramePropertyChange (property: string, str_value?: string, int_value?: number) {
    const frame = this.state.frame
    frame[property] = str_value || int_value
    this.setState((prevState) => ({
      ...prevState,
      frame: frame
    }), () => {
      this.saveToDom()
    })
  }

  handleEditTemplate () {
    this.handleFramePropertyChange('modify_template', null, 1)
    this.setState((prevState) => ({
      ...prevState,
      is_edit_template_mode: true
    }))
  }

  handleCopyTemplate () {
    const frame = this.state.frame
    frame.id = null
    frame.is_template = false
    // set this flag explicit to support copy for existing configurations
    frame.create_copy = true

    const ports = []
    this.state.ports.forEach(p => {
      // create deep copy
      const port = JSON.parse(JSON.stringify(p))
      port.id = null
      ports.push(port)
    })

    this.setState((prevState) => ({
      ...prevState,
      frame: frame,
      ports: ports
    }), () => {
      this.saveToDom()
    })
  }

  handleSizeChange (size: string) {
    this.setState((prevState) => ({
      ...prevState,
      current_size: size
    }))

    const url = this.props.ajax_url + '&func=set-size&size=' + size
    loadAjax(url, this.props.session).then(() => {})
  }

  handleAngleChange (e: React.ChangeEvent<HTMLSelectElement>) {
    const angle = e.currentTarget.value
    this.setState((prevState) => ({
      ...prevState,
      current_angle: angle
    }))
  }

  handleFrameDrop (target_x: number, target_y: number, dataTransfer) {
    // cast type save
    target_x = parseInt(target_x.toString())
    target_y = parseInt(target_y.toString())

    const connector_id = dataTransfer.getData('connector_id')
    const current_x = dataTransfer.getData('current_x')
    if (current_x) {
      // remove source port (this one was dragged)
      const current_y = dataTransfer.getData('current_y')
      this.handleDeletePort(current_x, current_y)
    }

    const available_ports = this.props.connectors.filter(p => p.connector_id == connector_id)
    if (available_ports.length == 0) {
      return
    }

    const id = dataTransfer.getData('id')
    const available_port = available_ports[0]
    const assigned_ports = this.state.ports.filter(p => p.connector_id == connector_id)
    let assigned_port = assigned_ports.length > 0 ? assigned_ports[0] : null
    if (!assigned_port) {
      assigned_port = {
        ...available_port,
        id: id,
        title: available_port.title ? available_port.title : available_port.connector_title,
        number: target_x,
        pos_x: target_x,
        pos_y: target_y,
        rotation: 0,
        label_orientation: '',
        status: '',
        angle: this.state.current_angle
      }
    } else {
      // update port
      assigned_port.pos_x = target_x
      assigned_port.pos_y = target_y
      assigned_port.url = available_port.url
    }

    // get all ports except on target location
    const ports = this.state.ports.filter(p => p.angle != this.state.current_angle || !(p.pos_x == target_x && p.pos_y == target_y))
    ports.push(assigned_port)
    this.updatePortState(ports)
  }

  handleShowFrameButtons (frameId?: string) {
    this.setState((prevState) => ({
      ...prevState,
      visible_frame_buttons: frameId
    }))
  }

  handleRotatePort (x: number, y: number) {
    const ports: PortProps[] = []
    this.state.ports.forEach(p => {
      if (p.angle == this.state.current_angle && p.pos_x == x && p.pos_y == y) {
        const rotation = parseInt(p.rotation.toString())
        p.rotation = rotation >= 270 ? 0 : rotation + 90
      }
      ports.push(p)
    })
    this.updatePortState(ports)
  }

  handleDeletePort (x: number, y: number) {
    const ports = this.state.ports.filter(p => p.angle != this.state.current_angle || !(p.pos_x == x && p.pos_y == y))
    this.updatePortState(ports)
  }

  handleMergePort (connector_id: number) {
    // get requested port
    let ports = this.state.ports.filter(p => p.id == connector_id)
    const port = ports.length > 0 ? ports[0] : null
    if (!port) {
      return
    }

    const available_port = this._findMatchingConnectorByTemplate(port)
    port.merged = available_port != null
    if (available_port) {
      port.connector_id = available_port.connector_id
      port.url = available_port.url
    }
    port.is_new = false

    // add to assigned ports
    ports = this.state.ports.filter(p => p.id != connector_id)
    ports.push(port)
    this.updatePortState(ports)
  }

  _findMatchingConnectorByTemplate (conn: PortProps) {
    const ports = this.props.connectors.filter(c => conn.connector_type == c.connector_type && conn.connector_type_id == c.connector_type_id && conn.connector_title == c.connector_title)
    return ports.length > 0 ? ports[0] : null
  }

  handleAutoGeneration (direction) {
    // assign ports to grid
    if (direction == '') {
      return
    }

    let row_count = this.state.frame.row_count
    if (this.state.current_angle == 'B' && this.state.frame.row_count_back) {
      row_count = this.state.frame.row_count_back
    }
    let column_count = this.state.frame.column_count
    if (this.state.current_angle == 'B' && this.state.frame.column_count_back) {
      column_count = this.state.frame.column_count_back
    }

    let x = 1
    let y = 1
    let ports = this.state.ports
    this.props.connectors.forEach(connector => {
      if (x > column_count || y > row_count) {
        // more ports than frames, abort
        return
      }
      ports = this.assignConnectorToGrid(connector, x, y, ports)

      // update coords
      if (direction == 'l_r') {
        if ((x + 1) > column_count) {
          x = 1
          y++
        } else {
          x++
        }
      }

      if (direction == 't_b') {
        if ((y + 1) > row_count) {
          y = 1
          x++
        } else {
          y++
        }
      }
    })
    this.updatePortState(ports)
  }

  assignConnectorToGrid (connectorObj, x, y, ports) {
    const connector_id = connectorObj.connector_id
    const assigned_ports = ports.filter(c => c.connector_id == connector_id)
    let port = assigned_ports.length > 0 ? assigned_ports[0] : null

    if (!port) {
      // add available connector to grid
      const title = connectorObj.title ? connectorObj.title : connectorObj.connector_title
      port = {
        id: null,
        connector_id: connector_id,
        title: title,
        number: x,
        pos_x: x,
        pos_y: y,
        rotation: 0,
        label_orientation: '',
        status: '',
        angle: this.state.current_angle,
        url: null
      }
    }

    // update connector
    port.pos_x = x
    port.pos_y = y
    port.url = connectorObj.url

    // get all ports except from given position
    const updated_ports = ports.filter(c => c.angle != this.state.current_angle || !(c.pos_x == x && c.pos_y == y))

    // get all port except given connector
    ports = updated_ports.filter(p => p.connector_id != connector_id)
    ports.push(port)
    return ports
  }

  handleInsertEmptyLane (e: React.MouseEvent<HTMLButtonElement>) {
    const lane = e.currentTarget.name
    const input = e.currentTarget.parentElement.querySelector('input')
    const pos = input.value

    this._updateGridCount(lane, '+')
    this._moveConnectorsOfLane(this.state.ports, pos, lane, '+')
  }

  handleRemoveLane (e: React.MouseEvent<HTMLButtonElement>) {
    const lane = e.currentTarget.name
    const input = e.currentTarget.parentElement.querySelector('input')
    const pos = input.value

    let coord = lane == 'x' ? 'pos_x' : 'pos_y'
    let ports = this.state.ports.filter(c => c[coord] != pos)

    this._updateGridCount(lane, '-')
    this._moveConnectorsOfLane(ports, pos, lane, '-')
  }

  _updateGridCount (lane, operation) {
    let frame_coord = lane == 'x' ? 'column_count' : 'row_count'
    frame_coord += this.state.current_angle == 'B' ? '_back' : ''
    let count = parseInt(this.state.frame[frame_coord])
    count = operation == '+' ? count + 1 : count - 1

    const frame = this.state.frame
    frame[frame_coord] = count < 1 ? 1 : count
    this.setState((prevState) => ({
      ...prevState,
      frame: frame
    }))
  }

  _moveConnectorsOfLane (assigned_ports: PortProps[], ref_pos, lane, operation) {
    const ports = []
    for (const key in assigned_ports) {
      if (!assigned_ports.hasOwnProperty(key)) {
        continue
      }

      let coordinate = lane == 'x' ? 'pos_x' : 'pos_y'
      let port = assigned_ports[key]
      if (port[coordinate] >= ref_pos) {
        // move this connector one column
        let pos = parseInt(port[coordinate])
        pos = operation == '+' ? pos + 1 : pos - 1
        port[coordinate] = pos
      }
      ports.push(port)
    }
    this.updatePortState(ports)
  }

  handleSelectedTemplateChanged (template_id) {
    const url = this.props.ajax_url + '&func=get-template&grid_template_id=' + template_id
    loadAjax(url, this.props.session)
      .then(res => {
        this.mergeGridTemplate(template_id, { ...res.data })
      })
  }

  mergeGridTemplate (grid_template_id, template) {
    // try to merge connector templates with connector data
    const grid = template.grid as Frame
    const connector_templates = template.connectors as PortProps[]

    const frame = grid
    frame.id = grid_template_id
    // frame.modify_template = false
    // frame.create_copy = false
    frame.is_template = this.state.frame.is_template
    frame.title = this.state.frame.title
    const ports = []

    console.debug('merge grid template')
    console.debug(frame)

    connector_templates.forEach(connector_template => {
      // try to find this connector
      const connector = this._findMatchingConnectorByTemplate(connector_template)
      connector_template.merged = connector != null
      if (connector) {
        connector_template.connector_id = connector.connector_id
      }
      ports.push(connector_template)
    })

    this.setState((prevState) => ({
      ...prevState,
      frame: frame,
      ports: ports
    }), () => {
      this.saveToDom()
    })
  }

  updatePortState (ports, saveToDom = true) {
    this.setState((prevState) => ({
      ...prevState,
      ports: ports
    }), () => {
      if (saveToDom) {
        this.saveToDom()
      }
    })
  }

  saveToDom () {
    // save json to DOM
    const json = JSON.stringify({
      frame: this.state.frame,
      ports: this.state.ports
    })
    document.getElementById('C__CATG__SMVIEW_PATCH_GRID').setAttribute('value', json)
  }

  renderLaneTitle (lane: string, pos: number) {
    return <td key={lane + '_' + pos}>
      <button type="button" className="button icon" name={lane} onClick={this.handleInsertEmptyLane}>
        <img src={this.props.image_url + 'add.png'} className="vam" alt="+" />
      </button>
      <input id={'col-title-' + pos} disabled={true} className="input small" value={pos} />
      <button type="button" className="button icon" name={lane} onClick={this.handleRemoveLane}>
        <img src={this.props.image_url + 'cross.png'} className="vam" alt="-" />
      </button>
    </td>
  }

  renderSizeWidgets(columnCount: number, rowCount: number, col_span: number) {
      let size_widgets
      if (! this.props.is_editable) {
          return null
      }

      if (!this.state.frame.is_template || this.state.is_edit_template_mode || this.state.is_initial_mode) {
          size_widgets = <tr>
              <td colSpan={col_span}>
                  <table>
                      <tbody>
                          <tr key="size_options_titles">
                              <td>&nbsp;</td>
                              <td className="text-center">{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__ROW_COUNT}</td>
                              <td className="text-center">{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__COLUMN_COUNT}</td>
                          </tr>
                          <tr key="size_options">
                              <td>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__SIZE}</td>
                              <td className="text-center">
                                  <SizeWidget widget_id="row_count" angle={this.state.current_angle} value={rowCount} onWidgetSizeChange={this.handleFramePropertyChange} />
                              </td>
                              <td className="text-center">
                                  <SizeWidget widget_id="column_count" angle={this.state.current_angle} value={columnCount} onWidgetSizeChange={this.handleFramePropertyChange} />
                              </td>
                          </tr>
                      </tbody>
                  </table>
              </td>
          </tr>
      }
      return size_widgets
  }

  render () {
    let rowCount = this.state.frame.row_count
    if (this.state.current_angle == 'B' && this.state.frame.row_count_back) {
      rowCount = this.state.frame.row_count_back
    }
    let columnCount = this.state.frame.column_count
    if (this.state.current_angle == 'B' && this.state.frame.column_count_back) {
      columnCount = this.state.frame.column_count_back
    }

    // render angle selector
    let col_span = columnCount + 1
    col_span = !col_span ? 1 : col_span
    const angle_options = [
      <option id="F" key="F" value="F">{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_FRONT}</option>,
      <option id="B" key="B" value="B">{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_BACK}</option>
    ]
    const angle_selector = <tr>
      <td colSpan={col_span}>
        <select id="angle_selector" className="input input-mini" value={this.state.current_angle} onChange={this.handleAngleChange}>
          {angle_options}
        </select>
      </td>
    </tr>

    // render column titles
    let column_titles = null
    const is_editable = this.props.is_editable && (!this.state.frame.is_template || this.state.is_edit_template_mode)
    if (is_editable) {
      const tds = []
      for (let x = 0; x <= columnCount; x++) {
        const lane_title = x > 0 ? this.renderLaneTitle('x', x) : <td>&nbsp;</td>
        tds.push(lane_title)
      }
      column_titles = <tr>{tds}</tr>
    }

    // render frames
    const frames = []
    for (let y = 1; y <= rowCount; y++) {
      const lane_title = is_editable ? this.renderLaneTitle('y', y) : <td>&nbsp;</td>
      const tds = []
      for (let x = 1; x <= columnCount; x++) {
        const ports = this.state.ports.filter(p => p.angle == this.state.current_angle && p.pos_x == x && p.pos_y == y)
        const port = ports.length > 0 ? ports[0] : null
        tds.push(<td key={'port_' + x + '_' + y}>
          <PortFrame x={x}
            y={y}
            is_editable={is_editable}
            is_display_new={this.props.is_editable && this.state.frame.is_template}
            visibleFrameButtons={this.state.visible_frame_buttons}
            onShowFrameButtons={this.handleShowFrameButtons}
            onRotateClick={this.handleRotatePort}
            onDeleteClick={this.handleDeletePort}
            onMergeClick={this.handleMergePort}
            onDrop={this.handleFrameDrop}
            language={this.props.language}
            image_url={this.props.image_url}
            port={port}
            right_align_connector_titles={this.props.right_align_connector_titles} />
        </td>)
      }
      frames.push(<tr key={'frame_' + y}>
        {lane_title}
        {tds}
      </tr>)
    }

    const size = this.props.sizes[this.state.current_size]
    const layoutClassName = 'layout ' + size.cssClass
    return <div id="grid_editor">
      <SetupTable language={this.props.language}
        frame={this.state.frame}
        templates={this.props.templates}
        is_editable={this.props.is_editable}
        is_edit_template_mode={this.state.is_edit_template_mode}
        is_initial_mode={this.state.is_initial_mode}
        onFramePropertyChange={this.handleFramePropertyChange}
        onTemplateChange={this.handleSelectedTemplateChanged}
        onAutoGeneration={this.handleAutoGeneration}
        onEditTemplate={this.handleEditTemplate}
        onCopyTemplate={this.handleCopyTemplate} />
      <div className="layout-container">
        <div className="gradient">
          <span>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__FRAME_EDITOR}</span>
        </div>
        <div>
          <table id="layout" className={layoutClassName}>
            <SizeSelector sizes={this.props.sizes}
              current_size={this.state.current_size}
              col_span={col_span}
              onChange={this.handleSizeChange}
              language={this.props.language}
              is_editable={is_editable} />
            <tbody>
              {angle_selector}
              {this.renderSizeWidgets(columnCount, rowCount, col_span)}
              {column_titles}
              {frames}
            </tbody>
          </table>
        </div>
      </div>
      <AvailablePortTable all_connectors={this.props.connectors}
        assigned_ports={this.state.ports}
        language={this.props.language}
        is_editable={is_editable}
        right_align_connector_titles={this.props.right_align_connector_titles} />
    </div>
  }
}
