import * as React from 'react'
import { Frame } from '../GridEditorCompose'

interface SetupTableProps {
  language: {[key: string]: string},
  frame: Frame,
  templates: {[key: number]: string},
  is_editable: boolean,
  is_edit_template_mode: boolean,
  is_initial_mode: boolean,
  onFramePropertyChange: Function,
  onTemplateChange: Function,
  onAutoGeneration: Function,
  onEditTemplate: Function,
  onCopyTemplate: Function,
}

interface SetupTableState {
  direction: string,
}

export class SetupTable extends React.Component<SetupTableProps, SetupTableState> {
  constructor (props: SetupTableProps) {
    super(props)
    this.state = {
      direction: 'l_r'
    }
    this.handleTitleChange = this.handleTitleChange.bind(this)
    this.handleIsTemplateChange = this.handleIsTemplateChange.bind(this)
    this.handleTemplateChange = this.handleTemplateChange.bind(this)
    this.handleEditTemplate = this.handleEditTemplate.bind(this)
    this.handleCopyTemplate = this.handleCopyTemplate.bind(this)
    this.handleDirectionChange = this.handleDirectionChange.bind(this)
    this.handleGenerateFrame = this.handleGenerateFrame.bind(this)
    this.getGridTemplates = this.getGridTemplates.bind(this)
    this.getUsedBy = this.getUsedBy.bind(this)
  }

  handleTitleChange (e: React.FormEvent<HTMLInputElement>) {
    this.props.onFramePropertyChange('title', e.currentTarget.value)
  }

  handleIsTemplateChange (e: React.FormEvent<HTMLInputElement>) {
    this.props.onFramePropertyChange('is_template', e.currentTarget.checked)
  }

  handleTemplateChange (e: React.FormEvent<HTMLSelectElement>) {
    if (e.currentTarget.value != '0') {
      this.props.onTemplateChange(e.currentTarget.value)
    }
  }

  handleEditTemplate () {
    this.props.onEditTemplate()
  }

  handleCopyTemplate () {
    this.props.onCopyTemplate()
  }

  handleDirectionChange (e: React.FormEvent<HTMLSelectElement>) {
    const dir = e.currentTarget.value
    this.setState((prevState) => ({
      ...prevState,
      direction: dir
    }))
  }

  handleGenerateFrame () {
    const direction = this.state.direction
    this.props.onAutoGeneration(direction)
  }

  getGridTemplates () {
      let options = Object.entries(this.props.templates)
          .sort((a,b) => a[1].toLowerCase() > b[1].toLowerCase() ? 1 : -1)
          .map(option => <option key={option[0]} value={option[0]}>{option[1]}</option>)
      options.unshift(<option key={0} value={0}>-</option>)

    const frameId = this.props.frame.id > 0 ? this.props.frame.id : 0
    return <select className="input"
      defaultValue={frameId}
      disabled={this.props.frame.is_template && !this.props.is_edit_template_mode}
      onChange={this.handleTemplateChange}>
      {options}
    </select>
  }

  getUsedBy (): HTMLElement[] {
    let uses = null
    if (!this.props.frame.used_by) {
      return null
    }
    uses = [
      <span key={'used_by_title'}>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__TEMPLATE_USED_BY}:</span>
    ]

    let counter = 1
    const max_elements = 4
    this.props.frame.used_by.forEach(usage => {
      if (counter > max_elements) {
        return
      }
      uses.push(<a key={'used_by_' + usage.title} href={usage.url} target='_blank'>
        {usage.title + ' (' + usage.type_title + ')'}
      </a>)
      counter++
    })

    if (this.props.frame.used_by.length > max_elements) {
      uses.push(<span key={'used_by_oversize'}>
        {'+ ' + (this.props.frame.used_by.length - max_elements) + ' ' + this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__OTHER}
      </span>)
    }
    return uses
  }

  render () {
    if (!this.props.is_editable) {
      return <div>&nbsp;</div>
    }

    const trs = []
    if (this.props.templates) {
      // render templates
      const select = this.getGridTemplates()
      const uses = this.getUsedBy()
      trs.push(<tr key="use_template">
        <td>
          {this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__USE_TEMPLATE}
        </td>
        <td>
          {select}
        </td>
        <td id='template_used_by'>
          {uses}
        </td>
      </tr>)
    }

    // template options
    if (this.props.frame.is_template && !this.props.is_edit_template_mode) {
      trs.push(<tr key="template_options">
        <td>&nbsp;</td>
        <td>
          <button id="setup_edit_template" type="button" className="button" onClick={this.handleEditTemplate}>
            {this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__EDIT_TEMPLATE}
          </button>
        </td>
        <td>
          <button id="setup_copy_template" type="button" className="button" onClick={this.handleCopyTemplate}>
            {this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__CREATE_TEMPLATE_COPY}
          </button>
        </td>
      </tr>)
    }

    // initial options
    if (this.props.is_initial_mode) {
      const options = [
        <option value="l_r" key="l_r">{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGNMENT_L-R']}</option>,
        <option value="t_b" key="t_b">{this.props.language['LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGNMENT_T-B']}</option>,
        <option value="" key="no">{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGNMENT_NO}</option>
      ]
      const select = <select id="auto_create_direction" className="input" defaultValue={'l_r'} onChange={this.handleDirectionChange}>{options}</select>

      trs.push(<tr key="auto_assignment">
        <td>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__AUTO_ASSIGNMENT}</td>
        <td colSpan={2}>{select}</td>
      </tr>)

      trs.push(<tr key="generate_frame">
        <td>&nbsp;</td>
        <td colSpan={2}>
          <button id="setup_generate_frame" type="button" className="btn" onClick={this.handleGenerateFrame}>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__GENERATE_ASSIGNMENT}</button>
        </td>
      </tr>)
    }

    return <div className="layout_setup">
      <div>
        <table id="layout_setup">
          <thead className="gradient">
            <tr>
              <td colSpan={3}>
                {this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__FRAME_PROPERTIES}
              </td>
            </tr>
          </thead>
          <tbody>
            <tr>
              <td>
                {this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__DESCRIPTION}
              </td>
              <td colSpan={2}>
                <input type='text'
                  // id='layout_description'
                  className='input'
                  onChange={this.handleTitleChange}
                  value={this.props.frame.title}
                  disabled={!this.props.is_editable || (this.props.frame.is_template && !this.props.is_edit_template_mode)} />
              </td>
            </tr>
            <tr>
              <td>
                {this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__IS_TEMPLATE}
              </td>
              <td colSpan={2}>
                <input type='checkbox'
                  className='input'
                  onChange={this.handleIsTemplateChange}
                  defaultChecked={this.props.frame.is_template}
                  disabled={!this.props.is_editable || this.props.frame.is_template} />
              </td>
            </tr>
            {trs}
          </tbody>
        </table>
      </div>
    </div>
  }
}
