import * as React from 'react'

import { loadAjax } from '../api/ajax'
import { ConnectionRowProps, ConnectionTable } from './components/ConnectionTable'
import { MatchTable, DataProps } from './components/MatchTable'
import { Select } from './components/Select'

export interface MapperComposeProps {
    ajax_url: string,
    spinner_url: string,
    session: string,
    obj_id: number,
    language: {[key: string]: string},
    object_types: {[key: number]: string},
    connector_types: {[key: number]: string},
}

interface MapperComposeState {
    ref_obj_id: number,
    data: DataProps[],
    connectionData: ConnectionRowProps[],
    selected_object_type: number,
    objects: {[key: number]: string},
    is_loading: boolean
}

export class MapperCompose extends React.Component<MapperComposeProps, MapperComposeState> {
  constructor (props: MapperComposeProps) {
    super(props)
    this.state = {
      ref_obj_id: null,
      data: [],
      connectionData: [],
      selected_object_type: null,
      objects: [],
      is_loading: false
    }
    this.handleObjectTypeSelection = this.handleObjectTypeSelection.bind(this)
    this.handleChange = this.handleChange.bind(this)
    this.handleAssignConnector = this.handleAssignConnector.bind(this)
    this.handleAddConnector = this.handleAddConnector.bind(this)
    this.handleAddAllConnectors = this.handleAddAllConnectors.bind(this)
    this.loadObjectsByType = this.loadObjectsByType.bind(this)
    this.loadConnectors = this.loadConnectors.bind(this)
    this.loadObject = this.loadObject.bind(this)
  }

  componentDidMount () {
    this.loadConnectors()
    this.loadObjectsByType()
  }

  handleObjectTypeSelection (type: number) {
    this.setState((prevState) => ({
      ...prevState,
      selected_object_type: type,
      is_loading: true
    }), () => {
      this.loadObjectsByType()
    })
  }

  handleChange (id: number) {
    this.setState((prevState) => ({
      ...prevState,
      ref_obj_id: id
    }))
    this.loadObject(id)
  }

  handleAssignConnector (pin_id: number, connector_id: number) {
    const url = `${this.props.ajax_url}&func=assign-connector&pin_id=${pin_id}&connector_id=${connector_id}&obj_id=${this.props.obj_id}`
    loadAjax(url, this.props.session)
      .then(() => {
        const obj_id = this.state.ref_obj_id
        this.loadObject(obj_id)
        this.loadConnectors()
      })
  }

  handleAddConnector (pin_id: number, connector_type_id: number) {
    const url = `${this.props.ajax_url}&func=add-connector&obj_id=${this.props.obj_id}&pin_id=${pin_id}&connector_type_id=${connector_type_id}`
    loadAjax(url, this.props.session)
      .then(() => {
        const obj_id = this.state.ref_obj_id
        this.loadObject(obj_id)
        this.loadConnectors()
      })
  }

  handleAddAllConnectors (connector_type_id: number) {
    const url = `${this.props.ajax_url}&func=add-all-connectors&obj_id=${this.props.obj_id}&ref_obj_id=${this.state.ref_obj_id}&connector_type_id=${connector_type_id}`
    loadAjax(url, this.props.session)
      .then(() => {
        const obj_id = this.state.ref_obj_id
        this.loadObject(obj_id)
        this.loadConnectors()
      })
  }

  loadObjectsByType () {
    const url = `${this.props.ajax_url}&func=get-objects&obj_type_id=${this.state.selected_object_type}&obj_id=${this.props.obj_id}`
    loadAjax(url, this.props.session)
      .then(res => {
        const objects = { ...res.data }
        this.setState((prevState) => ({
          ...prevState,
          objects: objects,
          is_loading: false
        }))
      })
  }

  loadConnectors () {
    const url = `${this.props.ajax_url}&func=get-connectors&obj_id=${this.props.obj_id}`
    loadAjax(url, this.props.session)
      .then(res => {
        const values = { ...res.data }
        const listing:ConnectionRowProps[] = []
        for (const key in values) {
          if (values.hasOwnProperty(key)) {
            listing.push({ ...values[key] })
          }
        }

        this.setState((prevState) => ({
          ...prevState,
          connectionData: listing
        }))
      })
  }

  loadObject (obj_id: number) {
    const url = `${this.props.ajax_url}&func=get-object&obj_id=${this.props.obj_id}&ref_obj_id=${obj_id}`
    this.setState((prevState) => ({
      ...prevState,
      is_loading: true
    }), () => {
      loadAjax(url, this.props.session)
        .then(res => {
          const values = { ...res.data }
          const listing:DataProps[] = []
          for (const key in values) {
            if (values.hasOwnProperty(key)) {
              listing.push({ ...values[key] })
            }
          }

          this.setState((prevState) => ({
            ...prevState,
            data: listing,
            is_loading: false
          }))
        })
    })
  }

  render () {
    let spinner = null
    if (this.state.is_loading) {
      spinner = <img src={this.props.spinner_url} alt="..." />
    }

    const connectors = {}
    this.state.connectionData.forEach(c => {
      connectors[c.connector_id] = c.connector_title + ' (' + c.connector_type_title + ')'
    })

    return <div id="pin_mapper">
      <fieldset className="overview">
        <legend>
          <span>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_CONNECTED_PINS}</span>
        </legend>
        <div className="simple-listing">
          <ConnectionTable language={this.props.language}
            data={this.state.connectionData} />
        </div>
      </fieldset>

      <fieldset className="overview">
        <legend>
          <span>{this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_MATCHING_PINS}</span>
        </legend>
        <div className="simple-listing">
          <Select opts={this.props.object_types} onChange={this.handleObjectTypeSelection} />
          <Select opts={this.state.objects} onChange={this.handleChange} disabled={this.state.is_loading} />
          {spinner}
          <MatchTable language={this.props.language}
            data={this.state.data}
            connector_types={this.props.connector_types}
            connectors={connectors}
            onAssignConnector={this.handleAssignConnector}
            onAddConnector={this.handleAddConnector}
            onAddAllConnectors={this.handleAddAllConnectors} />
        </div>
      </fieldset>
    </div>
  }
}
