import * as React from 'react'
import { loadAjax } from '../api/ajax'
import { SelectableTable } from './SelectableTable'

export interface ParkingDeckComposeProps {
  ajax_url: string,
  session: string,
  language: {[key: string]: string},
  is_editable: boolean,
  is_merge_view: boolean,
  local_object: CommonObject,
  parked_object: CommonObject,
  local_connectors: Connector[],
  parked_connectors: Connector[],
}

interface ParkingDeckComposeState {
  local_connectors: Connector[],
  parked_connectors: Connector[],
  parked_object: CommonObject,
  is_loading: boolean,
  selected_local_connector_id: number,
  selected_parked_connector_id: number,
}

export interface CommonObject {
  obj_id: number,
  obj_title: string,
  obj_type_title: string,
}

export interface Connector {
  connector_id: number,
  connector_title: string,
  connector_type: number,
  connection: Connection,
}

export interface Connection {
  object_type_title: string,
  object_title: string,
  connector_title: string,
  connector_url: string,
}

export class ParkingDeckCompose extends React.Component<ParkingDeckComposeProps, ParkingDeckComposeState> {
  constructor (props: ParkingDeckComposeProps) {
    super(props)
    this.state = {
      local_connectors: this.props.local_connectors,
      parked_connectors: this.props.parked_connectors,
      parked_object: this.props.parked_object,
      is_loading: false,
      selected_local_connector_id: null,
      selected_parked_connector_id: null
    }
    this.handleTitleChange = this.handleTitleChange.bind(this)
    this.handlePark = this.handlePark.bind(this)
    this.handleParkLinkConnections = this.handleParkLinkConnections.bind(this)
    this.handleParkPatchConnections = this.handleParkPatchConnections.bind(this)
    this.handleAssign = this.handleAssign.bind(this)
    this.handleOverwrite = this.handleOverwrite.bind(this)
    this.handleSelectionChange = this.handleSelectionChange.bind(this)
    this._handleConnect = this._handleConnect.bind(this)
    this._parkByUrl = this._parkByUrl.bind(this)
    this._parkConnectionsByType = this._parkConnectionsByType.bind(this)
    this._updateConnectors = this._updateConnectors.bind(this)
  }

  componentDidMount () {
    // setup initial selections
    let selectedLocalConnector = null
    let selectedParkedConnector = null
    if (this.props.is_merge_view) {
      const firstParkedConnector = this.state.parked_connectors.length > 0 ? this.state.parked_connectors[0] : null
      selectedParkedConnector = firstParkedConnector ? firstParkedConnector.connector_id : null

      // try to find matching local connector by name
      this.state.local_connectors.forEach(c => {
        if (firstParkedConnector && c.connector_title == firstParkedConnector.connector_title && c.connector_type == firstParkedConnector.connector_type) {
          selectedLocalConnector = c.connector_id
        }
      })
    } else {
      const firstLocalConnector = this.state.local_connectors.length > 0 ? this.state.local_connectors[0] : null
      selectedLocalConnector = firstLocalConnector ? firstLocalConnector.connector_id : null
    }
    this.setState((prevState) => ({
      ...prevState,
      selected_local_connector_id: selectedLocalConnector,
      selected_parked_connector_id: selectedParkedConnector
    }))
  }

  handleTitleChange (title: string) {
    if (title.length == 0) {
      return
    }

    const parked_object = this.state.parked_object
    parked_object.obj_title = title
    this.setState((prevState) => ({
      ...prevState,
      parked_object: parked_object
    }))

    if (!parked_object.obj_id) {
      // parked object was not saved yet
      return
    }

    // change title
    const url = `${this.props.ajax_url}&func=change-title&obj_id=${this.state.parked_object.obj_id}&title=${this.state.parked_object.obj_title}`
    loadAjax(url, this.props.session).then()
  }

  handleSelectionChange (side: string, connector_id: number) {
    const prop = side == 'local' ? 'selected_local_connector_id' : 'selected_parked_connector_id'
    this.setState((prevState) => ({
      ...prevState,
      [prop]: connector_id
    }))
  }

  handlePark () {
    const connector_id = this.state.selected_local_connector_id
    const url = `${this.props.ajax_url}&func=park&connector_id=${connector_id}`
    this._parkByUrl(url)
  }

  handleParkLinkConnections () {
    this._parkConnectionsByType(1)
  }

  handleParkPatchConnections () {
    this._parkConnectionsByType(2)
  }

  _parkConnectionsByType (type: number) {
    const connectors = []
    this.state.local_connectors.forEach(c => {
      if (c.connector_type == type && c.connection.connector_title) {
        connectors.push(c.connector_id)
      }
    })

    const connector_ids = connectors.join(',')
    const url = `${this.props.ajax_url}&func=park_all&connector_ids=${connector_ids}`
    this._parkByUrl(url)
  }

  _parkByUrl (url: string) {
    this.setState((prevState) => ({
      ...prevState,
      is_loading: true
    }))

    if (this.state.parked_object.obj_id) {
      url += `&obj_id=${this.state.parked_object.obj_id}`
    }
    if (this.state.parked_object.obj_title) {
      url += `&title=${this.state.parked_object.obj_title}`
    }
    loadAjax(url, this.props.session)
      .then(res => {
        if (res.success == false) {
          this.setState((prevState) => ({
            ...prevState,
            is_loading: false
          }))
          return;
        }

        const returned_object = { ...res.data }
        this._updateConnectors(returned_object)
      })
      .catch(() => {
        this.setState((prevState) => ({
          ...prevState,
          is_loading: false
        }))
      })
  }

  handleAssign () {
    this._handleConnect('assign')
  }

  handleOverwrite () {
    this._handleConnect('overwrite')
  }

  _handleConnect (mode: string) {
    const parked_connector_id = this.state.selected_parked_connector_id
    const local_connector_id = this.state.selected_local_connector_id
    this.setState((prevState) => ({
      ...prevState,
      is_loading: true
    }))

    const url = `${this.props.ajax_url}&func=connect&mode=${mode}&parked_connector_id=${parked_connector_id}&local_connector_id=${local_connector_id}`
    loadAjax(url, this.props.session)
      .then(res => {
        if (res.success == false) {
          this.setState((prevState) => ({
            ...prevState,
            is_loading: false
          }))
          return;
        }

        const returned_object = { ...res.data }
        this._updateConnectors(returned_object)
      })
      .catch(() => {
        this.setState((prevState) => ({
          ...prevState,
          is_loading: false
        }))
      })
  }

  _updateConnectors (returned_object) {
    // setup parked object
    const parked_object = this.state.parked_object
    parked_object.obj_id = returned_object.obj_id
    parked_object.obj_title = returned_object.obj_title

    const local_connectors = returned_object.local_connectors
    const parked_connectors = returned_object.parked_connectors

    // handle selection of next element
    const nextLocalConnector = this._getNextConnector(this.state.local_connectors, this.state.selected_local_connector_id, !this.props.is_merge_view)
    let nextParkedConnector
    if (this.state.selected_parked_connector_id) {
      nextParkedConnector = this._getNextConnector(this.state.parked_connectors, this.state.selected_parked_connector_id)
    }

    this.setState((prevState) => ({
      ...prevState,
      parked_object: parked_object,
      local_connectors: local_connectors,
      parked_connectors: parked_connectors,
      is_loading: false,
      selected_local_connector_id: nextLocalConnector ? nextLocalConnector.connector_id : null,
      selected_parked_connector_id: nextParkedConnector ? nextParkedConnector.connector_id : null
    }))
  }

  _getNextConnector (connectors: Connector[], selected_connector_id, withConnection = true): Connector {
    let nextConnector = null
    let useNext = false
    let connectorType = null
    connectors.forEach(c => {
      if (useNext && c.connector_type == connectorType) {
        if (withConnection && !c.connection.connector_title) {
          return
        }
        nextConnector = c
        useNext = false
      }
      if (c.connector_id == selected_connector_id) {
        useNext = true
        connectorType = c.connector_type
      }
    })
    return nextConnector
  }

  render () {
    const buttons = []
    const local_connectors = this.state.local_connectors.filter(c => c.connector_id == this.state.selected_local_connector_id)
    const local_connector = local_connectors.length > 0 ? local_connectors[0] : null
    if (local_connector) {
      if (this.props.is_merge_view) {
        if (this.state.selected_parked_connector_id) {
          if (local_connector.connection.connector_title) {
            buttons.push(<button key="btn_overwrite"
              type="button"
              className="btn"
              disabled={this.state.is_loading}
              onClick={this.handleOverwrite}>
              {'<-'} {this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__OVERWRITE_CONNECTOR}
            </button>)
          } else {
            buttons.push(<button key="btn_assign"
              type="button"
              className="btn"
              disabled={this.state.is_loading}
              onClick={this.handleAssign}>
              {'<-'} {this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__ASSIGN_CONNECTOR}
            </button>)
          }
        }
      } else {
        if (local_connector.connection.connector_title) {
          buttons.push(<button key="btn_park"
            type="button"
            className="btn"
            disabled={this.state.is_loading}
            onClick={this.handlePark}>
            {this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__PARK_CONNECTOR} {'->'}
          </button>)
        }

        // check connector types and add buttons if necessary
        let has_link_connections = false
        let has_patch_connections = false
        this.state.local_connectors.forEach(c => {
          if (c.connector_type == 1 && c.connection.connector_title) {
            has_link_connections = true
          }
          if (c.connector_type == 2 && c.connection.connector_title) {
            has_patch_connections = true
          }
        })

        if (has_link_connections) {
          buttons.push(<button key="btn_park_links"
            type="button"
            className="btn"
            disabled={this.state.is_loading}
            onClick={this.handleParkLinkConnections}>
            {this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__PARK_ALL_LINK_CONNECTORS} {'->'}
          </button>)
        }

        if (has_patch_connections) {
          buttons.push(<button key="btn_park_patches"
            type="button"
            className="btn"
            disabled={this.state.is_loading}
            onClick={this.handleParkPatchConnections}>
            {this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__PARK_ALL_PATCH_CONNECTORS} {'->'}
          </button>)
        }
      }
    }

    return <div id="parking_deck_container" className="simple-listing">
      <div>
        <SelectableTable language={this.props.language}
          title={this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__LOCAL_OBJECT}
          object={this.props.local_object}
          isEditable={false}
          side="local"
          rows={this.state.local_connectors}
          selected_connector={this.state.selected_local_connector_id}
          onSelectionChange={this.handleSelectionChange} />
      </div>
      <div>
        {buttons}
      </div>
      <div>
        <SelectableTable language={this.props.language}
          title={this.props.language.LC__CMDB__CATG__SMVIEW_PATCH__PARKING_DECK_OBJECT}
          object={this.props.parked_object}
          isEditable={!this.props.is_merge_view}
          side="parked"
          rows={this.state.parked_connectors}
          selected_connector={this.state.selected_parked_connector_id}
          onSelectionChange={this.handleSelectionChange}
          onTitleChange={this.handleTitleChange} />
      </div>
    </div>
  }
}
