<?php

/**
 * i-doit
 *
 * Connector Template DAO for SM-VIEW Patch.
 *
 * @package    idoit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_cmdb_dao_smview_patch_connector_template extends isys_cmdb_dao_smview_patch_abstract
{
    
    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_patch_connector_template__id'                => 'id',
        'isys_catg_smview_patch_connector_template__grid_template_id'  => 'grid_template_id',
        'isys_catg_smview_patch_connector_template__connector_title'   => 'connector_title',
        'isys_catg_smview_patch_connector_template__connector_type'    => 'connector_type',
        'isys_catg_smview_patch_connector_template__connector_type_id' => 'connector_type_id',
        'isys_catg_smview_patch_connector_template__title'             => 'title',
        'isys_catg_smview_patch_connector_template__number'            => 'number',
        'isys_catg_smview_patch_connector_template__row_title'         => 'row_title',
        'isys_catg_smview_patch_connector_template__pos_x'             => 'pos_x',
        'isys_catg_smview_patch_connector_template__pos_y'             => 'pos_y',
        'isys_catg_smview_patch_connector_template__rotation'          => 'rotation',
        'isys_catg_smview_patch_connector_template__label_orientation' => 'label_orientation',
        'isys_catg_smview_patch_connector_template__status'            => 'status',
        'isys_catg_smview_patch_connector_template__angle'             => 'angle',
    );
    
    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_patch_connector_template';
    
    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_patch_connector_template__id';
    
    /**
     * Validation of connector template positions.
     *
     * @var boolean
     */
    private $_position_validation_enabled = true;
    
    /**
     * Disable validation of connector template positions.
     *
     * Validation must be handled individually before.
     */
    public function disable_position_validation()
    {
        $this->_position_validation_enabled = false;
    }

    /**
     * Create a new object.
     *
     * @param  array $params
     * @param  array $optionalParams
     * @return int|false
     * @throws isys_exception_validation
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function create_object($params, $optionalParams = [])
    {
        if (empty($params['grid_template_id'])) {
            throw new isys_exception_validation('grid_template_id is required', []);
        }
        
        if (empty($params['pos_x']) || empty($params['pos_y'])) {
            throw new isys_exception_validation('pos_x and pos_y are required', []);
        }
        
        if ($this->_position_validation_enabled) {
            $p_angle = !empty($params['angle']) ? $params['angle'] : $optionalParams['angle'];
            $angle = ! empty($p_angle) ? $p_angle : 'F';

            $grid_template_id = $params['grid_template_id'];
            $connectors = $this->filter_objects(
                array(
                'grid_template_id' => $grid_template_id,
                'angle'            => $angle,
                )
            );

            $pos_x = $params['pos_x'];
            $pos_y = $params['pos_y'];
            foreach ($connectors as $connector) {
                if ($connector['pos_x'] == $pos_x && $connector['pos_y'] == $pos_y) {
                    throw new isys_exception_validation('there is already a connector assigned to this position', []);
                }
            }
        }
        return parent::create_object($params, $optionalParams);
    }

    /**
     * Update an existing object.
     *
     * @param  int $id
     * @param  array $params
     * @param  array $optionalParams
     * @return boolean
     * @throws isys_exception_validation
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function update_object($id, $params, $optionalParams = [])
    {
        if (! $this->_position_validation_enabled) {
            return parent::update_object($id, $params, $optionalParams);
        }
        
        $obj = $this->get_object('id', $id);
        $grid_template_id = $obj['grid_template_id'];

        $angle = !empty($optionalParams['angle']) ? $optionalParams['angle'] : $obj['angle'];
        $connectors = $this->filter_objects(
            array(
            'grid_template_id' => $grid_template_id,
            'angle'            => $angle,
            )
        );

        $pos_x = !empty($optionalParams['pos_x']) ? $optionalParams['pos_x'] : $obj['pos_x'];
        $pos_y = !empty($optionalParams['pos_y']) ? $optionalParams['pos_y'] : $obj['pos_y'];
        foreach ($connectors as $connector) {
            if ($connector['id'] == $id) {
                continue;
            }

            if ($connector['pos_x'] == $pos_x && $connector['pos_y'] == $pos_y) {
                throw new isys_exception_validation('there is already a connector assigned to this position', []);
            }
        }
        return parent::update_object($id, $params, $optionalParams);
    }

    /**
     * Delete all connectors assigned to a grid template.
     *
     * @param  int $grid_template_id
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function delete_all_by_grid_template_id($grid_template_id)
    {
        $connectors = $this->filter_objects(array('grid_template_id' => $grid_template_id));
        foreach ($connectors as $connector) {
            $this->delete_object($connector['id']);
        }
    }
}
