<?php

/**
 * i-doit
 *
 * Grid DAO for SM-VIEW Patch.
 *
 * @package    idoit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_cmdb_dao_smview_patch_grid extends isys_cmdb_dao_smview_patch_abstract
{
    
    /**
     * Mapping of database and API keys
     *
     * @var array
     */
    protected $db_to_api_map = array(
        'isys_catg_smview_patch_grid_list__id'               => 'id',
        'isys_catg_smview_patch_grid_list__isys_obj__id'     => 'obj_id',
        'isys_catg_smview_patch_grid_list__description'      => 'description',
        'isys_catg_smview_patch_grid_list__grid_template_id' => 'grid_template_id',
    );
    
    /**
     * Name of the table.
     *
     * @var string
     */
    protected $table_name = 'isys_catg_smview_patch_grid_list';
    
    /**
     * Name of the primary key field.
     *
     * @var string
     */
    protected $primary_key_field = 'isys_catg_smview_patch_grid_list__id';

    /**
     * Get assigned connectors by object ID.
     *
     * @param  int $obj_id
     * @return array
     * @throws isys_exception_database
     */
    public function get_connectors_by_obj_id($obj_id)
    {
        $connector_dao = new isys_cmdb_dao_category_g_connector(isys_application::instance()->database);
        $picture_dao = new isys_cmdb_dao_smview_patch_picture(isys_application::instance()->database);
        
        $l_data = $connector_dao->get_data(
            null,
            $obj_id,
            ' AND isys_catg_connector_list.isys_catg_connector_list__type = ' . C__CONNECTOR__OUTPUT,
            null,
            defined_or_default('C__RECORD_STATUS__NORMAL')
        );
        
        $ports = [];
        while ($row = $l_data->get_row()) {
            $ct_id = $row['isys_catg_connector_list__isys_connection_type__id'];
            
            $picture_row = $picture_dao->get_object('id', $ct_id);
            $url = $picture_dao->get_file_url($picture_row['obj_id']);
            $ports[] = [
                'id'    => $row['isys_catg_connector_list__id'],
                'title' => $row['isys_catg_connector_list__title'],
                'type'  => $row['isys_catg_connector_list__type'],
                'url'   => $url,
            ];
        }
        return $ports;
    }

    /**
     * Assign two connectors.
     *
     * @param  int $connection_a
     * @param  int $connection_b
     * @param  string|null $cable_name
     * @param  boolean $add_to_exiting_cable
     * @return int
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function assign_connectors($connection_a, $connection_b, $cable_name = null, $add_to_exiting_cable = false)
    {
        // remove current cable connection of the connectors
        $this->detach_cable($connection_a);
        $this->detach_cable($connection_b);

        $l_dao = isys_cmdb_dao_cable_connection::instance(isys_application::instance()->database);

        $l_cable_object_id = null;
        if ($add_to_exiting_cable && ! empty($cable_name)) {
            $l_cable_object_id = $l_dao->get_obj_id_by_title($cable_name, defined_or_default('C__OBJTYPE__CABLE'));
        }

        if (! $l_cable_object_id) {
            $l_cable_object_id = isys_cmdb_dao_cable_connection::add_cable();

            // update cable name
            if (! empty($cable_name)) {
                $cable_name = $this->convert_sql_text($cable_name);
                $sql = "UPDATE isys_obj SET isys_obj__title = $cable_name WHERE isys_obj__id = $l_cable_object_id";
                $l_dao->update($sql);
            }
        }

        $l_cable_con = $this->_attach_cable($l_cable_object_id);
        $l_dao->save_connection($connection_a, $connection_b, $l_cable_con, $connection_a);
        return $l_cable_object_id;
    }

    /**
     * Method for detaching a cable connection.
     *
     * @param  int $connector_id
     * @param  boolean $useDeletionStatus
     * @return boolean
     * @throws isys_exception_database
     * @throws isys_exception_cmdb
     */
    public function detach_cable($connector_id, $useDeletionStatus = false)
    {
        $dao = isys_cmdb_dao_cable_connection::instance(isys_application::instance()->database);
        $old_cable_connection = $dao->get_cable_connection_id_by_connector_id($connector_id);
        $cable_obj_id = $dao->get_cable_object_id_by_connection_id($old_cable_connection);

        // remove cable connection
        $return = $dao->delete_cable_connection($old_cable_connection);

        // remove cable
        if (! empty($cable_obj_id)) {
            // check other connections
            $connectionCount = $dao->get_cable_connection_by_cable_id($cable_obj_id)->count();
            if ($connectionCount == 0) {
                $cable_status = isys_settings::get(isys_module_smview_patch::CABLE_STATUS_AFTER_DELETION, null);
                if (! $useDeletionStatus || empty($cable_status)) {
                    // delete cable if status is empty
                    $dao->delete_object($cable_obj_id);
                } else {
                    // set cable status and do not delete cable
                    $dao->set_object_cmdb_status($cable_obj_id, $cable_status);
                }
            }
        }
        return $return;
    }

    /**
     * Method for creating a cable connection.
     *
     * @param  int $cable_obj_id
     * @return mixed
     */
    private function _attach_cable($cable_obj_id)
    {
        $dao = isys_cmdb_dao_cable_connection::instance(isys_application::instance()->database);
        if (isys_tenantsettings::get('cmdb.cable.change-cmdb-status-on-attach', 1)) {
            $dao->set_object_cmdb_status($cable_obj_id, defined_or_default('C__CMDB_STATUS__IN_OPERATION'));
        }
        return $dao->add_cable_connection($cable_obj_id);
    }

    /**
     * Assign virtual connectors.
     *
     * @param  int $connection_a
     * @param  int $connection_b
     * @param  string|null $cable_name
     * @param  boolean $add_to_exiting_cable
     * @return boolean|null
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function assign_virtual_connectors($connection_a, $connection_b, $cable_name = null, $add_to_exiting_cable = false)
    {
        // get connectors
        $connector_dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        $connectorA = $connector_dao->get_data($connection_a)->get_row();
        $connectorB = $connector_dao->get_data($connection_b)->get_row();
        return $this->_handle_create_virtual_connection($connectorA, $connectorB, 'new', $cable_name, $add_to_exiting_cable);
    }

    /**
     * Detach virtual connector.
     *
     * @param  int $connector_id
     * @return boolean|null
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function detach_virtual_connector($connector_id)
    {
        // get connectors
        $connector_dao = new isys_cmdb_dao_category_g_connector($this->m_db);
        $connectorA = $connector_dao->get_data($connector_id)->get_row();
        $connectorB = $connector_dao->get_data($connectorA['con_connector'])->get_row();
        return $this->_handle_create_virtual_connection($connectorA, $connectorB, 'remove');
    }

    /**
     * Detach virtual connector.
     *
     * @param  array $connectorA
     * @param  array $connectorB
     * @param  string $status
     * @param  string|null $cable_name
     * @param  boolean $add_to_exiting_cable
     * @return boolean|null
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _handle_create_virtual_connection($connectorA, $connectorB, $status, $cable_name = null, $add_to_exiting_cable = false)
    {
        // remove current connections
        $this->_remove_virtual_connector($connectorA['isys_catg_connector_list__id']);
        $this->_remove_virtual_connector($connectorB['isys_catg_connector_list__id']);

        // set CMDB status of obsolete assigned cable
        $p_cmdb_status = isys_settings::get(isys_module_smview_patch::CABLE_STATUS_PLANNING, null);
        $this->_set_cmdb_status_of_assigned_cables($connectorA['isys_catg_connector_list__id'], $connectorB['isys_catg_connector_list__id'], $p_cmdb_status);

        // add new connection
        $virtual_connection_dao = new isys_cmdb_dao_smview_patch_virtual_connection($this->m_db);
        return $virtual_connection_dao->create_object([
            'from_obj_id'           => $connectorA['isys_catg_connector_list__isys_obj__id'],
            'from_connector_id'     => $connectorA['isys_catg_connector_list__id'],
            'to_obj_id'             => $connectorB['isys_catg_connector_list__isys_obj__id'],
            'to_connector_id'       => $connectorB['isys_catg_connector_list__id'],
            'cable_name'            => $cable_name,
            'add_to_existing_cable' => $add_to_exiting_cable,
            'status'                => $status,
        ]);
    }

    /**
     * Activate virtual connection.
     *
     * @param  int $connector_a
     * @param  int $connector_b
     *
     * @throws isys_exception_cmdb
     * @throws isys_exception_database
     * @throws isys_exception_dao
     */
    public function activate_virtual_connection($connector_a, $connector_b)
    {
        $virtual_connection_dao = new isys_cmdb_dao_smview_patch_virtual_connection($this->m_db);
        $virtual_connectorA = $virtual_connection_dao->get_object('from_connector_id', $connector_a);
        if (! $virtual_connectorA) {
            $virtual_connectorA = $virtual_connection_dao->get_object('to_connector_id', $connector_a);
        }
        $virtual_connectorB = $virtual_connection_dao->get_object('from_connector_id', $connector_b);
        if (! $virtual_connectorB) {
            $virtual_connectorB = $virtual_connection_dao->get_object('to_connector_id', $connector_b);
        }

        if (empty($virtual_connectorA) || empty($virtual_connectorB)) {
            throw new isys_exception_cmdb('could not find both virtual connectors');
        }
        $this->_handle_activate_virtual_connection($virtual_connectorA, $connector_a, $connector_b);
    }

    /**
     * Activate virtual connections of a device.
     *
     * @param  int $obj_id
     * @throws isys_exception_cmdb
     * @throws isys_exception_database
     * @throws isys_exception_dao
     */
    public function activate_virtual_connections($obj_id)
    {
        $virtual_connection_dao = new isys_cmdb_dao_smview_patch_virtual_connection($this->m_db);
        $virtual_connectors = $virtual_connection_dao->filter_objects(['from_obj_id' => $obj_id]);
        foreach ($virtual_connectors as $vc) {
            $this->_handle_activate_virtual_connection($vc, $vc['from_connector_id'], $vc['to_connector_id']);
        }
        $virtual_connectors = $virtual_connection_dao->filter_objects(['to_obj_id' => $obj_id]);
        foreach ($virtual_connectors as $vc) {
            $this->_handle_activate_virtual_connection($vc, $vc['from_connector_id'], $vc['to_connector_id']);
        }
    }

    /**
     * Handle virtual connection.
     *
     * @param  array $virtual_connector
     * @param  int $connector_a
     * @param  int $connector_b
     * @return boolean|null
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _handle_activate_virtual_connection($virtual_connector, $connector_a, $connector_b)
    {
        switch ($virtual_connector['status']) {
            case 'new':
                $this->assign_connectors($connector_a, $connector_b, $virtual_connector['cable_name'], $virtual_connector['add_to_existing_cable']);
                $this->_activate_manually_end_device($connector_a);
                $this->_activate_manually_end_device($connector_b);
                break;
            case 'remove':
                $this->detach_cable($connector_a);
                break;
            default:
                throw new isys_exception_cmdb('unknown virtual connection status');
        }
        return $this->remove_virtual_connection($connector_a, $connector_b);
    }

    /**
     * Remove virtual connection.
     *
     * @param  int $connector_a
     * @param  int $connector_b
     *
     * @return boolean|null
     * @throws isys_exception_cmdb
     * @throws isys_exception_database
     * @throws isys_exception_dao
     */
    public function remove_virtual_connection($connector_a, $connector_b)
    {
        $virtual_connection_dao = new isys_cmdb_dao_smview_patch_virtual_connection($this->m_db);
        $virtual_connectorA = $virtual_connection_dao->get_object('from_connector_id', $connector_a);
        if ($virtual_connectorA) {
            if ($virtual_connectorA['to_connector_id'] != $connector_b) {
                throw new isys_exception_cmdb('connectors are not connected with each other');
            }

        } else {
            $virtual_connectorA = $virtual_connection_dao->get_object('to_connector_id', $connector_a);
            if (empty($virtual_connectorA)) {
                throw new isys_exception_cmdb('could not find virtual connector');
            }

            if ($virtual_connectorA['from_connector_id'] != $connector_b) {
                throw new isys_exception_cmdb('connectors are not connected with each other');
            }
        }

        // reset CMDB status of re-activated assigned cable
        $this->_set_cmdb_status_of_assigned_cables($connector_a, $connector_b);

        return $this->_remove_virtual_connector($connector_a);
    }

    /**
     * Remove virtual connector.
     *
     * @param  int $connector_id
     * @return boolean|null
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _remove_virtual_connector($connector_id)
    {
        $virtual_connection_dao = new isys_cmdb_dao_smview_patch_virtual_connection($this->m_db);
        $virtual_connectorA = $virtual_connection_dao->get_object('from_connector_id', $connector_id);
        if ($virtual_connectorA) {
            // remove current connection
            return $virtual_connection_dao->delete_object($virtual_connectorA['id']);
        }
        $virtual_connectorB = $virtual_connection_dao->get_object('to_connector_id', $connector_id);
        if ($virtual_connectorB) {
            // remove current connection
            return $virtual_connection_dao->delete_object($virtual_connectorB['id']);
        }
        return null;
    }

    /**
     * Activate manually end device by connector ID.
     *
     * @param  int $connector_id
     * @return boolean|null
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     */
    private function _activate_manually_end_device($connector_id)
    {
        $l_dao_connector = new isys_cmdb_dao_category_g_connector($this->get_database_component());
        $l_connector = $l_dao_connector->get_data($connector_id)->get_row();

        if ($l_connector['isys_catg_connector_list__type'] == C__CONNECTOR__OUTPUT) {
            // get sibling because this is a patch
            $l_connector = $l_dao_connector->get_data($l_connector["isys_catg_connector_list__isys_catg_connector_list__id"])->get_row();
        }

        if (empty($l_connector["con_connector"])) {
            // connector has no connected object
            return null;
        }

        // get connected connector
        $l_conn = $l_dao_connector->get_data($l_connector["con_connector"])->get_row();

        // get object of this connector
        $obj_id = $l_conn["isys_catg_connector_list__isys_obj__id"];
        if ($obj_id) {
            // activate planned name by rewriting current name property
            $end_device_dao = new isys_cmdb_dao_category_g_smview_patch_end_device(isys_application::instance()->database);
            $end_device = $end_device_dao->get_data(null, $obj_id)->get_row();
            if ($end_device && ! empty($end_device['isys_catg_smview_patch_end_device_list__planned_name'])) {
                // update only if planned name is not empty
                return $end_device_dao->save_data($end_device['isys_catg_smview_patch_end_device_list__id'],
                    [
                        'name'         => $end_device['isys_catg_smview_patch_end_device_list__planned_name'],
                        'planned_name' => '',
                    ]
                );
            }
        }
        return null;
    }

    /**
     * Set CMDB status of possibly assigned cables.
     *
     * @param int  $connectorIdA
     * @param int  $connectorIdB
     * @param null $cmdbStatusId
     *
     * @throws isys_exception_database
     */
    private function _set_cmdb_status_of_assigned_cables($connectorIdA, $connectorIdB, $cmdbStatusId = null)
    {
        if (empty($cmdbStatusId)) {
            $dao = new isys_cmdb_dao_status($this->m_db);
            $statusRow = $dao->get_cmdb_status('C__CMDB_STATUS__IN_OPERATION')->get_row();
            $cmdbStatusId = $statusRow['isys_cmdb_status__id'];
        }

        $cable_connection_dao = new isys_cmdb_dao_cable_connection($this->m_db);
        $cableA = $cable_connection_dao->get_assigned_cable($connectorIdA);
        if (! empty($cableA)) {
            $cable_connection_dao->update_object($cableA, null, null, null, null, null,
                null, null, null, null, null, null, $cmdbStatusId);
        }

        $cableB = $cable_connection_dao->get_assigned_cable($connectorIdB);
        if (! empty($cableB)) {
            $cable_connection_dao->update_object($cableB, null, null, null, null, null,
                null, null, null, null, null, null, $cmdbStatusId);
        }
    }
}
