<?php

/**
 * i-doit
 *
 * UI: global category for SM-VIEW Patch Grid.
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_ui_category_g_smview_patch_pin_mapper extends isys_cmdb_ui_category_global
{
    /**
     * Overwriting this method to set the correct template.
     *
     * @param  isys_cmdb_dao_category $p_cat
     * @throws Exception
     */
    public function init(isys_cmdb_dao_category $p_cat)
    {
        parent::init($p_cat);
        $this->set_template(isys_module_smview_patch::get_tpl_dir() . 'module/cmdb/catg__smview_patch_pin_mapper.tpl');
    }

    /**
     * Processes the UI for the category grid
     *
     * @param isys_cmdb_dao_category $p_cat The corresponding category DAO
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     */
    public function process(isys_cmdb_dao_category $p_cat)
    {
        parent::process($p_cat);
        $this->deactivate_commentary();
        $this->disable_ajax_save();

        $object_types = [];
        if (class_exists('isys_cmdb_dao_smview_cable_pin')) {
            // SM-VIEW cable is installed
            $object_types = $this->_get_pin_object_types();
        }

        // only load with edit permission
        if (! isys_auth_cmdb::instance()->has_rights_in_obj_and_category(\isys_auth::EDIT, $this->m_object_id, $p_cat->get_category_const())) {
            $this->get_template_component()->assign('not_editable', 1);
            return;
        }

        // create JS configuration
        $js_config = [
            'ajax_url'        => isys_helper_link::create_url([C__GET__AJAX => 1]),
            'spinner_url'     => isys_helper_link::get_base() . '/images/ajax-loading.gif',
            'session'         => $_COOKIE['PHPSESSID'],
            'language'        => $this->_get_js_translations(),
            'obj_id'          => $this->m_object_id,
            'object_types'    => $object_types,
            'connector_types' => $this->_get_connector_types(),
        ];
        $this->get_template_component()->assign('js_config', json_encode($js_config));

        isys_component_template_navbar::getInstance()
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__SAVE'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__CANCEL'));
    }

    /**
     * Get all object types, having cable pins.
     *
     * @return array
     * @throws isys_exception_database
     */
    private function _get_pin_object_types()
    {
        $lang = isys_application::instance()->container->get('language');
        $global_dao = new isys_cmdb_dao_category_g_global($this->get_database_component());
        $sql = 'SELECT isys_obj_type__id, isys_obj_type__title FROM isys_obj o
                JOIN isys_obj_type t ON o.isys_obj__isys_obj_type__id = t.isys_obj_type__id
                JOIN isys_catg_smview_cable_pin_list p ON isys_obj__id = isys_catg_smview_cable_pin_list__isys_obj__id
                GROUP BY isys_obj_type__id;';
        $res = $global_dao->retrieve($sql);
        $object_types = [];
        while ($row = $res->get_row()) {
            $type_title = $lang->get($row['isys_obj_type__title']);
            $object_types[$row['isys_obj_type__id']] = $type_title;
        }
        return $object_types;
    }

    /**
     * Get all connector types.
     *
     * @return array
     * @throws isys_exception_database
     */
    private function _get_connector_types()
    {
        $global_dao = new isys_cmdb_dao_category_g_global($this->get_database_component());
        $sql = 'SELECT * FROM isys_connection_type';
        $res = $global_dao->retrieve($sql);
        $connector_types = [];
        while ($row = $res->get_row()) {
            $connector_types[$row['isys_connection_type__id']] = $row['isys_connection_type__title'];
        }
        return $connector_types;
    }

    /**
     * Get translated strings to use with javascript.
     *
     * @return array
     * @throws Exception
     */
    private function _get_js_translations()
    {
        // language templates
        $language_templates = [
            'LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_CONNECTED_PIN',
            'LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_CONNECTED_PINS',
            'LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_MATCHING_PINS',
            'LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_CONNECTOR',
            'LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_PIN',
            'LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_MATCHES',
            'LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_ASSIGNED_TO',
            'LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_ASSIGN_CONNECTOR',
            'LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_ASSIGN_OTHER_CONNECTOR',
            'LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_ADD_CONNECTOR',
            'LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_ADD_ALL_CONNECTORS',
        ];

        // translate
        $language = [];
        $lang = isys_application::instance()->container->get('language');
        foreach ($language_templates as $l) {
            $language[$l] = $lang->get($l);
        }
        return $language;
    }

    /**
     * Fetches category's title from database.
     *
     * @param  isys_cmdb_dao_category &$p_cat Category's DAO
     * @return string  Category's title.
     * @throws Exception
     */
    public function gui_get_title(isys_cmdb_dao_category &$p_cat)
    {
        $lang = isys_application::instance()->container->get('language');
        return $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__PIN_MAPPER_TITLE');
    }
}
