<?php

use idoit\AddOn\AuthableInterface;
use idoit\Module\Patch\DependentObjectManager\AbstractManager;
use idoit\Module\Patch\DependentObjectManager\AnalyzerPresetManager;
use idoit\Module\Patch\DependentObjectManager\ImageMappingManager;
use idoit\Module\Patch\Exchange\GridExporter;
use idoit\Module\Patch\Exchange\GridImporter;
use idoit\Module\Patch\Importer\Excel;
use idoit\Module\Patch\Importer\ImporterException;

/**
 * i-doit
 *
 * SM-VIEW Patch module.
 *
 * @package    idoit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2019 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_module_smview_patch extends isys_module implements AuthableInterface
{
    const DISPLAY_IN_MAIN_MENU = true;
    const DISPLAY_IN_SYSTEM_MENU = false;
    const DEFAULT_SOCKET_TYPE = 'smview_patch.patch_editor.default_socket_type';
    const IMPORT_TEMPLATE_PATH = 'smview_patch.importer.template_path';
    const CABLE_STATUS_AFTER_DELETION = 'smview_patch.patch_editor.cable_status_after_deletion';
    const CABLE_STATUS_PLANNING = 'smview_patch.patch_editor.cable_status_planning';
    const RIGHT_ALIGN_CONNECTOR_TITLE = 'smview_patch.grid_editor.right_align_connector_title';
    const TRACE_RIGHT_PORT_DIRECTLY = 'smview_patch.grid_editor.trace_right_port_directly';

    /**
     * @var bool
     */
    protected static $m_licenced = true;

    /**
     * Variable which holds the database component.
     *
     * @var isys_component_database
     */
    protected $m_db = null;

    /**
     * Variable which holds the template component.
     *
     * @var isys_component_template
     */
    protected $m_tpl = null;
    
    /**
     * Variable which holds the template component.
     *
     * @var isys_component_tree
     */
    protected $m_tree = null;

    /**
     * Get related auth class for module
     *
     * @return isys_auth
     */
    public static function getAuth()
    {
        return isys_auth_smview_patch::instance();
    }

    /**
     * Static method for retrieving the path, to the modules templates.
     *
     * @static
     * @return string
     */
    public static function get_tpl_dir() 
    {
        return __DIR__ . '/templates/';
    }

    /**
     * Represents the name, of the corresponding auth class - "false" means, there is none.
     *
     * @var string
     */
    const AUTH_CLASS_NAME = 'isys_auth_smview_patch';

    /**
     * Method for retrieving the path to the module directory (needed for includes).
     *
     * @static
     * @return string
     */
    public static function get_module_dir() 
    {
        return __DIR__ . DS;
    }
    
    /**
     * Method for retrieving the www-path to the assets directory (needed for the frontend).
     *
     * @static
     * @return  string
     */
    public static function get_assets_www_dir() {
        return isys_application::instance()->www_path . 'src/classes/modules/smview_patch/assets/';
    }

    /**
     * Check if this add-on is licensed or not.
     *
     * @return boolean
     */
    public static function is_licensed()
    {
        if (! class_exists('smview_cmdb_dao_lic')) {
            return true;
        }

        // get complete license status
        $status = smview_cmdb_dao_lic::instance(isys_application::instance()->database)->getLicStatus();
        if (empty($status) || ! isset($status["PATCH"])) {
            return false;
        }
        $licenseInfo = $status["PATCH"];
        if (empty($licenseInfo["expire"])) {
            // no expiration date set
            return true;
        }

        // check expiration date
        $expireDateString = '31-' . $licenseInfo["expire"];
        $expireDate = DateTime::createFromFormat('d-m-Y', $expireDateString);
        $now = time();
        return $expireDate->getTimestamp() > $now;
    }

    /**
     * Get editor colors.
     *
     * @return array
     */
    public static function get_editor_colors()
    {
        $color_setup = [
            'color_patch' => [
                'key'     => 'smview_patch.editor.color_patch',
                'default' => '00FE00',
            ],
            'color_patch_direct' => [
                'key'     => 'smview_patch.editor.color_patch_direct',
                'default' => '61EEFE',
            ],
            'color_link' => [
                'key'     => 'smview_patch.editor.color_link',
                'default' => 'FEFE00',
            ],
            'color_socket' => [
                'key'     => 'smview_patch.editor.color_socket',
                'default' => '808000',
            ],
            'color_cable_pin' => [
                'key'     => 'smview_patch.editor.color_cable_pin',
                'default' => 'B200FF',
            ],
        ];

        $colors = [];
        foreach ($color_setup as $name => $color) {
            $colors[$name] = isys_settings::get($color['key'], $color['default']);
        }
        return $colors;
    }

    /**
     * Initializes the module.
     *
     * @param isys_module_request & $p_req
     * @return isys_module_smview_patch
     */
    public function init(isys_module_request &$p_req) 
    {
        $this->m_db = $p_req->get_database();
        $this->m_tpl = $p_req->get_template();
        $this->m_tree = $p_req->get_menutree();

        return $this;
    }

    /**
     * Start method.
     *
     * @desc   Starts module process
     * @throws isys_exception_database
     * @throws Exception
     */
    public function start() 
    {
        global $g_dirs;
        
        $l_template = $this->m_tpl;
        $l_tree = $this->m_tree;

        # image mapping
        $lang = isys_application::instance()->container->get('language');
        $l_tree->add_node(0, -1, $lang->get("LC__CMDB__CATG__SMVIEW_PATCH__TREE_TITLE"),
            isys_helper_link::create_url(
                [
                    C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_PATCH')
                ]
            ));
        $l_tree->add_node(
            1, 0, $lang->get("LC__CMDB__CATG__SMVIEW_PATCH__IMAGE_MAPPING_TREE_TITLE"),
            isys_helper_link::create_url(
                [
                    C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_PATCH'),
                    C__GET__TREE_NODE => 1,
                    'request'         => 'image_mapping',
                ]
            ),
            '',
            $g_dirs['images'] . 'icons/silk/image_link.png'
        );

        // patch editor colors
        $l_tree->add_node(
            2, 0, $lang->get("LC__CMDB__CATG__SMVIEW_PATCH__EDITOR_COLORS_TREE_TITLE"),
            isys_helper_link::create_url(
                [
                    C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_PATCH'),
                    C__GET__TREE_NODE => 2,
                    'request'         => 'patch_colors',
                ]
            ),
            '',
            $g_dirs['images'] . 'icons/silk/color_swatch.png'
        );

        // grid template overview
        $l_tree->add_node(
            3, 0, $lang->get("LC__CMDB__CATG__SMVIEW_PATCH__GRID_TEMPLATE_TREE_TITLE"),
            isys_helper_link::create_url(
                [
                C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_PATCH'),
                C__GET__TREE_NODE => 3,
                'request'         => 'grid_template_overview',
                ]
            ),
            '',
            $g_dirs['images'] . 'icons/silk/page_copy.png'
        );

        // grid templates
        $grid_template_dao = new isys_cmdb_dao_smview_patch_grid_template($this->m_db);
        $grid_templates = $grid_template_dao->filter_objects(array('is_template' => '1'));
        $i = 4;
        $templates_in_tree = [];
        if (count($grid_templates)) {
            foreach ($grid_templates as $grid_template) {
                $l_tree->add_node(
                    $i, 3, $grid_template['title'],
                    isys_helper_link::create_url(
                        [
                            C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_PATCH'),
                            C__GET__TREE_NODE => $i,
                            'request'         => 'grid_template',
                            'instance_id'     => $grid_template['id'],
                        ]
                    ),
                    '',
                    $g_dirs['images'] . 'icons/silk/page_gear.png'
                );
                $templates_in_tree[$grid_template['id']] = $i;
                $i++;
            }
        }

        $l_tree->add_node(
            $i, 0, $lang->get("LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_TREE_TITLE"),
            isys_helper_link::create_url(
                [
                    C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_PATCH'),
                    C__GET__TREE_NODE => $i,
                    'request'         => 'analyzer',
                ]
            ),
            '',
            $g_dirs['images'] . 'icons/silk/chart_organisation.png'
        );

        if (isys_auth_auth::instance()->is_allowed_to(isys_auth::CREATE, "MODULE/C__MODULE__SMVIEW_PATCH")) {
            $i++;
            $l_tree->add_node(
                $i, 0, $lang->get("LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_TITLE"),
                isys_helper_link::create_url(
                    [
                        C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_PATCH'),
                        C__GET__TREE_NODE => $i,
                        'request'         => 'importer',
                    ]
                ),
                '',
                $g_dirs['images'] . 'icons/silk/database_copy.png'
            );
        }

        if (isys_auth_auth::instance()->is_allowed_to(isys_auth::CREATE, "MODULE/C__MODULE__SMVIEW_PATCH")) {
            $i++;
            $l_tree->add_node(
                $i, 0, $lang->get("LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_PRESET_TREE_TITLE"),
                isys_helper_link::create_url(
                    [
                        C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_PATCH'),
                        C__GET__TREE_NODE => $i,
                        'request'         => 'analyzer_presets',
                    ]
                ),
                '',
                $g_dirs['images'] . 'icons/silk/image_link.png'
            );
        }

        $l_template->assign("menu_tree", $l_tree->process($_GET[C__GET__TREE_NODE]));
        
        $l_posts = isys_module_request::get_instance()->get_posts();
        switch ($l_posts[C__GET__NAVMODE]) {
            case defined_or_default('C__NAVMODE__NEW'):
            case defined_or_default('C__NAVMODE__EDIT'):
                isys_component_template_navbar::getInstance()
                    ->set_active(isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, "MODULE/C__MODULE__SMVIEW_PATCH"),
                        defined_or_default('C__NAVBAR_BUTTON__SAVE'))
                    ->set_active(isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, "MODULE/C__MODULE__SMVIEW_PATCH"),
                        defined_or_default('C__NAVBAR_BUTTON__CANCEL'));
                break;
        }
        
        if (isset($_GET['request'])) {
            $instance_id = isset($_GET['instance_id']) ? $_GET['instance_id'] : null;
            $instance_id = isset($_POST['instance_id']) ? $_POST['instance_id'] : $instance_id;
            $this->route($_GET['request'], $instance_id, $templates_in_tree);
        } else {
            $this->route('/');
        }
    }

    /**
     * Routing.
     *
     * @param  string $p_request
     * @param  int|null $p_instance_id
     * @param  array $templates_in_tree
     * @throws isys_exception_database
     * @throws Exception
     */
    public function route($p_request, $p_instance_id = null, $templates_in_tree = []) 
    {
        $l_template = $this->m_tpl;
        $l_posts = isys_module_request::get_instance()->get_posts();
        
        if (empty($p_instance_id)) {
            $items = isset($l_posts['id']) ? $l_posts['id'] : [];
        } else {
            $items = array($p_instance_id);
        }
        
        $grid_template_dao = new isys_cmdb_dao_smview_patch_grid_template($this->m_db);
        switch ($p_request) {
            case 'image_mapping':
                $manager = new ImageMappingManager();
                $l_template->include_template('contentbottomcontent', __DIR__ . '/templates/image_mapping.tpl');
                $this->_manage($manager, $l_posts, $p_instance_id, $items);
                break;
            case 'grid_template_overview':
                if ($_SERVER['REQUEST_METHOD'] === 'POST') {
                    $ids = $_POST['table_ids'];
                    if (! empty($ids)) {
                        $exporter = new GridExporter();
                        $exporter->export($ids);
                    }
                }

                $grid_template_res = $grid_template_dao->filter_objects(array('is_template' => '1'));
                $grid_templates = [];
                foreach ($grid_template_res as $grid_template) {
                    $assigned_objects = $this->_get_assigned_objects($grid_template['id']);
                    $grid_template['object_count'] = count($assigned_objects);
                    $treeId = $templates_in_tree[$grid_template['id']] ?: -1;
                    $grid_template['url'] = isys_helper_link::create_url([
                            C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_PATCH'),
                            C__GET__TREE_NODE => $treeId,
                            'request'         => 'grid_template',
                            'instance_id'     => $grid_template['id'],
                    ]);
                    $grid_templates[] = $grid_template;
                }
                $l_template->assign('grid_templates', $grid_templates);
                $l_template->include_template('contentbottomcontent', __DIR__ . '/templates/grid_template_overview.tpl');

                $l_navbar = isys_component_template_navbar::getInstance();
                $l_navbar->append_button(
                    'LC__CMDB__CATG__SMVIEW_PATCH__NAVBAR_BUTTON_EXPORT',
                    'export',
                    [
                        "icon"       => "icons/silk/package_green.png",
                        "js_onclick" => "document.isys_form.submit();",
                ]);
                if (isys_auth_auth::instance()->is_allowed_to(isys_auth::CREATE, "MODULE/C__MODULE__SMVIEW_PATCH")) {
                    $l_navbar->append_button(
                        'LC__CMDB__CATG__SMVIEW_PATCH__NAVBAR_BUTTON_IMPORT',
                        'import',
                        [
                            "icon" => "icons/silk/package_add.png",
                            "url" => isys_helper_link::create_url([
                                C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_PATCH'),
                                'request'         => 'grid_template_import',
                            ])
                        ]);
                }
                break;
            case 'grid_template':
                $lang = isys_application::instance()->container->get('language');
                if (! isys_module_smview_patch::is_licensed()) {
                    $l_template
                        ->assign('license_error', $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__LICENSE_ERROR'))
                        ->include_template('contentbottomcontent', __DIR__ . '/templates/grid_template.tpl');
                    return;
                }

                $grid_template_dao = new isys_cmdb_dao_smview_patch_grid_template($this->m_db);
                $grid_dao = new isys_cmdb_dao_category_g_smview_patch_grid($this->m_db);
                
                // get grid data
                $grid_template = $grid_template_dao->get_object('id', $p_instance_id);
                $assigned_connectors = $this->_get_connector_templates($p_instance_id);
                $grid['ports'] = array_values($assigned_connectors);
                $grid['frame'] = [
                    'id'                => $p_instance_id,
                    'title'             => $grid_template['title'],
                    'column_count'      => $grid_template['column_count'],
                    'row_count'         => $grid_template['row_count'],
                    'column_count_back' => $grid_template['column_count_back'],
                    'row_count_back'    => $grid_template['row_count_back'],
                    'is_template'       => $grid_template['is_template'],
                    'used_by'           => [],
                ];

                // create JS configuration
                $js_config = [
                    'ajax_url'    => isys_helper_link::create_url([C__GET__AJAX => 1, C__GET__AJAX_CALL => 'patch_grid_template']),
                    'session'     => $_COOKIE['PHPSESSID'],
                    'image_url'   => isys_helper_link::get_base() . '/images/icons/silk/',
                    'grid'        => $grid,
                    'language'   => [
                        'LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_FRONT'  => $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_FRONT'),
                        'LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_BACK'   => $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__ANGLE_BACK'),
                        'LC__CMDB__CATG__SMVIEW_PATCH__FRAME_EDITOR' => $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__FRAME_EDITOR'),
                    ],
                    'size'        => isys_usersettings::get('smview_patch.editor.icon_size', 'm'),
                    'sizes'       => isys_cmdb_dao_smview_patch_grid_template::get_available_sizes(),
                    'is_editable' => false,
                ];

                $l_template
                    ->assign('assets_dir', isys_module_smview_patch::get_assets_www_dir())
                    ->assign('js_config', json_encode($js_config))
                    ->assign('usages', $grid_dao->get_objects_using_grid_template($p_instance_id))
                    ->include_template('contentbottomcontent', __DIR__ . '/templates/grid_template.tpl');
                break;
            case 'grid_template_import':
                $this->_handle_grid_import($l_template);
                break;
            case 'patch_colors':
                $this->m_tpl->include_template('contentbottomcontent', __DIR__ . '/templates/editor_colors.tpl');

                $colors = self::get_editor_colors();
                $color_from_cable = isys_settings::get('smview_patch.editor.color_from_cable', false);
                if (isys_glob_get_param(C__GET__NAVMODE) == defined_or_default('C__NAVMODE__SAVE')) {
                    foreach ($colors as $name => $color) {
                        $c = $_POST[$name];
                        $c = strlen($c) == 6 ? $c : $color['default'];
                        isys_settings::set('smview_patch.editor.' . $name, $c);
                    }

                    $color_from_cable = isset($_POST['color_from_cable']);
                    isys_settings::set('smview_patch.editor.color_from_cable', $color_from_cable);
                }

                $colors = self::get_editor_colors();
                foreach ($colors as $name => $c) {
                    $l_template->assign($name, $c);
                }
                $l_template->assign('color_from_cable', $color_from_cable ? 'checked="checked"' : '');

                $l_navbar = isys_component_template_navbar::getInstance();
                $l_navbar->set_active(isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, "MODULE/C__MODULE__SMVIEW_PATCH"),
                    defined_or_default('C__NAVBAR_BUTTON__SAVE'));
                $l_navbar->set_active(isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, "MODULE/C__MODULE__SMVIEW_PATCH"),
                    defined_or_default('C__NAVBAR_BUTTON__CANCEL'));
                break;
            case 'print_rack':
                $rack = $this->_get_rack($p_instance_id);
                $this->m_tpl->assign('rack', $rack);
                $this->m_tpl->assign('devices', $rack['devices']);
                $this->m_tpl->include_template('contentbottomcontent', __DIR__ . '/templates/rack_print.tpl');
                break;
            case 'print_device':
                $device = $this->_get_device($p_instance_id);
                $this->m_tpl->assign('device', $device['device']);
                $this->m_tpl->assign('connectors', $device['connectors']);
                $this->m_tpl->include_template('contentbottomcontent', __DIR__ . '/templates/device_print.tpl');
                break;
            case 'analyzer':
                if (! isys_module_smview_patch::is_licensed()) {
                    $lang = isys_application::instance()->container->get('language');
                    $this->m_tpl->assign('license_error', $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__LICENSE_ERROR'));
                } else {
                    $js_config = isys_cmdb_ui_category_g_smview_patch_analyzer::get_analyzer_configuration();
                    $this->m_tpl->assign('js_config', json_encode($js_config));
                }
                $this->m_tpl->include_template('contentbottomcontent', __DIR__ . '/templates/analyzer.tpl');
                break;
            case 'importer':
                $this->_handle_excel_import($l_template);
                break;
            case 'analyzer_presets':
                $manager = new AnalyzerPresetManager();
                $l_template->include_template('contentbottomcontent', __DIR__ . '/templates/analyzer_preset.tpl');
                $this->_manage($manager, $l_posts, $p_instance_id, $items);
                break;
            default:
                switch ($l_posts[C__GET__NAVMODE]) {
                    case defined_or_default('C__NAVMODE__SAVE'):
                        $this->_save_config();
                        $this->_process_config_page();
                        break;
                    default:
                        $this->_process_config_page();
                }
                $l_template->include_template('contentbottomcontent', __DIR__ . '/templates/index.tpl');
                break;
        }
        $l_template->assign('ajax_url', isys_helper_link::create_url([C__GET__AJAX => 1, C__GET__AJAX_CALL => 'patch_grid_template']));
        $l_template->assign('assets_dir', $this->get_assets_www_dir());
        $l_template->assign('image_url', isys_helper_link::get_base() . '/images/');
    }

    /**
     * Manage common CRUD operations.
     *
     * @param  AbstractManager $manager
     * @param  array $l_posts
     * @param  int $p_instance_id
     * @param  array $items
     * @throws isys_exception_database
     * @throws isys_exception_dao
     */
    private function _manage($manager, $l_posts, $p_instance_id, $items) {
        switch ($l_posts[C__GET__NAVMODE]) {
            case defined_or_default('C__NAVMODE__NEW'):
                $manager->handleNewMode();
                break;
            case defined_or_default('C__NAVMODE__SAVE'):
                $manager->saveObject($p_instance_id);
                $manager->processListing();
                break;
            case defined_or_default('C__NAVMODE__DELETE'):
                foreach ($items as $id) {
                    $manager->deleteObject($id);
                }
                $manager->processListing();
                break;
            default:
                if (! empty($items)) {
                    $id = $items[0];
                    $manager->processPage($id);
                } else {
                    $manager->processListing();
                }
        }
    }

    /**
     * Get assigned objects.
     *
     * @param  int $grid_template_id
     * @return array
     * @throws isys_exception_database
     */
    private function _get_assigned_objects($grid_template_id) 
    {
        $dao = isys_cmdb_dao_smview_patch_grid::instance($this->m_db);
        $common_dao = isys_cmdb_dao_category_g_global::instance($this->m_db);
        $assigned_objects = [];
        $grid_templates = $dao->filter_objects(array('grid_template_id' => $grid_template_id));
        foreach ($grid_templates as $grid)  {
            $obj = $common_dao->get_object($grid['obj_id'])->get_row();
            $assigned_objects[$grid['obj_id']] = $obj['isys_obj__title'];
        }
        return $assigned_objects;
    }

    /**
     * Get configured connector templates.
     *
     * @param  int $grid_template_id
     * @return array
     * @throws isys_exception_database
     */
    private function _get_connector_templates($grid_template_id) {
        $connector_template_dao = new isys_cmdb_dao_smview_patch_connector_template($this->m_db);
        $connector_templates = [];
        $tpls = $connector_template_dao->filter_objects(array('grid_template_id' => $grid_template_id));
        foreach ($tpls as $tpl) {
            $picture_dao = new isys_cmdb_dao_smview_patch_picture($this->m_db);
            $pic = $picture_dao->get_object('id', $tpl['connector_type_id']);
            $type_title = ! empty($pic) ? $pic['title'] : null;
            
            $tpl['connector_type_title'] = $type_title;
            $tpl['url']                  = $picture_dao->get_file_url($pic['obj_id']);
            
            $connector_templates[$tpl['id']] = $tpl;
        }
        return $connector_templates;
    }

    /**
     * Handle grid import.
     *
     * @param  isys_component_template $l_template
     * @throws isys_exception_general
     * @throws isys_exception_validation
     */
    private function _handle_grid_import($l_template)
    {
        global $g_dirs;

        $isUpload = true;
        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            if (! isys_auth_auth::instance()->is_allowed_to(isys_auth::CREATE, "MODULE/C__MODULE__SMVIEW_PATCH")) {
                return;
            }

            $importer = new GridImporter();
            if (empty($_FILES['C__CATS__FILE_UPLOAD'])) {
                // import selected grids
                $ids = $_POST['table_ids'];
                if (! empty($ids)) {
                    $dir = $_POST['temp_dir'];
                    $importer->import($dir, $ids);
                }
                return;
            }

            // show grid selection
            $new_file = $this->_save_uploaded_file("C__CATS__FILE_UPLOAD");
            $dir = $importer->unzip($new_file, $g_dirs["temp"]);
            $grids = $importer->get_grids($dir);

            $l_template
                ->assign('grids', $grids)
                ->assign('temp_dir', $dir)
                ->include_template('contentbottomcontent', __DIR__ . '/templates/grid_template_preview.tpl');

            isys_component_template_navbar::getInstance()
                ->set_save_mode('formsubmit')
                ->append_button(
                'LC__CMDB__CATG__SMVIEW_PATCH__NAVBAR_BUTTON_IMPORT',
                'import',
                [
                    "icon"       => "icons/silk/package_add.png",
                    "js_onclick" => "document.isys_form.submit();",
                ]);
            return;
        }

        // show upload dialog
        $l_template
            ->assign('encType', 'multipart/form-data')
            ->assign('isUpload', $isUpload)
            ->include_template('contentbottomcontent', __DIR__ . '/templates/grid_template_import.tpl');

        // This is necessary for the file-upload.
        if (isys_auth_auth::instance()->is_allowed_to(isys_auth::CREATE, "MODULE/C__MODULE__SMVIEW_PATCH")) {
            isys_component_template_navbar::getInstance()
                ->set_save_mode('formsubmit')
                ->append_button(
                    'LC__CMDB__CATG__SMVIEW_PATCH__NAVBAR_BUTTON_UPLOAD',
                    'upload',
                    [
                        "icon"       => "icons/silk/page_save.png",
                        "js_onclick" => "document.isys_form.submit();",
                    ]);
        }
    }

    /**
     * Get rack by object ID.
     *
     * @param  int $obj_id
     * @return array
     * @throws isys_exception_database
     */
    private function _get_rack($obj_id)
    {
        $model_dao = new isys_cmdb_dao_category_g_model($this->m_db);
        $model_row = $model_dao->get_data(null, $obj_id)->get_row();

        $patch_dao = isys_cmdb_dao_category_g_smview_patch_editor::instance($this->m_db);
        $rack = $patch_dao->get_rack($obj_id);
        $rack['manufacturer_title'] = $model_row['isys_model_manufacturer__title'];
        $rack['model_title']        = $model_row['isys_model_title__title'];

        $devices = [];
        for ($i = 1; $i <= $rack['units']; $i++) {
            $position = $rack['sorting'] == 'asc' ? $i : $rack['units'] - $i + 1;
            $position = $rack['has_half_he'] ? $position / 2 : $position;

            $dev = null;
            foreach ($rack['devices_front'] as $d) {
                if ($d['position'] == $i) {
                    $dev = $d;
                }
            }

            if (! $dev) {
                $devices[$i] = [
                    'position'  => $position,
                    'image_url' => null,
                    'title'     => null,
                    'he'        => 1,
                ];
                continue;
            }

            $he = (int) $dev['he'];
            if (empty($dev['he'])) {
                // fix invalid HE
                $he = 1;
            }

            if ($rack['has_half_he'] && ! $dev['has_half_he']) {
                // duplicate HE
                $he = $he * 2;
            }

            if (! $rack['has_half_he'] && $dev['has_half_he']) {
                // halve HE
                $he = $he % 2 == 0 ? ($he / 2) : (($he + 1) / 2);
            }

            if ($he == 0) {
                // set to 1 if it is 0
                $he = 1;
            }

            $obj = $model_dao->get_object($dev['id'])->get_row();
            $cmdb_status = defined_or_default('C__CMDB_STATUS__PLANNED', 1);
            $devices[$i] = [
                'position'   => $position,
                'image_url'  => $dev['image_url'],
                'title'      => $dev['title'],
                'he'         => $he,
                'color'      => '#' . $dev['type_color'],
                'is_planned' => $obj['isys_obj__isys_cmdb_status__id'] == $cmdb_status,
            ];

            // handle multiple HEs
            if ($he > 1) {
                $i += ($he - 1);
            }
        }

        $rack['devices'] = $devices;
        return $rack;
    }

    /**
     * Get grid by object ID.
     *
     * @param  int $obj_id
     * @return array
     * @throws isys_exception_database
     * @throws Exception
     */
    private function _get_device($obj_id)
    {
        $lang = isys_application::instance()->container->get('language');
        $ip_dao = new isys_cmdb_dao_category_g_ip($this->m_db);
        $model_dao = new isys_cmdb_dao_category_g_model($this->m_db);
        $inventory_dao = new isys_cmdb_dao_category_g_accounting($this->m_db);

        $obj = $ip_dao->get_object($obj_id)->get_row();
        $ip_row = $ip_dao->get_data(null, $obj_id)->get_row();
        $model_row = $model_dao->get_data(null, $obj_id)->get_row();
        $inventory_row = $inventory_dao->get_data(null, $obj_id)->get_row();

        $device = [
            'id'                 => $obj_id,
            'title'              => $obj['isys_obj__title'],
            'description'        => $obj['isys_obj__description'],
            'type_title'         => $lang->get($obj['isys_obj_type__title']),
            'img_url'            => isys_cmdb_dao_smview_patch_picture::get_device_image($obj_id),
            'manufacturer_title' => $model_row['isys_model_manufacturer__title'],
            'model_title'        => $model_row['isys_model_title__title'],
            'serial_number'      => $model_row['isys_catg_model_list__serial'],
            'inventory_number'   => $inventory_row['isys_catg_accounting_list__inventory_no'],
            'ip'                 => $ip_row['isys_cats_net_ip_addresses_list__title'],
        ];

        $patch_dao = isys_cmdb_dao_category_g_smview_patch_editor::instance($this->m_db);
        $available_connectors = $patch_dao->get_available_connectors($obj_id);

        $grid_list_dao = new isys_cmdb_dao_category_g_smview_patch_grid($this->m_db);
        $item = $grid_list_dao->get_data(null, $obj_id)->get_row();
        if (empty($item)) {
            // use all available connectors
            $assigned_connectors = $available_connectors;
        } else {
            // use only assigned connectors
            $list_id = $item['isys_catg_smview_patch_grid_list__id'];
            $grid_template_id = $item['isys_catg_smview_patch_grid_list__grid_template_id'];
            $assigned_connectors = $patch_dao->get_assigned_connector_templates($list_id, $grid_template_id, $available_connectors);
        }

        return [
            'device'     => $device,
            'connectors' => $assigned_connectors,
        ];
    }

    /**
     * Handle import of uploaded excel file.
     *
     * @param  isys_component_template $l_template
     * @throws Exception
     */
    private function _handle_excel_import($l_template)
    {
        $l_template->include_template('contentbottomcontent', __DIR__ . '/templates/importer.tpl');
        $l_template->assign('encType', 'multipart/form-data');
        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            if (! isys_auth_auth::instance()->is_allowed_to(isys_auth::CREATE, "MODULE/C__MODULE__SMVIEW_PATCH")) {
                return;
            }

            $file_types = [
                'RACK',
                'PATCH',
                'DEVICE',
                'MODULE',
                'SOCKET',
                'LINK_PP_TO_PP',
                'PATCH_PP_TO_DEVICE',
            ];

            $file_type = $_POST['C__CATS__FILE_UPLOAD_TYPE'];
            $import_after_simulation = isset($_POST['import_after_simulation']);
            $forceImport = isset($_POST['C__CATS__FILE_UPLOAD_FORCE_IMPORT']) ? $_POST['C__CATS__FILE_UPLOAD_FORCE_IMPORT'] == 'on' : false;
            if ($forceImport) {
                // remove lock file to start a new import
                Excel::deleteLockFile();
            }

            try {
                if (in_array($file_type, $file_types) && $_FILES['C__CATS__FILE_UPLOAD_FILE']['size'] > 0) {
                    $new_file = $this->_save_uploaded_file($file_type);
                    $simulate = isset($_POST['C__CATS__FILE_UPLOAD_SIMULATE']) ? $_POST['C__CATS__FILE_UPLOAD_SIMULATE'] == 'on' : false;
                    $importer = new Excel($this->m_db, $new_file, $simulate);
                    $log = $importer->import($file_type);
                    $l_template->assign('file_name', $new_file);
                    $l_template->assign('file_type', $file_type);
                    $l_template->assign('was_simulation', $simulate);
                    $l_template->assign('success', true);
                    $l_template->assign('log', $log);
                }
                if ($import_after_simulation) {
                    $file_name = $_POST['file_name'];
                    $importer = new Excel($this->m_db, $file_name, false);
                    $log = $importer->import($file_type);
                    $l_template->assign('success', true);
                    $l_template->assign('log', $log);
                }
            } catch (ImporterException $e) {
                $l_template->assign('error', $e->getMessage());
                $l_template->assign('log', $e->getLog());
            } catch (Exception $e) {
                $l_template->assign('error', $e->getMessage());
            }
        }

        if (Excel::isInProgress()) {
            $lang = isys_application::instance()->container->get('language');
            $l_template->assign('import_in_progress', $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_IN_PROGRESS'));
        }

        // This is necessary for the file-upload.
        if (isys_auth_auth::instance()->is_allowed_to(isys_auth::CREATE, "MODULE/C__MODULE__SMVIEW_PATCH")) {
            isys_component_template_navbar::getInstance()
                ->set_save_mode('formsubmit')
                ->append_button(
                    'LC__CMDB__CATG__SMVIEW_PATCH__NAVBAR_BUTTON_UPLOAD',
                    'upload', [
                    "icon"       => "icons/silk/page_save.png",
                    "js_onclick" => "trigger_submit();", //"document.isys_form.submit();",
                ]);
        }
    }

    /**
     * Save uploaded file.
     *
     * @param  string $key
     * @return string
     */
    private function _save_uploaded_file($key)
    {
        global $g_dirs;

        $key = 'C__CATS__FILE_UPLOAD_' . $key;
        $l_new_filename = isys_helper_upload::prepare_filename('grids__' . time() . '__' . $_FILES['C__CATS__FILE_UPLOAD_FILE']['name']);
        try {
            isys_helper_upload::save($_FILES['C__CATS__FILE_UPLOAD_FILE'], $l_new_filename, $g_dirs["temp"], 0644);
        } catch (isys_exception_filesystem $e) {
            isys_application::instance()->container['notify']->error($e->getMessage() . '<br />' . isys_helper_upload::get_error($key) . '<br />');
        }
        return $l_new_filename;
    }

    /**
     * Process cache configuration page.
     */
    private function _process_config_page()
    {
        $l_navbar = isys_component_template_navbar::getInstance();
        $canEdit = isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, "MODULE/C__MODULE__SMVIEW_PATCH");
        if ($_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__EDIT')) {
            $l_navbar->set_selected($canEdit, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
                ->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__SAVE'))
                ->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__CANCEL'));
        } else {
            $l_navbar->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
                ->set_visible($canEdit, defined_or_default('C__NAVBAR_BUTTON__EDIT'));
        }

        $lang = isys_application::instance()->container->get('language');
        $dao = new isys_cmdb_dao_object_type(isys_application::instance()->database);
        $res = $dao->retrieve("SELECT t.* FROM isys_obj_type t;");
        $object_types = [];
        while ($row = $res->get_row()) {
            $object_types[$row['isys_obj_type__id']] = $lang->get($row['isys_obj_type__title']);
        }
        $l_rules['C__SMVIEW_PATCH__DEFAULT_SOCKET_TYPE']['p_arData'] = $object_types;

        $default_socket_type = isys_settings::get(self::DEFAULT_SOCKET_TYPE, null);
        $l_rules['C__SMVIEW_PATCH__DEFAULT_SOCKET_TYPE']['p_strSelectedID'] = $default_socket_type ?: '';

        $template_path = isys_settings::get(self::IMPORT_TEMPLATE_PATH, null);
        $l_rules['C__SMVIEW_PATCH__IMPORT_TEMPLATE_PATH']['p_strSelectedID'] = $template_path ?: '';

        $res = $dao->retrieve("SELECT * FROM isys_cmdb_status;");
        $status = [];
        while ($row = $res->get_row()) {
            $status[$row['isys_cmdb_status__id']] = $lang->get($row['isys_cmdb_status__title']);
        }
        $l_rules['C__SMVIEW_PATCH__CABLE_STATUS_ON_DELETION']['p_arData'] = $status;
        $l_rules['C__SMVIEW_PATCH__CABLE_STATUS_PLANNING']['p_arData'] = $status;

        $cable_status = isys_settings::get(self::CABLE_STATUS_AFTER_DELETION, null);
        $l_rules['C__SMVIEW_PATCH__CABLE_STATUS_ON_DELETION']['p_strSelectedID'] = $cable_status ?: '';

        $cable_status_planning = isys_settings::get(self::CABLE_STATUS_PLANNING, null);
        $l_rules['C__SMVIEW_PATCH__CABLE_STATUS_PLANNING']['p_strSelectedID'] = $cable_status_planning ?: '';

        $trimConnectorTitle = isys_settings::get(self::RIGHT_ALIGN_CONNECTOR_TITLE, 0);
        $l_rules['C__SMVIEW_PATCH__RIGHT_ALIGN_CONNECTOR_TITLE']['p_strSelectedID'] = $trimConnectorTitle;
        $l_rules['C__SMVIEW_PATCH__RIGHT_ALIGN_CONNECTOR_TITLE']['p_arData'] = get_smarty_arr_YES_NO();

        $traceDirectly = isys_settings::get(self::TRACE_RIGHT_PORT_DIRECTLY, 0);
        $l_rules['C__SMVIEW_PATCH__TRACE_RIGHT_PORT_DIRECTLY']['p_strSelectedID'] = $traceDirectly;
        $l_rules['C__SMVIEW_PATCH__TRACE_RIGHT_PORT_DIRECTLY']['p_arData'] = get_smarty_arr_YES_NO();
        isys_application::instance()->template->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);
    }

    /**
     * Save cache configuration.
     */
    private function _save_config()
    {
        $fieldName = 'C__SMVIEW_PATCH__DEFAULT_SOCKET_TYPE';
        $default_socket_type = isset($_POST[$fieldName]) && $_POST[$fieldName] != -1 ? $_POST[$fieldName] : null;
        isys_settings::set(self::DEFAULT_SOCKET_TYPE, $default_socket_type);

        $fieldName = 'C__SMVIEW_PATCH__IMPORT_TEMPLATE_PATH__HIDDEN';
        $template_path = ! empty($_POST[$fieldName]) ? $_POST[$fieldName] : null;
        isys_settings::set(self::IMPORT_TEMPLATE_PATH, $template_path);

        $fieldName = 'C__SMVIEW_PATCH__CABLE_STATUS_ON_DELETION';
        $cable_status = isset($_POST[$fieldName]) && $_POST[$fieldName] != -1 ? $_POST[$fieldName] : null;
        isys_settings::set(self::CABLE_STATUS_AFTER_DELETION, $cable_status);

        $fieldName = 'C__SMVIEW_PATCH__CABLE_STATUS_PLANNING';
        $cable_status = isset($_POST[$fieldName]) && $_POST[$fieldName] != -1 ? $_POST[$fieldName] : null;
        isys_settings::set(self::CABLE_STATUS_PLANNING, $cable_status);

        $fieldName = 'C__SMVIEW_PATCH__RIGHT_ALIGN_CONNECTOR_TITLE';
        $trimConnectorTitle = $_POST[$fieldName] ?? null;
        isys_settings::set(self::RIGHT_ALIGN_CONNECTOR_TITLE, $trimConnectorTitle);

        $fieldName = 'C__SMVIEW_PATCH__TRACE_RIGHT_PORT_DIRECTLY';
        $traceDirectly = $_POST[$fieldName] ?? null;
        isys_settings::set(self::TRACE_RIGHT_PORT_DIRECTLY, $traceDirectly);
    }
}
