<?php

namespace idoit\Module\Patch;

use Exception;
use idoit\Module\Patch\DependentObjectManager\AnalyzerPresetManager;
use isys_application;
use isys_cmdb_dao_category_g_connector;
use isys_cmdb_dao_category_g_global;
use isys_cmdb_dao_category_g_location;
use isys_cmdb_dao_smview_patch_analyzer_preset;
use isys_exception_database;

/**
 * Analyzer class.
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class Analyzer
{

    /**
     * Listing of translated object type titles.
     *
     * @var array
     */
    private $_object_type_translations = [];

    /**
     * Get all root level elements.
     *
     * @return array
     * @throws Exception
     */
    public function get_levels_from_root()
    {
        $lang = isys_application::instance()->container->get('language');
        $location_dao = new isys_cmdb_dao_category_g_location(isys_application::instance()->database);

        $sql = " AND isys_catg_location_list__parentid = 1";
        $res = $location_dao->get_data(null, null, $sql);
        $levels = [];
        while ($row = $res->get_row()) {
            if (empty($levels['root'])) {
                $levels['root'] = [
                    'id'         => null,
                    'title'      => null,
                    'type_id'    => $row['isys_obj_type__id'],
                    'type_const' => $row['isys_obj_type__const'],
                    'type_title' => $lang->get($row['isys_obj_type__title']),
                    'type_color' => $row['isys_obj_type__color'],
                    'objects'    => [],
                ];
            }
            $levels['root']['objects'][] = [
                'id'         => $row['isys_obj__id'],
                'title'      => $lang->get($row['isys_obj_type__title']) . ' - ' . $row['isys_obj__title'],
                'type_id'    => $row['isys_obj_type__id'],
                'type_const' => $row['isys_obj_type__const'],
                'type_title' => $lang->get($row['isys_obj_type__title']),
                'type_color' => $row['isys_obj_type__color'],
            ];
        }
        return array_values($levels);
    }

    /**
     * Get dynamic levels based on object location.
     *
     * @param  int $obj_id
     * @return array
     * @throws isys_exception_database
     */
    public function get_levels_by_object_id($obj_id)
    {
        // add all level above this object
        $lang = isys_application::instance()->container->get('language');
        $location_dao = new isys_cmdb_dao_category_g_location(isys_application::instance()->database);
        $global_dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
        $path = $location_dao->get_location_path($obj_id);

        // start with root element
        $path = array_reverse($path);

        // add this object
        $path[] = $obj_id;

        $levels = [];
        $lastObjId = null;
        foreach ($path as $p) {
            $obj = $global_dao->get_object($p)->get_row();
            $type_const = $obj['isys_obj_type__const'];
            $levels[$type_const] = [
                'id'         => $p,
                'title'      => $obj['isys_obj__title'],
                'type_id'    => $obj['isys_obj_type__id'],
                'type_const' => $type_const,
                'type_title' => $lang->get($obj['isys_obj_type__title']),
                'type_color' => $obj['isys_obj_type__color'],
                'objects'    => self::get_elements_by_type($obj['isys_obj_type__id'], $lastObjId),
            ];
            $lastObjId = $p;
        }
        return array_values($levels);
    }

    /**
     * Get all defined presets.
     *
     * @return array
     * @throws Exception
     */
    public function get_presets()
    {
        // todo: add filter of category properties (e.g.: a specific model)
        $lang = isys_application::instance()->container->get('language');
        $preset_dao = new isys_cmdb_dao_smview_patch_analyzer_preset(isys_application::instance()->database);
        $dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
        $manager = new AnalyzerPresetManager();
        $presets = $preset_dao->get_objects();
        $prepared_presets = [];
        foreach ($presets as $preset) {
            $object_type_ids = $manager->get_assigned_object_types($preset['id']);
            $condition = implode(',', $object_type_ids);
            $sql = "SELECT * from isys_obj_type WHERE isys_obj_type__id IN ($condition)";
            $res = $dao->retrieve($sql);
            $object_types = [];
            while ($row = $res->get_row()) {
                $object_types[$row['isys_obj_type__const']] = $lang->get($row['isys_obj_type__title']);
            }
            $preset['object_types'] = $object_types;
            $prepared_presets[] = $preset;
        }
        return $prepared_presets;
    }

    /**
     * Get connector types.
     *
     * @return array
     * @throws isys_exception_database
     * @throws Exception
     */
    public function get_connector_types()
    {
        $connector_dao = new isys_cmdb_dao_category_g_connector(isys_application::instance()->database);
        $status = defined_or_default('C__RECORD_STATUS__NORMAL');
        $l_sql = "SELECT * FROM isys_connection_type WHERE isys_connection_type__status = $status;";

        $res = $connector_dao->retrieve($l_sql);
        $lang = isys_application::instance()->container->get('language');
        $types = [];
        while ($row = $res->get_row()) {
            $title = $row['isys_connection_type__title'];
            $types[$row['isys_connection_type__id']] = $lang->get($title);
        }
        return $types;
    }

    /**
     * Get all elements of the requested preset level.
     *
     * @param  int $obj_id
     * @param  string $preset_id
     * @param  int $connectionType
     * @param  array|null $connectorTypes
     * @return array
     * @throws Exception
     */
    public function get_dynamic_elements_by_level($obj_id, $preset_id, $connectionType = 1, $connectorTypes = null)
    {
        $presets = $this->get_presets();
        $preset = null;
        foreach ($presets as $p) {
            if ($p['id'] == $preset_id) {
                $preset = $p;
                break;
            }
        }

        if (empty($preset)) {
            return [];
        }
        $level_objects = $this->_get_children_of_object($obj_id, array_keys($preset['object_types']), $connectionType, $connectorTypes);

        // consolidate device connections up to elements of this preset level
        $connections = [];
        foreach ($level_objects as $level_object) {
            foreach ($level_object['connections'] as $obj_id => $connected_objects) {
                foreach ($connected_objects as $connected_obj_id => $connected_object) {
                    // try to get a matching connected device
                    foreach ($level_objects as $le2) {
                        $found = $le2['connections'][$connected_obj_id][$obj_id];
                        if (! $found) {
                            continue;
                        }

                        // found matching connected device
                        $elementId = $level_object['id'];
                        $sibElementId = $le2['id'];
                        if ($elementId == $sibElementId) {
                            continue;
                        }

                        // get connections of connected object
                        $obj = $le2['connections'][$connected_obj_id][$obj_id];

                        // get link-connection count
                        $count = count($connected_object['patched']) + count($connected_object['un_patched']);
                        $unPatchedCount = 0;
                        foreach ($connected_object['un_patched'] as $sib_connected_connector) {
                            // check if the other side of the connection is also not patched
                            if (! isset($obj['patched'][$sib_connected_connector])) {
                                // this connection is not patched on the other side
                                $unPatchedCount++;
                            }
                        }

                        $connections[$elementId][$sibElementId] = [
                            'sibling'     => $sibElementId,
                            'used'        => $count - $unPatchedCount,
                            'count'       => $count,
                            'cable_color' => $connected_object['cable_color'],
                        ];
                    }
                }
            }
        }

        // rewrite connections with information of this level
        $mod_level_elements = [];
        foreach ($level_objects as $level_object) {
            if ($connections[$level_object['id']]) {
                $level_object['connections'] = $connections[$level_object['id']];
            }
            $mod_level_elements[] = $level_object;
        }

        return $mod_level_elements;
    }

    /**
     * Get all children of an object, matching one of the provided object types.
     *
     * @param  int $obj_id
     * @param  array $types
     * @param  int $connectionType
     * @param  array|null $connectorTypes
     * @param  boolean $withConnections
     * @return array
     * @throws isys_exception_database
     */
    private function _get_children_of_object($obj_id, $types, $connectionType = 1, $connectorTypes = null, $withConnections = true)
    {
        // todo: maybe use memcached to cache some sub-results
        $lang = isys_application::instance()->container->get('language');
        $dao = new isys_cmdb_dao_category_g_location(isys_application::instance()->database);
        $sql = " AND isys_catg_location_list__parentid = $obj_id";
        $res = $dao->get_data(null, null, $sql, null, defined_or_default('C__RECORD_STATUS__NORMAL'));
        $objects = [];
        while ($row = $res->get_row()) {
            if (in_array($row['isys_obj_type__const'], $types)) {
                $obj_type_id = $row['isys_obj_type__id'];
                if (! isset($this->_object_type_translations[$obj_type_id])) {
                    $this->_object_type_translations[$obj_type_id] = $lang->get($row['isys_obj_type__title']);
                }
                $obj = [
                    'id'         => $row['isys_obj__id'],
                    'title'      => $row['isys_obj__title'],
                    'type_id'    => $obj_type_id,
                    'type_const' => $row['isys_obj_type__const'],
                    'type_title' => $this->_object_type_translations[$obj_type_id],
                    'type_color' => $row['isys_obj_type__color'],
                ];
                if ($withConnections) {
                    $obj['connections'] = $this->_get_connections($row['isys_obj__id'], $connectionType, $connectorTypes);
                }
                $objects[] = $obj;
            }

            $children = $this->_get_children_of_object($row['isys_obj__id'], $types, $connectionType, $connectorTypes, $withConnections);
            $objects = array_merge($objects, $children);
        }
        return $objects;
    }

    /**
     * Get all incoming connections of an object.
     *
     * @param  int $upper_obj_id
     * @param  int|null $connectionType
     * @param  array|null $connectorTypes
     * @return array
     * @throws isys_exception_database
     */
    private function _get_connections($upper_obj_id, $connectionType = null, $connectorTypes = null)
    {
        $connector_dao = new isys_cmdb_dao_category_g_connector(isys_application::instance()->database);

        // todo: make object types, treated as "devices", configurable
        $device_object_types = ['C__OBJTYPE__PATCH_PANEL', 'C__OBJTYPE__SWITCH', 'C__SMD__GERAET'];

        // check if provided object is already a device
        $obj = $connector_dao->get_object_by_id($upper_obj_id)->get_row();
        if (! in_array($obj['isys_obj_type__const'], $device_object_types)) {
            // object is not a device, get its children
            $devices = $this->_get_children_of_object($upper_obj_id, $device_object_types, $connectionType, $connectorTypes, false);
        } else {
            // object is a device, use it directly
            $devices[] = ['id' => $upper_obj_id];
        }

        // only consider directly connected devices
        $connections = [];
        $connectionType = $connectionType ?: 1;
        foreach ($devices as $device) {
            $obj_id = $device['id'];

            // get all link-connections (inputs) of an object, which have a connected object
            $p_status = defined_or_default('C__RECORD_STATUS__NORMAL');
            $l_sql = "SELECT 
                isys_catg_connector_list.isys_catg_connector_list__id,
                isys_connection_type.*,
                isys_cable_colour.isys_cable_colour__title AS cable_color,
                connected_connector.isys_catg_connector_list__id AS con_connector,
                connected_obj.isys_obj__id AS connected_obj_id,
                connected_obj.isys_obj__title AS connected_obj_title,   
                sib_connector.isys_catg_connector_list__isys_cable_connection__id AS patch_cable_connection_id
                FROM isys_catg_connector_list
                LEFT JOIN isys_connection_type ON isys_catg_connector_list.isys_catg_connector_list__isys_connection_type__id = isys_connection_type__id
                LEFT JOIN isys_cable_connection ON isys_catg_connector_list.isys_catg_connector_list__isys_cable_connection__id = isys_cable_connection__id
                LEFT JOIN isys_catg_cable_list ON isys_catg_cable_list.isys_catg_cable_list__isys_obj__id = isys_cable_connection__isys_obj__id
                LEFT JOIN isys_cable_colour ON isys_cable_colour.isys_cable_colour__id = isys_catg_cable_list__isys_cable_colour__id
                LEFT JOIN isys_catg_connector_list AS connected_connector ON connected_connector.isys_catg_connector_list__isys_cable_connection__id = isys_cable_connection__id
                    AND (connected_connector.isys_catg_connector_list__id != isys_catg_connector_list.isys_catg_connector_list__id OR connected_connector.isys_catg_connector_list__id IS NULL)
                LEFT JOIN isys_obj AS connected_obj ON connected_connector.isys_catg_connector_list__isys_obj__id = connected_obj.isys_obj__id
                LEFT JOIN isys_catg_connector_list sib_connector ON isys_catg_connector_list.isys_catg_connector_list__isys_catg_connector_list__id = sib_connector.isys_catg_connector_list__id
                WHERE connected_connector.isys_catg_connector_list__id IS NOT NULL
                  AND isys_catg_connector_list.isys_catg_connector_list__type = $connectionType
                  AND isys_catg_connector_list.isys_catg_connector_list__isys_obj__id = " . $connector_dao->convert_sql_id($obj_id) . "
                  AND isys_catg_connector_list.isys_catg_connector_list__status = " . $connector_dao->convert_sql_int($p_status);
            $res = $connector_dao->retrieve($l_sql);

            while ($row = $res->get_row()) {
                if ($row['con_connector'] <= 0) {
                    // connector is not connected to any object (normally not possible with this query)
                    continue;
                }

                if (! empty($connectorTypes) && ! in_array($row['isys_connection_type__id'], $connectorTypes)) {
                    // connector type does not match
                    continue;
                }

                $connected_obj_id = $row['connected_obj_id'];
                if (isset($connections[$obj_id][$connected_obj_id])) {
                    // current object has at least one connection to the connected object
                    $sib = $connections[$obj_id][$connected_obj_id];
                    $patched = $sib['patched'];
                    $unPatched = $sib['un_patched'];
                } else {
                    // current object is not connected to the connected object yet
                    $patched = [];
                    $unPatched = [];
                }

                // check, if the corresponding output has a cable connection
                // if so, this connector will be marked as "patched"
                $isPatched = ! empty($row['patch_cable_connection_id']);
                $connectorId = $row['isys_catg_connector_list__id'];
                if ($isPatched) {
                    // add connector to patched-list with the connected connector
                    $patched[$connectorId] = $row['con_connector'];
                } else {
                    // add connector to un-patched-list with the connected connector
                    $unPatched[$connectorId] = $row['con_connector'];
                }

                // add or update both lists
                $connections[$obj_id][$connected_obj_id] = [
                    'patched'     => $patched,
                    'un_patched'  => $unPatched,
                    'cable_color' => $row['cable_color'],
                ];
            }
        }
        return $connections;
    }

    /**
     * Get all elements by object type and optional as parent of provided object ID.
     *
     * @param  string $type_id
     * @param  int|null $obj_id
     * @throws Exception
     */
    public function get_elements_by_type($type_id, $obj_id = null)
    {
        $lang = isys_application::instance()->container->get('language');
        $dao = new isys_cmdb_dao_category_g_location(isys_application::instance()->database);
        $sql = " AND isys_obj_type__id = " . $dao->convert_sql_int($type_id);
        if (! empty($obj_id)) {
            $sql .= " AND isys_catg_location_list__parentid = " . $dao->convert_sql_int($obj_id);
        }
        $res = $dao->get_data(null, null, $sql);
        $objects = [];
        while ($row = $res->get_row()) {
            $objects[] = [
                'id'         => $row['isys_obj__id'],
                'title'      => $row['isys_obj__title'],
                'type_id'    => $row['isys_obj_type__id'],
                'type_const' => $row['isys_obj_type__const'],
                'type_title' => $lang->get($row['isys_obj_type__title']),
                'type_color' => $row['isys_obj_type__color'],
            ];
        }
        return $objects;
    }

    /**
     * Get all elements underneath provided object.
     *
     * @param  int $obj_id
     * @return array
     * @throws Exception
     */
    public function get_elements_underneath_object($obj_id)
    {
        $lang = isys_application::instance()->container->get('language');
        $dao = new isys_cmdb_dao_category_g_location(isys_application::instance()->database);
        $sql = " AND isys_catg_location_list__parentid = " . $dao->convert_sql_int($obj_id);
        $res = $dao->get_data(null, null, $sql);
        $object_types = [];
        while ($row = $res->get_row()) {
            $typeId = $row['isys_obj_type__id'];
            if (empty($object_types[$typeId])) {
                $object_types[$typeId] = [
                    'id'      => $row['isys_obj_type__id'],
                    'const'   => $row['isys_obj_type__const'],
                    'title'   => $lang->get($row['isys_obj_type__title']),
                    'color'   => $row['isys_obj_type__color'],
                    'objects' => [],
                ];
            }
            $object_types[$typeId]['objects'][] = [
                'id'         => $row['isys_obj__id'],
                'title'      => $row['isys_obj__title'],
                'type_id'    => $row['isys_obj_type__id'],
                'type_const' => $row['isys_obj_type__const'],
                'type_title' => $lang->get($row['isys_obj_type__title']),
                'type_color' => $row['isys_obj_type__color'],
            ];
        }
        return array_values($object_types);
    }
}
