<?php

namespace idoit\Module\Smview_Patch\Console\Command;

use Exception;
use idoit\Console\Command\AbstractCommand;
use isys_application;
use isys_cmdb_dao_cable_connection;
use isys_cmdb_dao_category_g_connector;
use isys_cmdb_dao_category_g_formfactor;
use isys_cmdb_dao_category_g_location;
use isys_cmdb_dao_location;
use isys_cmdb_dao_object;
use isys_cmdb_dao_smview_patch_connector_template;
use isys_cmdb_dao_smview_patch_formfactor;
use isys_cmdb_dao_smview_patch_grid;
use isys_cmdb_dao_smview_patch_grid_connector;
use isys_cmdb_dao_smview_patch_grid_template;
use isys_cmdb_dao_smview_patch_picture;
use Symfony\Component\Console\Input\InputDefinition;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Command to generate some test data.
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class TestDataGeneratorCommand extends AbstractCommand
{

    /**
     * @var OutputInterface
     */
    private $_output;

    /**
     * Get name for command
     *
     * @return string
     */
    public function getCommandName()
    {
        return 'smview_patch_test_data_generator';
    }

    /**
     * Get description for command
     *
     * @return string
     */
    public function getCommandDescription()
    {
        return 'Command to generate some SM-VIEW patch test data.';
    }

    /**
     * Retrieve Command InputDefinition
     *
     * @return InputDefinition
     */
    public function getCommandDefinition()
    {
        return new InputDefinition();
    }

    /**
     * Checks if a command can have a config file via --config
     *
     * @return bool
     */
    public function isConfigurable()
    {
        return false;
    }

    /**
     * Execute command.
     *
     * @param  InputInterface  $input
     * @param  OutputInterface $output
     * @return int
     * @throws Exception
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $this->_output = $output;

        try {
            $output->writeln("start\n");

            // assign categories
            $this->_assign_category('C__CATG__SMVIEW_PATCH_EDITOR', 'C__OBJTYPE__PATCH_PANEL');
            $this->_assign_category('C__CATG__FORMFACTOR', 'C__OBJTYPE__PATCH_PANEL');
            $this->_assign_category('C__CATG__SMVIEW_PATCH_FORMFACTOR', 'C__OBJTYPE__PATCH_PANEL');
            $this->_assign_category('C__CATG__SMVIEW_PATCH_EDITOR', 'C__OBJTYPE__PATCH_PANEL');
            $this->_assign_category('C__CATG__LOCATION', 'C__OBJTYPE__PATCH_PANEL');

            $this->_assign_category('C__CATG__FORMFACTOR', 'C__OBJTYPE__ENCLOSURE');
            $this->_assign_category('C__CATG__SMVIEW_PATCH_FORMFACTOR', 'C__OBJTYPE__ENCLOSURE');
            $this->_assign_category('C__CATG__LOCATION', 'C__OBJTYPE__ENCLOSURE');

            $this->_assign_category('C__CATG__SMVIEW_PATCH_PARKING_DECK', 'C__OBJTYPE__PATCH_PANEL');
            $this->_assign_category('C__CATG__SMVIEW_PATCH_PARKING_DECK', 'C__OBJTYPE__SERVER');

            // also assign a category to test SM-VIEW Feed
            $this->_assign_category('C__CATG__SMVIEW_FEED_JIRA', 'C__OBJTYPE__APPLICATION');

            // add objects
            $patch01_obj_id = $this->_add_object('First-Patch-Panel', C__OBJTYPE__PATCH_PANEL);
            $patch02_obj_id = $this->_add_object('Second-Patch-Panel', C__OBJTYPE__PATCH_PANEL);
            $patch03_obj_id = $this->_add_object('Third-Patch-Panel', C__OBJTYPE__PATCH_PANEL);
            $rack01_obj_id = $this->_add_object('First-Rack', C__OBJTYPE__ENCLOSURE);
            $rack02_obj_id = $this->_add_object('Second-Rack', C__OBJTYPE__ENCLOSURE);

            // add connectors to first patch panel
            $patchConnectors01 = [];
            $linkConnectors01 = [];
            for ($i = 1; $i <= 4; $i++) {
                $patch = $this->_add_connector($patch01_obj_id, $i, C__CONNECTOR__OUTPUT);
                $patchConnectors01[] = $patch;
                $linkConnectors01[] = $this->_add_connector($patch01_obj_id, $i, C__CONNECTOR__INPUT, $patch);
            }

            // add connectors to second patch panel
            $patchConnectors02 = [];
            $linkConnectors02 = [];
            for ($i = 1; $i <= 2; $i++) {
                $patch = $this->_add_connector($patch02_obj_id, $i, C__CONNECTOR__OUTPUT);
                $patchConnectors02[] = $patch;
                $linkConnectors02[] = $this->_add_connector($patch02_obj_id, $i, C__CONNECTOR__INPUT, $patch);
            }

            // add link connections
            foreach ($linkConnectors02 as $index => $connector_b) {
                $connector_a = $linkConnectors01[$index];
                $this->_assign_connectors($connector_a, $connector_b);
            }

            // add first grid and assign connectors
            $template01_id = $this->_add_grid_template('Grid01');
            $grid01_list_id = $this->_add_grid($patch01_obj_id, $template01_id, 'Grid01');
            foreach ($patchConnectors01 as $index => $connector_id) {
                $connector_template_id = $this->_add_connector_template($template01_id, ($index + 1));
                $this->_add_connector_instance($grid01_list_id, $connector_template_id, $connector_id);
            }

            // add second grid and assign connectors
            $template02_id = $this->_add_grid_template('Grid02');
            $grid02_list_id = $this->_add_grid($patch02_obj_id, $template02_id, 'Grid02');
            foreach ($patchConnectors02 as $index => $connector_id) {
                $connector_template_id = $this->_add_connector_template($template02_id, ($index + 1));
                $this->_add_connector_instance($grid02_list_id, $connector_template_id, $connector_id);
            }

            // setup racks with location and form factor
            $this->_setup_rack($rack01_obj_id);
            $this->_setup_rack($rack02_obj_id);
            $this->_setup_rack_half_he($rack02_obj_id);

            // add devices with location, rack and form factor
            $this->_assign_device_to_rack($rack01_obj_id, $patch01_obj_id, 4);
            $this->_assign_device_to_rack($rack01_obj_id, $patch02_obj_id, 8);
            $this->_assign_device_to_rack($rack02_obj_id, $patch03_obj_id, 4);

            $output->writeln("finished\n");
        } catch (Exception $e) {
            $output->writeln("");
            $output->writeln($e);
            $output->writeln("");
        }
        return 0;
    }

    /**
     * Returns an array of command usages
     *
     * @return array
     */
    public function getCommandUsages(): array
    {
        return [
            'smview_patch_test_data_generator -u admin -p admin -i 1',
        ];
    }

    private function _assign_category($catg_name, $obj_type)
    {
        $dao = isys_cmdb_dao_object::instance(isys_application::instance()->database);
        $sql = "SELECT isys_obj_type__id FROM isys_obj_type WHERE isys_obj_type__const = '$obj_type'";
        $ot = $dao->retrieve($sql)->get_row();
        $obj_type_id = $ot['isys_obj_type__id'];

        $sql = "SELECT isysgui_catg__id FROM isysgui_catg WHERE isysgui_catg__const = '$catg_name'";
        $catg = $dao->retrieve($sql)->get_row();
        $catg_id = $catg['isysgui_catg__id'];

        $sql = "SELECT isys_obj_type_2_isysgui_catg__isys_obj_type__id FROM isys_obj_type_2_isysgui_catg
                WHERE isys_obj_type_2_isysgui_catg__isys_obj_type__id = $obj_type_id
                AND isys_obj_type_2_isysgui_catg__isysgui_catg__id = $catg_id";
        try {
            $row = $dao->retrieve($sql)->get_row();
        } catch (Exception $e) {
            $this->_output->writeln("Could not execute query: $sql\n");
            return;
        }
        if (! empty($row)) {
            // category already assigned
            return;
        }

        $sql = "INSERT INTO isys_obj_type_2_isysgui_catg
                SET isys_obj_type_2_isysgui_catg__isys_obj_type__id = $obj_type_id,
                    isys_obj_type_2_isysgui_catg__isysgui_catg__id = $catg_id";
        $dao->update($sql);
    }

    private function _add_object($name, $obj_type)
    {
        $dao = isys_cmdb_dao_object::instance(isys_application::instance()->database);
        $id = $dao->get_obj_id_by_title($name, $obj_type);
        if ($id === false) {
            $id = $dao->create_object($name, $obj_type);
            $this->_output->writeln("added new object: '$name'\n");

        }
        return $id;
    }

    private function _add_connector($obj_id, $index, $type = C__CONNECTOR__OUTPUT, $sibling = null)
    {
        $connector_dao = new isys_cmdb_dao_category_g_connector(isys_application::instance()->database);
        return $connector_dao->create(
            $obj_id,
            $type,
            null,
            null,
            '0' . $index,
            '',
            $sibling);
    }

    private function _assign_connectors($connection_a, $connection_b)
    {
        $l_cable_object_id = isys_cmdb_dao_cable_connection::add_cable();
        $dao = isys_cmdb_dao_cable_connection::instance(isys_application::instance()->database);
        $l_cable_con = $dao->add_cable_connection($l_cable_object_id);

        $l_dao = isys_cmdb_dao_cable_connection::instance(isys_application::instance()->database);
        return $l_dao->save_connection($connection_a, $connection_b, $l_cable_con, $connection_a);
    }

    private function _add_grid_template($name)
    {
        $template_dao = new isys_cmdb_dao_smview_patch_grid_template(isys_application::instance()->database);
        $row = $template_dao->get_object('title', $name);
        if (! empty($row)) {
            return $row['id'];
        }

        $this->_output->writeln("added new grid template: '$name'\n");
        $params = [
            'title' => $name,
            'column_count' => 4,
            'row_count' => 2,
            'version' => 1,
            'is_template' => 1,
        ];
        return $template_dao->create_object($params);
    }

    private function _add_grid($obj_id, $template_id, $name)
    {
        $grid_dao = new isys_cmdb_dao_smview_patch_grid(isys_application::instance()->database);
        $row = $grid_dao->get_object('obj_id', $obj_id);
        if (! empty($row)) {
            return $row['id'];
        }

        $this->_output->writeln("added new grid: '$name'\n");
        $params = [
            'obj_id' => $obj_id,
            'description' => $name,
            'grid_template_id' => $template_id,
        ];
        return $grid_dao->create_object($params);
    }

    private function _add_connector_template($template_id, $index)
    {
        $picture_dao = new isys_cmdb_dao_smview_patch_picture(isys_application::instance()->database);
        $type_row = $picture_dao->get_object('title', 'RJ-45');

        $connector_template_dao = new isys_cmdb_dao_smview_patch_connector_template(isys_application::instance()->database);
        $items = $connector_template_dao->filter_objects(array(
            'grid_template_id' => $template_id,
            'pos_x' => $index,
            'pos_y' => 1,
        ));
        if (! empty($items)) {
            $item = $items[0];
            return $item['id'];
        }

        $this->_output->writeln("add connector template: $index");
        $params = [
            'grid_template_id' => $template_id,
            'connector_title' => 'RJ-45',
            'connector_type' => '1',
            'connector_type_id' => $type_row['id'],
            'title' => 'Connector ' . $index,
            'number' => $index,
            'pos_x' => $index,
            'pos_y' => 1,
            'angle' => 'F',
        ];



        return $connector_template_dao->create_object($params);
    }

    private function _add_connector_instance($grid_list_id, $connector_template_id, $connector_id) {
        $connector_dao = new isys_cmdb_dao_smview_patch_grid_connector(isys_application::instance()->database);
        $params = [
            'grid_list_id'          => $grid_list_id,
            'connector_template_id' => $connector_template_id,
            'connector_id'          => $connector_id,
        ];
        $connector_dao->create_object($params);
    }

    private function _setup_rack($obj_id)
    {
        $dao = new isys_cmdb_dao_category_g_formfactor(isys_application::instance()->database);
        $data = [
            'isys_obj__id' => $obj_id,
            'rackunits'    => 14,
        ];
        $dao->create_data($data);

        $l_dao = new isys_cmdb_dao_category_g_location(isys_application::instance()->database);
        $locations = $l_dao->get_data_as_array();
        $l = $locations[0];
        $l_dao->create($obj_id, $l['isys_catg_location_list__id']);
    }

    private function _setup_rack_half_he($obj_id)
    {
        $dao = new isys_cmdb_dao_smview_patch_formfactor(isys_application::instance()->database);
        $dao->create_object([
            'obj_id' => $obj_id,
            'half_he' => 1,
        ]);
    }

    private function _assign_device_to_rack($rack_obj_id, $device_obj_id, $position)
    {
        $l_dao = new isys_cmdb_dao_category_g_location(isys_application::instance()->database);
        $locations = $l_dao->get_data_as_array(null, $rack_obj_id);
        if (empty($locations)) {
            $this->_output->writeln("could not find location of rack\n");
            return;
        }

        $locations = $l_dao->get_data_as_array(null, $device_obj_id);
        if (empty($locations)) {
            $this->_output->writeln("assign device to rack\n");
            $l_dao->create($device_obj_id, $rack_obj_id);

            $loc = isys_cmdb_dao_location::instance(isys_application::instance()->database);
            $loc->update_position($device_obj_id, 3, 1, $position);
        }
    }
}
