<?php

namespace idoit\Module\Patch\DependentObjectManager;

use Exception;
use isys_application;
use isys_cmdb_dao_category_g_global;
use isys_cmdb_dao_smview_patch_analyzer_preset;
use isys_component_template_navbar;
use isys_exception_dao;
use isys_exception_database;

/**
 * Analyzer Preset Manager class.
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2021 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class AnalyzerPresetManager extends AbstractManager
{

    /**
     * Property mapping.
     *
     * @var array
     */
    protected $_mapping = array(
        'description'    => 'C__CATG__SMVIEW_PATCH_DESCRIPTION',
        'show_rack_view' => 'C__CATG__SMVIEW_PATCH_SHOW_RACK_VIEW',
        'object_types'   => 'C__CATG__SMVIEW_PATCH_OBJECT_TYPES',
    );

    /**
     * List of object types.
     *
     * @var array
     */
    private $_object_types;

    /**
     * Constructor.
     *
     * @throws isys_exception_database
     */
    public function __construct()
    {
        $this->_dao = new isys_cmdb_dao_smview_patch_analyzer_preset(isys_application::instance()->database);
        $this->_object_types = $this->_get_object_types();
    }

    /**
     * Get all available object types.
     *
     * @return array
     * @throws isys_exception_database
     */
    private function _get_object_types()
    {
        $object_types = [];
        $lang = isys_application::instance()->container->get('language');
        $sql = 'SELECT * from isys_obj_type';
        $dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
        $res = $dao->retrieve($sql);
        while ($row = $res->get_row()) {
            $object_types[$row['isys_obj_type__id']] = $lang->get($row['isys_obj_type__title']);
        }
        asort($object_types);
        return $object_types;
    }

    /**
     * Process generator listing.
     *
     * @throws Exception
     */
    public function processListing()
    {
        $lang = isys_application::instance()->container->get('language');
        $title = $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__ANALYZER_PRESET_TREE_TITLE');
        $headers = [
            "id"           => "ID",
            "description"  => $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__DESCRIPTION'),
            "object_types" => $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__OBJECT_TYPES'),
        ];
        $this->_processListing($title, $headers);
    }

    /**
     * Modify row method will be called for each row to alter its content.
     *
     * @param  array &$p_row
     * @throws isys_exception_database
     */
    public function modify_row(&$p_row)
    {
        $assigned_object_types = $this->get_assigned_object_types($p_row['id']);
        $object_types = [];
        foreach ($assigned_object_types as $assigned_object_type) {
            $object_types[] = $this->_object_types[$assigned_object_type];
        }
        $p_row['object_types'] = implode(', ', $object_types);
    }

    /**
     * Handle creation of new objects.
     *
     * @throws isys_exception_database
     */
    public function handleNewMode()
    {
        parent::handleNewMode();
        $this->_assign_object_types();
        $l_rules['C__CATG__SMVIEW_PATCH_SHOW_RACK_VIEW']['p_arData'] = get_smarty_arr_YES_NO();
        isys_application::instance()->template->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);
    }

    /**
     * Process page.
     *
     * @param  int $element_id
     * @param  array $additional_fields
     * @throws isys_exception_database
     */
    public function processPage($element_id, $additional_fields = [])
    {
        parent::processPage($element_id, $additional_fields);
        isys_component_template_navbar::getInstance()
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__DELETE'));

        $this->_assign_object_types($element_id);
        $row = $this->_dao->get_object('id', $element_id);
        $show_rack_view = (int) $row['show_rack_view'];
        $l_rules['C__CATG__SMVIEW_PATCH_SHOW_RACK_VIEW']['p_strSelectedID'] = $show_rack_view >= 0 ? $row['show_rack_view'] : null;
        $l_rules['C__CATG__SMVIEW_PATCH_SHOW_RACK_VIEW']['p_arData'] = get_smarty_arr_YES_NO();
        isys_application::instance()->template->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);
    }

    /**
     * Get assigned object types.
     *
     * @param  int $preset_id
     * @return array
     * @throws isys_exception_database
     */
    public function get_assigned_object_types($preset_id)
    {
        $dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
        $l_sql = 'SELECT isys_catg_smview_patch_preset_2_isys_obj_type__obj_type_id
                  FROM isys_catg_smview_patch_preset_2_isys_obj_type
                  WHERE isys_catg_smview_patch_preset_2_isys_obj_type__preset_id = ' . $dao->convert_sql_id($preset_id);
        $l_res = $dao->retrieve($l_sql);
        $l_return = [];
        while ($l_row = $l_res->get_row()) {
            $l_return[] = $l_row['isys_catg_smview_patch_preset_2_isys_obj_type__obj_type_id'];
        }
        return $l_return;
    }

    /**
     * Save object.
     *
     * @param  int $instance_id
     * @param  array $optionalParams
     * @throws isys_exception_dao
     */
    public function saveObject($instance_id = null, $optionalParams = [])
    {
        parent::saveObject($instance_id, $optionalParams);
        $instance_var = isys_application::instance()->template->get_template_vars("instance_id");
        $instance_id = is_int($instance_var) ? $instance_var : $instance_var['value'];

        $posted_object_types = $_POST['C__CATG__SMVIEW_PATCH_OBJECT_TYPES'];
        $this->_remove_assigned_object_types($instance_id);
        $this->_save_assigned_object_types($instance_id, $posted_object_types);
    }

    /**
     * Assign object types to template.
     *
     * @param  int|null $element_id
     * @param  array $assigned_object_types
     * @throws isys_exception_database
     */
    private function _assign_object_types($element_id = null, $assigned_object_types = [])
    {
        if (empty($assigned_object_types) && $element_id) {
            $assigned_object_types = $this->get_assigned_object_types($element_id);
        }

        $unassigned = $assigned = [];
        foreach ($this->_object_types as $id => $title) {
            if (in_array($id, $assigned_object_types)) {
                $assigned[$id] = $title;
            } else {
                $unassigned[$id] = $title;
            }
        }

        asort($unassigned);
        asort($assigned);
        isys_application::instance()->template
            ->assign('unassigned_object_types', $unassigned)
            ->assign('assigned_object_types', $assigned);
    }

    /**
     * Delete object.
     *
     * @param  int|null $element_id
     * @throws isys_exception_dao
     */
    public function deleteObject($element_id = null)
    {
        parent::deleteObject($element_id);
        $this->_remove_assigned_object_types($element_id);
    }

    /**
     * Deletes tags assignment
     *
     * @param  int $preset_id
     * @return boolean
     * @throws isys_exception_dao
     */
    private function _remove_assigned_object_types($preset_id)
    {
        $dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
        $l_sql = 'DELETE FROM isys_catg_smview_patch_preset_2_isys_obj_type
                  WHERE isys_catg_smview_patch_preset_2_isys_obj_type__preset_id = ' . $dao->convert_sql_id($preset_id);
        return ($dao->update($l_sql) && $dao->apply_update());
    }

    /**
     * Assigns tags to object
     *
     * @param  int $preset_id
     * @param  array $object_types
     * @return boolean
     * @throws Exception
     * @throws isys_exception_dao
     */
    private function _save_assigned_object_types($preset_id, $object_types)
    {
        $dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
        $l_return = true;
        if (is_array($object_types) && count($object_types)) {
            $l_inserts = [];
            foreach ($object_types as $obj_type) {
                if (is_numeric($obj_type)) {
                    $l_inserts[] = '(' . $dao->convert_sql_id($preset_id) . ', ' . $dao->convert_sql_id($obj_type) . ')';
                }
            }

            if (count($l_inserts)) {
                $l_return = ($dao->update('INSERT INTO isys_catg_smview_patch_preset_2_isys_obj_type(
                                                  isys_catg_smview_patch_preset_2_isys_obj_type__preset_id,
                                                  isys_catg_smview_patch_preset_2_isys_obj_type__obj_type_id)
                                                  VALUES ' . implode(',', $l_inserts)) && $dao->apply_update());
            }
        }
        return $l_return;
    }
}
