<?php

namespace idoit\Module\Patch\DependentObjectManager;

use Exception;
use isys_application;
use isys_auth;
use isys_auth_auth;
use isys_cmdb_dao_smview_patch_abstract;
use isys_cmdb_dao_smview_patch_picture;
use isys_component_template_navbar;
use isys_exception_dao;
use isys_exception_database;
use UnexpectedValueException;

/**
 * Image Mapping Manager class.
 *
 * @package i-doit
 * @subpackage smview_patch
 * @author Steve Groeber <steve.groeber@shd-online.de>
 * @copyright SHD System-Haus-Dresden GmbH
 * @license http://www.i-doit.com/license
 */
class ImageMappingManager extends AbstractManager
{
    
    /**
     * DAO.
     *
     * @var isys_cmdb_dao_smview_patch_abstract
     */
    protected $_dao;
    
    /**
     * Property mapping.
     *
     * @var array
     */
    protected $_mapping = array(
        'title'  => 'C__CATG__SMVIEW_PATCH_TITLE',
        'obj_id' => 'C__CATG__FILE_OBJ_FILE',
    );
    
    /**
     * Constructor.
     */
    public function __construct()
    {
        $this->_dao = new isys_cmdb_dao_smview_patch_picture(isys_application::instance()->database);
    }

    /**
     * Process generator listing.
     *
     * @throws Exception
     */
    public function processListing()
    {
        $lang = isys_application::instance()->container->get('language');
        $title = $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__IMAGE_MAPPING_TREE_TITLE');
        $headers = [
            "id"                => "ID",
            "title"             => $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__CONNECTION_TYPE_TITLE'),
            "filename_original" => $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__CONNECTION_TYPE_FILE')
        ];
        $this->_processListing($title, $headers);
        isys_component_template_navbar::getInstance()
                ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__NEW'))
                ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__DELETE'));
    }

    /**
     * Modify row method will be called for each row to alter its content.
     *
     * @param  array &$p_row
     * @throws isys_exception_database
     */
    public function modify_row(&$p_row)
    {
        $row = $this->_dao->get_object('id', $p_row['id']);
        $url = $this->_dao->get_file_url($row['obj_id']);

        if (! empty($url)) {
            $alt = $p_row['filename_original'];
            $title = $row['title'];
            $img = "<img src='$url' alt='$alt' title='$title' style='max-width: 128px; max-height: 32px;' />";
            $p_row['filename_original'] = $img;
        }
    }

    /**
     * Process page.
     *
     * @param  int $element_id
     * @param  array $additional_fields
     * @throws isys_exception_database
     */
    public function processPage($element_id, $additional_fields = [])
    {
        global $g_config;
        
        parent::processPage($element_id, $additional_fields);
        
        isys_component_template_navbar::getInstance()
                ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__DELETE'));
        
        $row = $this->_dao->get_object('id', $element_id);
        $url = $this->_dao->get_file_url($row['obj_id']);
        isys_application::instance()->template
            ->assign("dir_tools", $g_config["www_dir"] . "src/tools/")
            ->assign('url', $url);
    }

    /**
     * Save object.
     *
     * @param  int $instance_id
     * @param  array $optionalParams
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function saveObject($instance_id = null, $optionalParams = [])
    {
        isys_component_template_navbar::getInstance()
            ->set_active(isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, "MODULE/C__MODULE__SMVIEW_PATCH"),
                defined_or_default('C__NAVBAR_BUTTON__EDIT'));
        
        $row = $this->_dao->get_object('id', $instance_id);
        $obj_id = ! empty($_POST['C__CATG__FILE_OBJ_FILE__HIDDEN']) ? $_POST['C__CATG__FILE_OBJ_FILE__HIDDEN'] : null;
        $params = [
            'obj_id' => $obj_id,
        ];
        
        try {
            if (empty($row['obj_id'])) {
                $params['connection_type_id'] = $instance_id;
                $instance_id = $this->_dao->create_object($params);
            } else {
                if (! empty($obj_id)) {
                    $this->_dao->update_object($instance_id, $params);
                } else {
                    $this->_dao->delete_object($instance_id);
                }
            }
        } catch (UnexpectedValueException $e) {
            $_POST["navMode"] = defined_or_default('C__NAVMODE__EDIT');
        }
        
        $l_rules = [];
        foreach ($this->_mapping as $db => $field) {
            $value = $params[$db];
            $l_rules[$field]['p_strValue'] = $value;
        }

        isys_application::instance()->template->assign("instance_id", $instance_id)
            ->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);
    }
}
