<?php

namespace idoit\Module\Patch\Exchange;

use isys_application;
use isys_cmdb_dao_category_g_global;
use isys_cmdb_dao_smview_patch_connector_template;
use isys_cmdb_dao_smview_patch_grid_template;
use isys_cmdb_dao_smview_patch_picture;
use isys_exception_database;
use isys_exception_general;
use ZipArchive;

/**
 * Exchange Exporter class.
 *
 * @package i-doit
 * @subpackage smview_patch
 * @author Steve Groeber <steve.groeber@shd-online.de>
 * @copyright SHD System-Haus-Dresden GmbH
 * @license http://www.i-doit.com/license
 */
class GridExporter
{

    const GRID_FILE_NAME = 'grids.json';
    const CONNECTION_TYPE_FILE_NAME = 'connection_types.json';

    /**
     * Export selected grids as ZIP file.
     *
     * @param  array $ids
     * @return null
     * @throws isys_exception_general
     * @throws isys_exception_database
     */
    public function export($ids)
    {
        global $g_dirs;

        if (empty($ids)) {
            return null;
        }

        $temp_dir = $g_dirs['temp'];

        // create zip file
        $archive_file_name = 'grid.zip';
        $archive_path = $temp_dir . $archive_file_name;
        $zip = new ZipArchive();
        if ($zip->open($archive_path, ZIPARCHIVE::CREATE) !== true) {
            throw new isys_exception_general('could not create zip file');
        }

        // add grid data
        $data = $this->_get_grid_templates($ids);
        $json = json_encode($data['grids']);
        $json_file = $temp_dir . self::GRID_FILE_NAME;
        file_put_contents($json_file, $json);
        $zip->addFile($json_file, self::GRID_FILE_NAME);

        // add connection types
        $connection_types = $this->_get_connection_types();
        $json = json_encode($connection_types);
        $json_file = $temp_dir . self::CONNECTION_TYPE_FILE_NAME;
        file_put_contents($json_file, $json);
        $zip->addFile($json_file, self::CONNECTION_TYPE_FILE_NAME);

        // add image files
        $image_path = $g_dirs['fileman']['target_dir'];
        foreach ($data['types'] as $type) {
            $source_file = $image_path . $type['filename'];
            $zip->addFile($source_file, $type['filename']);
        }
        $zip->close();

        // setup header for download
        header("Content-type: application/zip");
        header("Content-Disposition: attachment; filename=$archive_file_name");
        header("Content-length: " . filesize($archive_path));
        header("Pragma: no-cache");
        header("Expires: 0");
        readfile($archive_path);

        unlink($json_file);
        unlink($archive_path);
        die;
    }

    /**
     * Get selected grids as an exchangeable array.
     *
     * @param  array $ids
     * @return array
     * @throws isys_exception_database
     */
    private function _get_grid_templates($ids)
    {
        $grid_template_dao = new isys_cmdb_dao_smview_patch_grid_template(isys_application::instance()->database);
        $connector_template_dao = new isys_cmdb_dao_smview_patch_connector_template(isys_application::instance()->database);
        $picture_dao = new isys_cmdb_dao_smview_patch_picture(isys_application::instance()->database);

        $grids = [];
        $connector_types = [];
        foreach ($ids as $id) {
            $grid_template = $grid_template_dao->get_object('id', $id);
            $connector_templates = $connector_template_dao->filter_objects(array('grid_template_id' => $id));

            $connectors = [];
            foreach ($connector_templates as $ct) {
                unset($ct['id']);
                unset($ct['grid_template_id']);

                $type_id = $ct['connector_type_id'];
                if (empty($connector_types[$type_id])) {
                    $pic = $picture_dao->get_object('id', $type_id);
                    $connector_types[$type_id] = $pic;
                }
                $pic = $connector_types[$type_id];
                $ct['connector_type_const'] = ! empty($pic) ? $pic['const'] : null;
                $ct['filename'] = ! empty($pic) ? $pic['filename'] : null;
                $connectors[] = $ct;
            }

            $grid = $grid_template;
            $grid['ports'] = $connectors;
            $grids[$id] = $grid;
        }
        return [
            'grids' => $grids,
            'types' => $connector_types
        ];
    }

    /**
     * Get all connection types.
     *
     * @return array
     * @throws isys_exception_database
     */
    private function _get_connection_types()
    {
        $sql = 'SELECT * FROM isys_connection_type as t';
        $dao = new isys_cmdb_dao_category_g_global(isys_application::instance()->database);
        $res = $dao->retrieve($sql);
        $connection_types = [];
        while ($row = $res->get_row()) {
            $connection_types[] = [
                'id'          => $row['isys_connection_type__id'],
                'title'       => $row['isys_connection_type__title'],
                'description' => $row['isys_connection_type__description'],
                'const'       => $row['isys_connection_type__const'],
            ];
        }
        return $connection_types;
    }
}
