<?php

namespace idoit\Module\Patch\Exchange;

use isys_application;
use isys_cmdb_dao_category_s_file;
use isys_cmdb_dao_dialog_admin;
use isys_cmdb_dao_smview_patch_connector_template;
use isys_cmdb_dao_smview_patch_grid_template;
use isys_cmdb_dao_smview_patch_picture;
use isys_exception_dao;
use isys_exception_database;
use isys_exception_general;
use isys_exception_validation;
use ZipArchive;

/**
 * Exchange Importer class.
 *
 * @package i-doit
 * @subpackage smview_patch
 * @author Steve Groeber <steve.groeber@shd-online.de>
 * @copyright SHD System-Haus-Dresden GmbH
 * @license http://www.i-doit.com/license
 */
class GridImporter
{

    const IMPORT_DIR = 'grid_import';

    /**
     * Unzip specified file to temporary directory.
     *
     * @param  string $filename Name of uploaded file
     * @param  string $dir Path to uploaded file
     * @return string|null
     * @throws isys_exception_general
     */
    public function unzip($filename, $dir)
    {
        global $g_dirs;

        // add temporary directory
        $temp_dir = $g_dirs['temp'] . self::IMPORT_DIR . DIRECTORY_SEPARATOR;
        if (! is_dir($temp_dir)) {
            mkdir($temp_dir);
        } else {
            unlink($temp_dir);
        }

        // unzip archive
        $zip = new ZipArchive();
        $zip->open($dir . $filename);
        $zip->extractTo($temp_dir);

        // check if grid file exists
        if (! is_file($temp_dir . GridExporter::GRID_FILE_NAME)) {
            unlink($temp_dir);
            throw new isys_exception_general('could not find grid file');
        }
        return $temp_dir;
    }

    /**
     * Get grids from uploaded archive.
     *
     * @param  string $dir Path to unzipped archive
     * @return array|null
     * @throws isys_exception_general
     * @throws isys_exception_database
     */
    public function get_grids($dir)
    {
        if (! is_file($dir . GridExporter::GRID_FILE_NAME)) {
            throw new isys_exception_general('could not find grid file');
        }

        $json_data = file_get_contents($dir . GridExporter::GRID_FILE_NAME);
        $grids = json_decode($json_data, true);

        $grid_template_dao = new isys_cmdb_dao_smview_patch_grid_template(isys_application::instance()->database);
        foreach ($grids as $id => $grid) {
            $templates = $grid_template_dao->filter_objects(array('title' => $grid['title'], 'is_template' => 1));
            $grids[$id]['is_in_db'] = count($templates) > 0;
        }

        return $grids;
    }

    /**
     * Import selected grids.
     *
     * @param  string $dir Path to unzipped archive
     * @param  array $ids List of selected grid IDs
     * @throws isys_exception_validation
     * @throws isys_exception_general
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function import($dir, $ids)
    {
        if (empty($ids)) {
            return;
        }

        $grid_template_dao = new isys_cmdb_dao_smview_patch_grid_template(isys_application::instance()->database);
        $grids = $this->get_grids($dir);
        $connection_types = $this->_get_connection_types($dir);
        foreach ($grids as $grid) {
            if (! in_array($grid['id'], $ids)) {
                continue;
            }

            $params = [
                'title'             => $grid['title'],
                'column_count'      => $grid['column_count'],
                'row_count'         => $grid['row_count'],
                'column_count_back' => $grid['column_count_back'],
                'row_count_back'    => $grid['row_count_back'],
            ];
            $optional_params = [
                'version'     => isset($grid['version']) ? $grid['version'] : null,
                'is_template' => isset($grid['is_template']) ? $grid['is_template'] : null,
            ];
            $grid_id = $grid_template_dao->create_object($params, $optional_params);
            $this->_import_ports($grid_id, $grid['ports'], $dir, $connection_types);
        }

        unlink($dir);
    }

    /**
     * Get grids from uploaded archive.
     *
     * @param  string $dir Path to unzipped archive
     * @return array|null
     * @throws isys_exception_general
     */
    private function _get_connection_types($dir)
    {
        if (! is_file($dir . GridExporter::CONNECTION_TYPE_FILE_NAME)) {
            throw new isys_exception_general('could not find connection type file');
        }

        $json_data = file_get_contents($dir . GridExporter::CONNECTION_TYPE_FILE_NAME);
        $types = json_decode($json_data, true);

        $connection_types = [];
        foreach ($types as $type) {
            $connection_types[$type['const']] = $type;
        }
        return $connection_types;
    }

    /**
     * Import ports of a grid.
     *
     * @param  int $grid_id
     * @param  array $ports
     * @param  string $dir
     * @param  array $connection_types
     * @throws isys_exception_general
     * @throws isys_exception_validation
     * @throws isys_exception_database
     * @throws isys_exception_dao
     */
    private function _import_ports($grid_id, $ports, $dir, $connection_types)
    {
        $connector_template_dao = new isys_cmdb_dao_smview_patch_connector_template(isys_application::instance()->database);
        foreach ($ports as $port) {
            $type = $this->_get_type_by_const($port['connector_type_const'], $connection_types);

            // add port
            $params = array (
                'grid_template_id'  => $grid_id,
                'connector_title'   => $port['connector_title'],
                'connector_type'    => $port['connector_type'],
                'connector_type_id' => $type['id'],
                'pos_x'             => $port['pos_x'],
                'pos_y'             => $port['pos_y'],
            );
            $optional_params = array(
                'title'             => isset($port['title']) ? $port['title'] : null,
                'number'            => isset($port['number']) ? $port['number'] : null,
                'row_title'         => isset($port['row_title']) ? $port['row_title'] : null,
                'label_orientation' => isset($port['label_orientation']) ? $port['label_orientation'] : null,
                'status'            => isset($port['status']) ? $port['status'] : null,
                'angle'             => isset($port['angle']) ? $port['angle'] : null,
            );
            $connector_template_dao->create_object($params, $optional_params);

            $this->_add_image_to_type($type, $dir, $port['filename']);
        }
    }

    /**
     * Get connector type by const.
     *
     * @param  string $const
     * @param  array $connection_types
     * @return array|null
     * @throws isys_exception_general
     * @throws isys_exception_database
     */
    private function _get_type_by_const($const, $connection_types)
    {
        $picture_dao = new isys_cmdb_dao_smview_patch_picture(isys_application::instance()->database);
        $type = $picture_dao->get_object('const', $const);
        if (empty($type)) {
            // import connection type
            if (empty($connection_types[$const])) {
                throw new isys_exception_general('could not find connector type: ' . $const);
            }

            $t = $connection_types[$const];
            $dao = isys_cmdb_dao_dialog_admin::instance(isys_application::instance()->database);
            $dao->create('isys_connection_type', $t['title'], null, $t['const'], 2, null, null, $t['description']);

            $type = $picture_dao->get_object('const', $const);
        }
        return $type;
    }

    /**
     * Assign image to connector if it is not assigned yet.
     *
     * @param  array $type
     * @param  string $dir
     * @param  string $upload_filename
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function _add_image_to_type($type, $dir, $upload_filename)
    {
        global $g_comp_session, $g_dirs;

        if (! empty($type['filename'])) {
            // type has already a file assigned
            return;
        }

        // check file name
        $filename = $upload_filename;
        $path = $g_dirs['fileman']['target_dir'] . $filename;
        if (is_file($path)) {
            // get unique filename
            $filename = time() . '__' . $filename;
            $path = $g_dirs['fileman']['target_dir'] . $filename;
        }

        // copy file to target directory
        copy($dir . $upload_filename, $path);
        $md5_hash = md5_file($path);

        // create new file object
        $dao = isys_cmdb_dao_category_s_file::instance(isys_application::instance()->database);
        $obj_id = $dao->create_object($type['title'], C__OBJTYPE__FILE);

        // create database entries
        $l_physical_id = $dao->create_physical_file($filename, $upload_filename, $md5_hash, $g_comp_session->get_user_id());
        $l_version_id = null;
        if ($l_physical_id > 0) {
            // add new version
            $l_version_id = $dao->create_version($obj_id, $l_physical_id, $type['title'], $type['title']);
        }

        // update file list link
        $l_cats_id = $dao->create_connector('isys_cats_file_list', $obj_id);
        $dao->update_cats_file_list($l_cats_id, $l_version_id);

        // assign image to connector type
        $picture_dao = new isys_cmdb_dao_smview_patch_picture(isys_application::instance()->database);
        $row = $picture_dao->get_object('id', $type['id']);
        $params = [
            'obj_id' => $obj_id,
        ];
        if (empty($row['obj_id'])) {
            $params['connection_type_id'] = $type['id'];
            $picture_dao->create_object($params);
        } else {
            $picture_dao->update_object($type['id'], $params);
        }
    }
}
