<?php

namespace idoit\Module\Patch\Importer;

use Exception;
use idoit\Context\Context;
use isys_application;
use isys_auth_dao;
use isys_cmdb_dao_category;
use isys_cmdb_dao_object_type;
use isys_exception_cmdb;
use isys_exception_database;
use isys_exception_general;
use isys_export_cmdb_object;
use isys_factory_log;
use isys_import_handler_cmdb;
use isys_log;
use stdClass;

/**
 * Importer Duplicator class.
 *
 * @package i-doit
 * @subpackage smview_patch
 * @author Steve Groeber <steve.groeber@shd-online.de>
 * @copyright SHD System-Haus-Dresden GmbH
 * @license http://www.i-doit.com/license
 */
class Duplicator
{

    /**
     * Duplicate object with new name.
     *
     * @param int    $objId
     * @param string $objName
     *
     * @return boolean|null
     * @throws isys_exception_cmdb
     * @throws isys_exception_general
     * @throws isys_exception_database
     */
    public function duplicate($objId, $objName)
    {
        $l_cmdb_dao = isys_cmdb_dao_object_type::instance(isys_application::instance()->database);
        $l_object_type = $l_cmdb_dao->get_objTypeID($objId);
        $categories = $this->_get_categories($l_object_type);

        $global_categories = $categories['categories'];
        $special_categories = $categories['specificCategories'];

        // remove slot and devices cats
        $special_categories = array_filter($special_categories, function($s) {
            return ! in_array($s, [
                defined_or_default('C__CATS__CHASSIS_SLOT'),
                defined_or_default('C__CATS__CHASSIS_DEVICES'),
            ]);
        });

        try {
            // Start logging:
            $l_log = isys_factory_log::get_instance('duplicate');
            $l_log->set_verbose_level(isys_log::C__NONE);

            // Retrieve categories:
            $l_categories = [];
            // Global categories:
            if (isset($global_categories) && !empty($global_categories)) {
                $l_catg = $global_categories;
                $l_categories[C__CMDB__CATEGORY__TYPE_GLOBAL] = $l_catg;
            }
            // Specific categories:
            if (isset($special_categories) && !empty($special_categories)) {
                $specificCategories = $special_categories;
                $l_categories[C__CMDB__CATEGORY__TYPE_SPECIFIC] = $specificCategories;
            }

            Context::instance()
                ->setImmutable(true)
                ->setContextTechnical(Context::CONTEXT_EXPORT_XML)
                ->setGroup(Context::CONTEXT_GROUP_DUPLICATE)
                ->setContextCustomer(Context::CONTEXT_DUPLICATE);

            // set fake request
            $request = new stdClass();
            $request->headers = new MyRequest();
            $request->object_id = $objId;
            if (! isys_application::instance()->container->initialized('request')) {
                isys_application::instance()->container->set('request', $request);
            }

            /*
             * WICHTIG: Diese Variable wird gesetzt, um im Cable-Modul die Objekt-PINs kopieren zu koennen, da
             * sonst keinerlei Referenz zum Template-Objekt vorhanden ist.
             * Siehe: isys_cmdb_dao_category_g_smview_cable_pin -> create_data()
             */
            isys_application::instance()->container->set('duplicator_request', $request);

            // Export data...
            $l_export = new isys_export_cmdb_object('isys_export_type_xml', isys_application::instance()->database);

            $l_parser = $l_export->export([$objId], $l_categories, defined_or_default('C__RECORD_STATUS__NORMAL'), true)
                ->parse();

            $l_data = $l_parser->get_export();

            unset($l_export, $l_parser);

            Context::instance()
                ->setImmutable(false)
                ->setContextTechnical(Context::CONTEXT_IMPORT_XML)
                ->setGroup(Context::CONTEXT_GROUP_DUPLICATE)
                ->setContextCustomer(Context::CONTEXT_DUPLICATE)
                ->setImmutable(true);

            // ...and import it:
            $l_import = new isys_import_handler_cmdb($l_log, isys_application::instance()->database);
            $l_import->set_option('update-object-changed', false)
                ->set_mode(isys_import_handler_cmdb::C__APPEND)
                ->set_multivalue_categories_mode(isys_import_handler_cmdb::C__APPEND);

            $l_import->set_update_globals();
            $l_import->load_xml_data($l_data);

            if ($l_import->parse() === false) {
                return false;
            }

            $l_import->prepare();

            // Set title:
            if ($objName !== null) {
                $l_import->set_replaced_title($objName, $objId);
            }

            if ($l_import->import() === false) {
                return false;
            }

            $l_object_ids = $l_import->get_object_ids();

            unset($l_import);

            $l_auth_dao = new isys_auth_dao(isys_application::instance()->database);

            // Call custom duplication methods:
            if ($l_cmdb_dao->has_cat($l_object_type, [
                'C__CATS__PERSON_GROUP',
                'C__CATS__PERSON'
            ])) {
                $l_auth_dao->duplicate($objId, $l_object_ids[$objId]);
            }

            unset($l_cmdb_dao);

            Context::instance()->setImmutable(false);

            return $l_object_ids[$objId];
        } catch (Exception $e) {
            isys_application::instance()->container->set('duplicator_request', null);
            throw $e;
        }
    }

    /**
     * Get categories of object type.
     *
     * @param  int $objTypeId
     * @return array
     * @throws isys_exception_database
     */
    private function _get_categories($objTypeId)
    {
        $l_cmdb_dao = isys_cmdb_dao_object_type::instance(isys_application::instance()->database);
        $globalCategories = [];
        $specificCategories = [];

        // Assign durable global categories:
        $l_cat = $l_cmdb_dao->get_durable_catg();
        $l_skipped_categories = isys_export_cmdb_object::get_skipped_categories(C__CMDB__CATEGORY__TYPE_GLOBAL);
        while ($l_row = $l_cat->get_row()) {
            if (isset($this->m_duplicate_blacklist[$l_row['isysgui_catg__id']])) {
                continue;
            }

            if (!class_exists($l_row['isysgui_catg__class_name'])) {
                continue;
            }

            $properties = (new $l_row['isysgui_catg__class_name'](isys_application::instance()->database))->get_properties();
            if (is_countable($properties) && count($properties) > 0 && isys_export_cmdb_object::isCategoryExportable($properties)) {
                $globalCategories[$l_row['isysgui_catg__id']] = $l_row['isysgui_catg__id'];
            }
        }

        // Assign global categories:
        $l_cat = $l_cmdb_dao->get_catg_by_obj_type($objTypeId);
        while ($l_row = $l_cat->get_row()) {
            // Don´t show skipped categories in GUI.
            if (isset($this->m_duplicate_blacklist[$l_row['isysgui_catg__id']])) {
                continue;
            }

            if (!class_exists($l_row['isysgui_catg__class_name'])) {
                continue;
            }

            if (array_key_exists($l_row['isysgui_catg__id'], $l_skipped_categories) || in_array($l_row['isysgui_catg__const'], ['C__CATG__VIRTUAL', 'C__CATG__GLOBAL'], true)) {
                // @todo C__CATG__GLOBAL is already set in durable catg above.
                continue;
            }

            if ($l_row['isysgui_catg__const'] == 'C__CATG__LOCATION') {
                // important: do not add location category - otherwise import hangs
                continue;
            }

            $properties = (new $l_row['isysgui_catg__class_name'](isys_application::instance()->database))->get_properties();
            if (is_countable($properties) && count($properties) > 0 && isys_export_cmdb_object::isCategoryExportable($properties)) {
                $globalCategories[$l_row['isysgui_catg__id']] = $l_row['isysgui_catg__id'];
            }
        }

        // Assign specific categories:
        $specificCategoryList = $l_cmdb_dao->get_specific_category($objTypeId, defined_or_default('C__RECORD_STATUS__NORMAL'), null, true)->__as_array();
        foreach ($specificCategoryList as $specificCategory) {
            if (!class_exists($specificCategory['isysgui_cats__class_name'])) {
                continue;
            }

            if ($specificCategory['isysgui_cats__type'] == isys_cmdb_dao_category::TYPE_FOLDER) {
                continue;
            }

            $properties = (new $specificCategory['isysgui_cats__class_name'](isys_application::instance()->database))->get_properties();
            if (is_countable($properties) && count($properties) > 0 && isys_export_cmdb_object::isCategoryExportable($properties)) {
                $specificCategories[$specificCategory['isysgui_cats__id']] = $specificCategory['isysgui_cats__id'];
            }
        }

        return [
            'categories'         => array_values($globalCategories),
            'specificCategories' => array_values($specificCategories),
        ];
    }
}

/**
 * Fake Request Class.
 *
 * @package idoit\Module\Patch\Importer
 */
class MyRequest
{
    function get($prop)
    {
        return $prop;
    }
}
