<?php

namespace idoit\Module\Patch\Importer;

use Exception;
use idoit\Component\Table\Filter\Operation\LocationPathOrderByOperation;
use isys_api_model_cmdb_category;
use isys_application;
use isys_cmdb_dao_category_g_accounting;
use isys_cmdb_dao_category_g_cable;
use isys_cmdb_dao_category_g_connector;
use isys_cmdb_dao_category_g_formfactor;
use isys_cmdb_dao_category_g_global;
use isys_cmdb_dao_category_g_ip;
use isys_cmdb_dao_category_g_location;
use isys_cmdb_dao_category_g_model;
use isys_cmdb_dao_category_g_relation;
use isys_cmdb_dao_category_g_smview_patch_end_device;
use isys_cmdb_dao_category_g_smview_patch_formfactor;
use isys_cmdb_dao_category_s_chassis;
use isys_cmdb_dao_category_s_chassis_slot;
use isys_cmdb_dao_category_s_chassis_view;
use isys_cmdb_dao_category_s_enclosure;
use isys_cmdb_dao_smview_patch_grid;
use isys_component_database;
use isys_component_template_language_manager;
use isys_exception_api;
use isys_exception_api_validation;
use isys_exception_cmdb;
use isys_exception_dao;
use isys_exception_dao_cmdb;
use isys_exception_database;
use isys_exception_general;
use isys_module_smview_patch;
use isys_settings;
use PhpOffice\PhpSpreadsheet\Reader\Xls;
use PhpOffice\PhpSpreadsheet\Reader\Xlsx;

/**
 * Importer Excel class.
 *
 * @package    i-doit
 * @subpackage smview_patch
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class Excel
{

    private static $PATH_SEPARATOR = ' > ';

    /*
     * Table header constants of import file.
     */
    const LOCATION = 'Standort';
    const RACK = 'Schrank';
    const HE = 'Einbauhöhe in 1/2HE';
    const PROTOTYPE = 'Prototyp';

    /**
     * Variable which holds the database component.
     *
     * @var isys_component_database
     */
    private $_database;

    /**
     * Path of imported file.
     *
     * @var string
     */
    private $_path;

    /**
     * Flag to simulate import.
     *
     * @var boolean
     */
    private $_simulate;

    /**
     * Translator.
     *
     * @var isys_component_template_language_manager
     */
    private $_language;

    /**
     * Log.
     *
     * @var array
     */
    private $_log = [];

    /**
     * List of locations.
     *
     * @var array
     */
    private $_locations;

    /**
     * Path to base template location.
     *
     * @var string
     */
    private $_template_location_path;

    /**
     * Excel constructor.
     *
     * @param  isys_component_database $database
     * @param  string $path
     * @param  boolean $simulate
     * @throws isys_exception_database
     */
    public function __construct($database, $path, $simulate = false)
    {
        global $g_dirs;

        $this->_database = $database;
        $this->_path = $g_dirs["temp"] . $path;
        $this->_simulate = $simulate;
        $this->_language = isys_application::instance()->container->get('language');
        $this->_locations = $this->_get_locations();
        $this->_template_location_path = $this->_get_template_location();
    }

    /**
     * Get a list of all locations.
     *
     * @return array
     * @throws isys_exception_database
     */
    private function _get_locations()
    {
        $dao = new isys_cmdb_dao_category_g_location($this->_database);
        $query = isys_cmdb_dao_category_g_location::build_location_path_query(8, 255, 40, '', true);
        $query = substr_replace($query, 'SELECT objMain.isys_obj__id, objMain.isys_obj__title, ', strpos($query, 'SELECT '), strlen('SELECT '));

        try {
            $res = $dao->retrieve($query);
        } catch (Exception $e) {
            // compatibility hack for version 1.19 (method parameters has changed)
            $query = isys_cmdb_dao_category_g_location::build_location_path_query(8, 40, '', true);
            $query = substr_replace($query, 'SELECT objMain.isys_obj__id, objMain.isys_obj__title, ', strpos($query, 'SELECT '), strlen('SELECT '));
            $res = $dao->retrieve($query);
        }

        $locations = [];
        while ($row = $res->get_row()) {
            if (empty($row['title']) || $row['isys_obj__title'] == 'Root location') {
                continue;
            }
            $parts = explode(' > ', $row['title']);

            // remove 'Root location'
            array_shift($parts);

            // add current object name
            $parts[] = $row['isys_obj__title'];

            $cleanParts = [];
            foreach ($parts as $part) {
                $cleanParts[] = trim($part);
            }

            $objId = (int) $row['isys_obj__id'];
            $locations[$objId] = implode(' > ', $cleanParts);
        }
        return $locations;
    }

    /**
     * Get path to template base location.
     *
     * @return string
     * @throws isys_exception_database
     */
    private function _get_template_location()
    {
        $location_paths = ['SHD', 'Templates'];
        $location_id = isys_settings::get(isys_module_smview_patch::IMPORT_TEMPLATE_PATH, null);
        if ($location_id) {
            $location_dao = new isys_cmdb_dao_category_g_location($this->_database);
            $query = $location_dao::build_location_path_query(
                LocationPathOrderByOperation::MAX_JOINS,
                1000,
                100,
                ' WHERE main.isys_catg_location_list__isys_obj__id = ' . $location_id,
                true
            );
            $query = substr_replace($query, 'SELECT objMain.isys_obj__id, objMain.isys_obj__title, ', strpos($query, 'SELECT '), strlen('SELECT '));

            try {
                $row = $location_dao->retrieve($query)->get_row();
            } catch (Exception $e) {
                // compatibility hack for version 1.19 (method parameters has changed)
                $query = isys_cmdb_dao_category_g_location::build_location_path_query(
                    8,
                    1000,
                    ' WHERE main.isys_catg_location_list__isys_obj__id = ' . $location_id,
                    true);
                $query = substr_replace($query, 'SELECT objMain.isys_obj__id, objMain.isys_obj__title, ', strpos($query, 'SELECT '), strlen('SELECT '));
                $row = $location_dao->retrieve($query)->get_row();
            }

            $parts = explode(' > ', $row['title']);

            // remove 'Root location'
            array_shift($parts);

            // add current object name
            $parts[] = $row['isys_obj__title'];

            $location_paths = [];
            foreach ($parts as $part) {
                $location_paths[] = trim($part);
            }
        }
        return implode(' > ', $location_paths);
    }

    /**
     * Import by file type.
     *
     * @param  string $file_type
     * @return array
     * @throws ImporterException
     * @throws isys_exception_general
     */
    public function import($file_type)
    {
        $this->_createLockFile();
        try {
            switch ($file_type) {
                case 'RACK': $this->_import_racks(); break;
                case 'PATCH': $this->_import_base_devices('patch', [self::HE, self::PROTOTYPE]); break;
                case 'DEVICE': $this->_import_base_devices('device', [self::HE, self::PROTOTYPE]); break;
                case 'MODULE': $this->_import_base_devices('module', ['Einbau Slot', self::PROTOTYPE]); break;
                case 'LINK_PP_TO_PP': $this->_import_connections('link'); break;
                case 'PATCH_PP_TO_DEVICE': $this->_import_connections('patch'); break;
                case 'SOCKET': $this->_import_sockets(); break;
            }
        } catch (Exception $e) {
            $msg = $this->_language->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_ERROR_COULD_NOT_IMPORT');
            $this->_log[] = "ERROR: $msg: " . $e->getMessage();
            self::deleteLockFile();

            if (! $this->_simulate) {
                $e = new ImporterException($e->getMessage());
                $e->setLog($this->_log);
                throw $e;
            }
        }
        self::deleteLockFile();
        return $this->_log;
    }

    /**
     * Check if an import is in progress.
     *
     * @return boolean
     */
    public static function isInProgress()
    {
        $lock_file = self::_getLockFile();
        return is_file($lock_file);
    }

    /**
     * Create lock file or fail if import is in progress.
     *
     * @throws isys_exception_general
     */
    private function _createLockFile()
    {
        if (self::isInProgress()) {
            throw new isys_exception_general($this->_language->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_IN_PROGRESS'));
        }
        $lock_file = self::_getLockFile();

        $fp = fopen($lock_file, 'w');
        fwrite($fp, '1');
        fclose($fp);
    }

    /**
     * Delete lock file.
     */
    public static function deleteLockFile()
    {
        $lock_file = self::_getLockFile();
        unlink($lock_file);
    }

    /**
     * Get lock file path.
     *
     * @return string
     */
    private static function _getLockFile()
    {
        global $g_dirs;
        return $g_dirs["temp"] . '__PATCH_IMPORT.lock';
    }

    /**
     * Import racks.
     *
     * @throws \PhpOffice\PhpSpreadsheet\Exception
     * @throws \PhpOffice\PhpSpreadsheet\Reader\Exception
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     */
    private function _import_racks()
    {
        $elements = $this->_get_elements();
        $racks = $elements['elements'];
        $columns = $elements['columns'];
        $requiredParameters = ['Höhe in HE'];
        $locationMapping = $this->_getLocationTypeMapping($columns);
        $lastLocationElement = array_pop($locationMapping);
        $location_dao = new isys_cmdb_dao_category_g_location($this->_database);
        foreach ($racks as $rack) {
            $name = $rack[$lastLocationElement['name']];
            if (! $this->_has_required_parameters($rack, $requiredParameters, $name)) {
                continue;
            }

            if ($this->_simulate) {
                $this->_log[] = "INFO: $name " . $this->_language->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_ABLE_TO_IMPORT');
                continue;
            }

            $parentLocationObjId = $this->_getOrCreateLocations($rack, $locationMapping);
            $objId = $this->_getObjectIdByLocation($parentLocationObjId, $name);
            if (! $objId) {
                $type = $lastLocationElement['type'] ?? 'C__OBJTYPE__ENCLOSURE';
                $objId = $location_dao->create_object($name, defined_or_default($type, 'C__OBJTYPE__ENCLOSURE'));
                $location_dao->create($objId, $parentLocationObjId);
            }

            $rackHeight = (int) $rack['Höhe in HE'];
            if ($rack['Halbe HE (ja/nein)']) {
                $val = strtolower($rack['Halbe HE (ja/nein)']);
                $this->_set_rack_half_he($objId, $val == 'ja');
                if ($val == 'ja') {
                    // duplicate rack height
                    $rackHeight = $rackHeight * 2;
                }
            }

            $this->_add_manufacturer($objId, null, $rack['Hersteller'], $rack['Typ']);
            $this->_add_rack_units($objId, $rackHeight);
            $this->_set_rack_sorting($objId, $rack, 'HE "1" oben (ja/nein)');

            $msg = $this->_language->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_ELEMENT_SUCCESS');
            $this->_log[] = "INFO: $name $msg";
        }
    }

    /**
     * Import patches.
     *
     * @param  string $type
     * @param  array $requiredParameters
     * @throws \PhpOffice\PhpSpreadsheet\Exception
     * @throws \PhpOffice\PhpSpreadsheet\Reader\Exception
     * @throws isys_exception_api
     * @throws isys_exception_api_validation
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     */
    private function _import_base_devices($type, $requiredParameters)
    {
        $elements = $this->_get_elements();
        $devices = $elements['elements'];
        $columns = $elements['columns'];
        $locationMapping = $this->_getLocationTypeMapping($columns);
        $lastLocationElement = array_pop($locationMapping);
        foreach ($devices as $device) {
            $name = $device[$lastLocationElement['name']];
            if (! $this->_has_required_parameters($device, $requiredParameters, $name)) {
                continue;
            }

            if ($this->_simulate) {
                $this->_log[] = "INFO: $name " . $this->_language->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_ABLE_TO_IMPORT');
                continue;
            }

            $objId = $this->_get_or_duplicate_object($device, $locationMapping, $lastLocationElement, self::PROTOTYPE, $type == 'device');
            $template_obj_id = $this->_get_template($device[self::PROTOTYPE]);
            $serialNumber = !empty($device['SN']) ? $device['SN'] : $device['Seriennummer'];
            $this->_add_manufacturer($objId, $template_obj_id, $device['Hersteller'], $device['Typ'], $serialNumber);
            $this->_add_rack_units($objId);
            $this->_add_inventory_number($objId, $template_obj_id, $device, 'Inventarnummer');
            $this->_remove_purpose($objId);

            if ($type == 'patch') {
                $this->_add_rack_position($objId, $device, self::HE, $locationMapping);
            } elseif ($type == 'device') {
                $this->_add_rack_position($objId, $device, self::HE, $locationMapping);
                $this->_add_ip_address($objId, $device, 'IP Adresse');
            } elseif ($type == 'module') {
                // update chassis
                $parentLocationObjId = $this->_getOrCreateLocations($device, $locationMapping);
                $this->_add_module_to_chassis($parentLocationObjId, $objId, $device['Einbau Slot']);
            }

            $msg = $this->_language->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_ELEMENT_SUCCESS');
            $this->_log[] = "INFO: $name $msg";
        }
    }

    /**
     * Import link patch panel to patch panel.
     *
     * @param  string $connectionType
     * @throws \PhpOffice\PhpSpreadsheet\Exception
     * @throws \PhpOffice\PhpSpreadsheet\Reader\Exception
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     * @throws Exception
     */
    private function _import_connections($connectionType)
    {
        $elements = $this->_get_elements(false);

        // get location mapping
        $requiredParameters = ['Portnummer'];
        $connections = $elements['elements'];
        $columns = $elements['columns'];
        $leftLocationMapping = $this->_getLocationTypeMapping($columns, 'left');
        $lastLeftLocationElement = array_pop($leftLocationMapping);
        $leftNameColumn = $lastLeftLocationElement['name'];
        $rightLocationMapping = $this->_getLocationTypeMapping($columns, 'right');
        $lastRightLocationElement = array_pop($rightLocationMapping);
        $rightNameColumn = $lastRightLocationElement['name'];

        $leftObjType = $lastLeftLocationElement['type'];
        $rightObjType = $lastRightLocationElement['type'];
        foreach ($connections as $connection) {
            $leftObject = [];
            $rightObject = [];
            $isRightObject = false;
            foreach ($connection as $i => $cell) {
                $column = $columns[$i];
                $rightColumn = $rightLocationMapping[$column['name']] ?: null;
                if ($rightColumn && $i == $rightColumn['index']) {
                    $isRightObject = true;
                }
                if ($isRightObject) {
                    $rightObject[$column['name']] = $cell;
                } else {
                    $leftObject[$column['name']] = $cell;
                }
            }

            // validate properties
            if (! $this->_has_required_parameters($leftObject, $requiredParameters, $leftObject[$leftNameColumn] ?: null) ||
                ! $this->_has_required_parameters($rightObject, $requiredParameters, $rightObject[$rightNameColumn] ?: null)) {
                continue;
            }

            $leftParentObjId = $this->_getLocationByPath($leftObject, $leftLocationMapping);
            $leftObjId = $this->_getObjectIdByLocation($leftParentObjId, $leftObject[$leftNameColumn], $leftObjType);
            $rightParentObjId = $this->_getLocationByPath($rightObject, $rightLocationMapping);
            $rightObjId = $this->_getObjectIdByLocation($rightParentObjId, $rightObject[$rightNameColumn], $rightObjType);
            if ($leftObjId && $rightObjId) {
                // try to get modules of both devices
                $leftModuleObjId = $this->_get_module_by_slot_nr($leftObjId, $leftObject['Portreihe / Slot'] ?: null);
                $leftObjId = ! empty($leftModuleObjId) ? $leftModuleObjId : $leftObjId;

                $rightModuleObjId = $this->_get_module_by_slot_nr($rightObjId, $rightObject['Portreihe / Slot'] ?: null);
                $rightObjId = ! empty($rightModuleObjId) ? $rightModuleObjId : $rightObjId;

                // add connection
                $cableName = isset($rightObject['Kabelbez.']) ? $rightObject['Kabelbez.'] : null;
                $connectorType = $connectionType == 'link' ? C__CONNECTOR__INPUT : C__CONNECTOR__OUTPUT;
                try {
                    $sourceConnectorId = $this->_getConnectorId($leftObjId, $leftObject[$leftNameColumn], $leftObject['Portnummer'], $connectorType);
                    $targetConnectorId = $this->_getConnectorId($rightObjId, $rightObject[$rightNameColumn], $rightObject['Portnummer'], $connectorType);
                } catch (Exception $e) {
                    $this->_log[] = "ERROR: " . $e->getMessage();
                    if ($this->_simulate) {
                        continue;
                    }
                    throw $e;
                }

                if ($this->_simulate) {
                    $msg = $leftObject[$leftNameColumn] . '/' . $leftObject['Portnummer'] . ' - ' . $rightObject[$rightNameColumn] . '/' . $rightObject['Portnummer'];
                    $this->_log[] = "INFO: $msg " . $this->_language->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_ABLE_TO_IMPORT');
                    continue;
                }

                $gridDao = new isys_cmdb_dao_smview_patch_grid($this->_database);
                $cableObjId = $gridDao->assign_connectors($sourceConnectorId, $targetConnectorId, $cableName, true);
                if (isset($rightObject['Länge'])) {
                    $l = $rightObject['Länge'];
                    if (is_string($l)) {
                        $l = str_replace(',', '.', $l);
                    }
                    $cableLength = (float) $l;
                    $this->_updateCableLength($cableObjId, $cableLength);
                }
            } else {
                $lang = isys_application::instance()->container->get('language');
                $message = $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_ERROR_NO_LOCATION');
                if (! $leftObjId) {
                    $this->_log[] = "ERROR: $message: " . $leftObject[$leftNameColumn] . ' - ' . $leftObject['Portnummer'];;
                }
                if (! $rightObjId) {
                    $this->_log[] = "ERROR: $message: " . $rightObject[$rightNameColumn] . ' - ' . $rightObject['Portnummer'];
                }
            }
        }
    }

    /**
     * Try to get module within a chassis, if slot number is set.
     *
     * @param  int $chassis_obj_id
     * @param  string|null $slotNr
     * @return int|null
     */
    private function _get_module_by_slot_nr($chassis_obj_id, $slotNr = null)
    {
        if (empty($slotNr)) {
            // value 0 means also no slot
            return null;
        }

        $slot_dao = isys_cmdb_dao_category_s_chassis_slot::instance($this->_database);
        $relation_dao = isys_cmdb_dao_category_g_relation::instance($this->_database);
        $slots = $slot_dao->get_data_as_array(null, $chassis_obj_id, null, null, defined_or_default('C__RECORD_STATUS__NORMAL'));
        if (empty($slots)) {
            // device has no slots
            return null;
        }

        foreach ($slots as $s) {
            // try to get slot by its number
            if ($s['isys_cats_chassis_slot_list__title'] != $slotNr) {
                continue;
            }

            // check if slot is assigned to a chassis
            $assigned_chassis = $slot_dao->get_assigned_chassis_items_by_cat_id($s['isys_cats_chassis_slot_list__id']);
            if (empty($assigned_chassis)) {
                // this slot is not assigned to a chassis
                return null;
            }

            // check if this chassis has an relation to a module
            foreach ($assigned_chassis as $c) {
                $relation_id = $c['isys_cats_chassis_list__isys_catg_relation_list__id'];
                if (empty($relation_id)) {
                    // this chassis has no device assigned
                    continue;
                }

                // return the module object id
                $relation = $relation_dao->get_data($relation_id)->get_row();
                return $relation['isys_catg_relation_list__isys_obj__id__slave'];
            }
        }
        return null;
    }

    /**
     * Import sockets.
     *
     * @throws \PhpOffice\PhpSpreadsheet\Exception
     * @throws \PhpOffice\PhpSpreadsheet\Reader\Exception
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     */
    private function _import_sockets()
    {
        $elements = $this->_get_elements(false);

        // get location mapping
        $requiredParameters = ['Portnummer'];
        $connections = $elements['elements'];
        $columns = $elements['columns'];
        $leftLocationMapping = $this->_getLocationTypeMapping($columns, 'left');
        $lastLeftLocationElement = array_pop($leftLocationMapping);
        $leftNameColumn = $lastLeftLocationElement['name'];
        $socketLocationMapping = $this->_getLocationTypeMapping($columns, 'right');

        $baseObjType = $lastLeftLocationElement['type'];
        foreach ($connections as $connection) {
            $baseObject = [];
            $socketObject = [];
            $isSocketObject = false;
            foreach ($connection as $i => $cell) {
                $column = $columns[$i];
                $rightColumn = $socketLocationMapping[$column['name']] ?: null;
                if ($rightColumn && $i == $rightColumn['index']) {
                    $isSocketObject = true;
                }
                if ($isSocketObject) {
                    $socketObject[$column['name']] = $cell;
                } else {
                    $baseObject[$column['name']] = $cell;
                }
            }

            // validate properties
            if (! $this->_has_required_parameters($baseObject, $requiredParameters, $baseObject[$leftNameColumn] ?: null)) {
                continue;
            }

            $baseParentObjId = $this->_getLocationByPath($baseObject, $leftLocationMapping);
            $baseObjId = $this->_getObjectIdByLocation($baseParentObjId, $baseObject[$leftNameColumn], $baseObjType);

            // try to get 'deepest' valid location
            $socketLocationMapping = $this->_getLocationTypeMapping($columns, 'right');
            try {
                $socketObjId = $this->_getOrCreateLocations($socketObject, $socketLocationMapping);
            } catch (Exception $e) {
                if ($this->_simulate) {
                    $this->_log[] = "ERROR: " . $e->getMessage();
                    continue;
                }
                throw $e;
            }
            $item = array_pop($socketLocationMapping);

            $lastElementName = $socketObject[$item['name']];
            if ($baseObjId && $socketObjId) {
                $connector_dao = new isys_cmdb_dao_category_g_connector($this->_database);
                $connectors = $connector_dao->get_data_as_array(null, $socketObjId);
                $existing_connector_in = null;
                $existing_connector_out = null;
                foreach ($connectors as $c) {
                    if (strtolower($c['isys_catg_connector_list__title']) == strtolower($socketObject['Anschluss'])) {
                        if ($c['isys_catg_connector_list__type'] ==  C__CONNECTOR__INPUT) {
                            $existing_connector_in = $c['isys_catg_connector_list__id'];
                        } else {
                            $existing_connector_out = $c['isys_catg_connector_list__id'];
                        }
                    }
                }

                if ($this->_simulate) {
                    $msg = $baseObject[$leftNameColumn] . '/' . $baseObject['Portnummer'] . ' - ' . $lastElementName . '/' . $socketObject['Anschluss'];
                    $this->_log[] = "INFO: $msg " . $this->_language->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_ABLE_TO_IMPORT');
                    continue;
                }

                if (! $existing_connector_in) {
                    // create a new input
                    $existing_connector_in = $connector_dao->create($socketObjId, C__CONNECTOR__INPUT, null, null, $socketObject['Anschluss'], null, $existing_connector_out);
                }

                if (! $existing_connector_out) {
                    // create a sibling output
                    $connector_dao->create($socketObjId, C__CONNECTOR__OUTPUT, null, null, $socketObject['Anschluss'], null, $existing_connector_in);
                }

                // add connection
                $connectorType = C__CONNECTOR__INPUT;
                try {
                    $sourceConnectorId = $this->_getConnectorId($baseObjId, $baseObject[$leftNameColumn], $baseObject['Portnummer'], $connectorType);
                    $targetConnectorId = $existing_connector_in;
                } catch (Exception $e) {
                    $this->_log[] = "ERROR: " . $e->getMessage();
                    if ($this->_simulate) {
                        continue;
                    }
                    throw $e;
                }

                $gridDao = new isys_cmdb_dao_smview_patch_grid($this->_database);
                $cableName = isset($socketObject['Kabelbez.']) ? $socketObject['Kabelbez.'] : null;
                $cableObjId = $gridDao->assign_connectors($sourceConnectorId, $targetConnectorId, $cableName, true);
                if (! empty($socketObject['Länge'])) {
                    $l = $socketObject['Länge'];
                    if (is_string($l)) {
                        $l = str_replace(',', '.', $l);
                    }
                    $cableLength = (float) $l;
                    $this->_updateCableLength($cableObjId, $cableLength);
                }
                if (! empty($socketObject['Endgerät'])) {
                    $this->_updateEndDevice($socketObjId, $socketObject['Endgerät']);
                }
            } else {
                $lang = isys_application::instance()->container->get('language');
                $message = $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_ERROR_NO_LOCATION');
                $name = $baseObject[$leftNameColumn] . '/' . $baseObject['Portnummer'] . ' - ' . $lastElementName . '/' . $socketObject['Anschluss'];
                $this->_log[] = "ERROR: $message: $name";
            }
        }
    }

    /**
     * Get elements from excel file.
     *
     * @param  boolean $indexColumns
     * @return array
     * @throws \PhpOffice\PhpSpreadsheet\Exception
     * @throws \PhpOffice\PhpSpreadsheet\Reader\Exception
     */
    private function _get_elements($indexColumns = true)
    {
        $column = 1;
        $row = 1;

        $reader = new Xlsx();
        if (! $reader->canRead($this->_path)) {
            $reader = new Xls();
        }
        $reader->setReadDataOnly(true);
        $spreadsheet = $reader->load($this->_path);
        $sheet = $spreadsheet->getActiveSheet();

        $columns = [];
        $hasTypeRow = false;
        for ($i = 1; $i < 99; $i++) {
            $cell = $sheet->getCellByColumnAndRow($column, $row, false);
            if (!$cell) {
                break;
            }
            $columnName = trim($cell->getValue());
            $columnIndex = $indexColumns ? $columnName : $i;
            $columns[$columnIndex] = [
                'index' => $column,
                'name'  => $columnName,
            ];
            $typeCell = $sheet->getCellByColumnAndRow($column, $row + 1, false);
            if ($typeCell) {
                $hasTypeRow = true;
                $t = trim($typeCell->getValue());
                if (strpos($t, 'C__') === 0) {
                    $columns[$columnIndex]['type'] = $t;
                }
            }
            $column++;
        }

        // get first index
        $firstIndex = null;
        foreach ($columns as $column) {
            $index = (int) $column['index'];
            if (! $firstIndex) {
                $firstIndex = $index;
            }
            $firstIndex = $index < $firstIndex ? $index : $firstIndex;
        }

        $elements = [];
        $row += $hasTypeRow ? 2 : 1;
        while (true) {
            $element = [];
            foreach ($columns as $column) {
                $index = $column['index'];
                $cell = $sheet->getCellByColumnAndRow($index, $row, false);
                $elementIndex = $indexColumns ? $column['name'] : $index;
                $element[$elementIndex] = $cell ? trim($cell->getCalculatedValue()) : null;
                if ($index == $firstIndex && $element[$elementIndex] == null) {
                    // last row
                    break 2;
                }
            }
            $elements[] = $element;
            $row++;
        }

        return [
            'columns'  => $columns,
            'elements' => $elements,
        ];
    }

    /**
     * Check if element has required parameters.
     *
     * @param  array $element
     * @param  array $requiredParameters
     * @param  string $name
     * @return boolean
     */
    private function _has_required_parameters($element, $requiredParameters, $name = '')
    {
        $isValid = true;
        foreach ($requiredParameters as $p) {
            if ($element[$p] == null) {
                $msg = $this->_language->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_ERROR_PARAMETER_MISSING');
                $this->_log[] = "ERROR: $name $msg: '$p'";
                $isValid = false;
            }
        }
        return $isValid;
    }

    /**
     * Get or duplicate object.
     *
     * @param  array $object
     * @param  array $locationMapping
     * @param  array $lastLocationElement
     * @param  string $templateProp
     * @param  boolean $withModules
     * @return int
     * @throws Exception
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     */
    private function _get_or_duplicate_object($object, $locationMapping, $lastLocationElement, $templateProp, $withModules = false)
    {
        $location_dao = new isys_cmdb_dao_category_g_location($this->_database);
        $parentLocationObjId = $this->_getOrCreateLocations($object, $locationMapping);
        $name = $object[$lastLocationElement['name']];
        $objId = $this->_getObjectIdByLocation($parentLocationObjId, $name);
        if (! $objId) {
            $duplicator = new Duplicator();
            $template_obj_id = $this->_get_template($object[$templateProp]);
            $objId = $duplicator->duplicate($template_obj_id, $name);
            if (empty($objId)) {
                throw new Exception("Could not duplicate object '$name'");
            }
            $location_dao->create($objId, $parentLocationObjId);

            if ($withModules) {
                $this->_duplicate_chassis_modules($template_obj_id, $objId);
            }
        }
        return $objId;
    }

    /**
     * Get location type mapping.
     *
     * @param  array $columns
     * @param  string $side
     * @return array
     */
    private function _getLocationTypeMapping($columns, $side = null)
    {
        $locationMapping = [];
        $isLeftSide = true;
        foreach ($columns as $c) {
            if (! isset($c['type'])) {
                if ($side != 'right') {
                    // break on first column without a type
                    break;
                } else {
                    $isLeftSide = false;
                    continue;
                }
            }

            if ($side == 'right' && $isLeftSide) {
                continue;
            }

            $name = $c['name'];
            $locationMapping[$name] = [
                'index' => $c['index'],
                'name'  => $name,
                'type'  => $c['type'],
            ];
        }
        return $locationMapping;
    }

    /**
     * Get or create missing locations.
     *
     * @param  array $object
     * @param  array $locationMapping
     * @return int|null
     * @throws Exception
     * @throws isys_exception_dao_cmdb
     */
    private function _getOrCreateLocations($object, $locationMapping)
    {
        // try to get location by path (short cut)
        $locationId = $this->_getLocationByPath($object, $locationMapping);
        if ($locationId) {
            return $locationId;
        }

        // create all missing locations, starting from top
        $location_dao = new isys_cmdb_dao_category_g_location($this->_database);
        $parentLocationId = 1;
        foreach ($locationMapping as $l) {
            $index = $l['name'];
            $name = $object[$index];
            $type = $l['type'];
            if (empty($type)) {
                // can not get location without type
                $lang = isys_application::instance()->container->get('language');
                $message = $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_ERROR_NO_TYPE');
                $name = $l['name'];
                throw new Exception("$message: $name");
            }

            if (! isset($name)) {
                // can not get location without name
                $lang = isys_application::instance()->container->get('language');
                $message = $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_ERROR_NO_NAME');
                $name = $l['type'];
                throw new Exception("$message: $name");
            }

            $objId = $this->_getObjectIdByLocation($parentLocationId, $name, $type);
            if (! $objId) {
                if ($this->_simulate) {
                    $lang = isys_application::instance()->container->get('language');
                    $message = $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_ABORT_ON_NEW_LOCATION');
                    $this->_log[] = "INFO: $message";
                    return null;
                }

                // create new location
                $typeObj = $location_dao->get_object_type(null, $type);
                $objId = $location_dao->create_object($name, $typeObj['isys_obj_type__id']);
                $location_dao->create($objId, $parentLocationId);
            }
            $parentLocationId = $objId;
        }
        return $parentLocationId;
    }

    /**
     * Get location by path from cache.
     *
     * @param  array $object
     * @param  array $locationMapping
     * @return int|false|null
     */
    private function _getLocationByPath($object, $locationMapping)
    {
        $locationPaths = [];
        foreach ($locationMapping as $l) {
            $index = $l['name'];
            $locationPaths[] = $object[$index];
        }

        // try to get location by path (short cut)
        $location_path = implode(self::$PATH_SEPARATOR, $locationPaths);
        if (! in_array($location_path, $this->_locations)) {
            return null;
        }
        return array_search($location_path, $this->_locations);
    }

    /**
     * Get object ID by location.
     *
     * @param  int $parentLocationObjId
     * @param  string $name
     * @param  string|null $type
     * @return int|null
     */
    private function _getObjectIdByLocation($parentLocationObjId, $name, $type = null)
    {
        if (empty($parentLocationObjId)) {
            return null;
        }

        $location_dao = new isys_cmdb_dao_category_g_location($this->_database);
        $condition = "AND isys_obj.isys_obj__title= '$name' "
            . "AND  isys_catg_location_list.isys_catg_location_list__parentid = $parentLocationObjId ";
        if ($type) {
            $condition .= "AND isys_obj_type.isys_obj_type__const= '$type' ";
        }
        $row = $location_dao->get_data(null, null, $condition, null, defined_or_default('C__RECORD_STATUS__NORMAL'))->get_row();
        return $row ? $row['isys_obj__id'] : null;
    }

    /**
     * Get template.
     *
     * @param  string $name
     * @return int|null
     * @throws Exception
     * @throws Exception
     */
    private function _get_template($name)
    {
        $lang = isys_application::instance()->container->get('language');
        $location_path = $this->_template_location_path;
        if (!in_array($location_path, $this->_locations)) {
            $message = $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_ERROR_NO_TEMPLATE_LOCATION');
            throw new Exception($message . ": $location_path");
        }
        $location_id = array_search($location_path, $this->_locations);
        $template_obj_id = $this->_getObjectIdByLocation($location_id, $name);
        if (empty($template_obj_id)) {
            $message = $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_ERROR_NO_TEMPLATE');
            $path = $location_path . " > $name";
            throw new Exception($message . ": $path");
        }
        return $template_obj_id;
    }

    /**
     * Add manufacturer.
     *
     * @param  int $objId
     * @param  string $manufacturer
     * @param  string|null $model
     * @param  string|null $serial_number
     * @return int
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     */
    private function _add_manufacturer($objId, $template_obj_id, $manufacturer, $model = null, $serial_number = null)
    {
        $model_dao = new isys_cmdb_dao_category_g_model($this->_database);
        $templateRow = $template_obj_id ? $model_dao->get_data(null, $template_obj_id)->get_row() : null;
        if (empty($manufacturer)) {
            $manufacturer = $templateRow ? $templateRow['isys_model_manufacturer__title'] : null;
        }
        $sql = "SELECT isys_model_manufacturer__id from isys_model_manufacturer WHERE isys_model_manufacturer__title = '$manufacturer'";
        $row = $model_dao->retrieve($sql)->get_row();
        if (! $row) {
            $sql = "INSERT INTO isys_model_manufacturer SET
                                        isys_model_manufacturer__title = '$manufacturer',
                                        isys_model_manufacturer__description = '$manufacturer',
                                        isys_model_manufacturer__status = " . defined_or_default('C__RECORD_STATUS__NORMAL');
            $model_dao->update($sql);
            $manufacturer_id = $model_dao->get_last_insert_id();
        } else {
            $manufacturer_id = $row['isys_model_manufacturer__id'];
        }
        $model_id = $model_dao->create($objId, $manufacturer_id);

        if (empty($model)) {
            $model = $templateRow ? $templateRow['isys_model_title__title'] : null;
        }

        if ($model) {
            $sql = "SELECT isys_model_title__id from isys_model_title WHERE isys_model_title__isys_model_manufacturer__id = $manufacturer_id AND isys_model_title__title = '$model'";
            $row = $model_dao->retrieve($sql)->get_row();
            if (! $row) {
                $sql = "INSERT INTO isys_model_title SET
                                            isys_model_title__title = '$model',
                                            isys_model_title__description = '$model',
                                            isys_model_title__isys_model_manufacturer__id = $manufacturer_id,
                                            isys_model_title__status = " . defined_or_default('C__RECORD_STATUS__NORMAL');
                $model_dao->update($sql);
                $title_id = $model_dao->get_last_insert_id();
            } else {
                $title_id = $row['isys_model_title__id'];
            }
            $model_dao->save_data($model_id, ['title' => $title_id]);
        }

        if (empty($serial_number)) {
            $serial_number = $templateRow ? $templateRow['isys_catg_model_list__serial'] : null;
        }

        if ($serial_number) {
            $model_dao->save_data($model_id, ['serial' => $serial_number]);
        }
        return $manufacturer_id;
    }

    /**
     * Add rack units.
     *
     * @param  int $objId
     * @param  int|null $units
     * @return boolean
     */
    private function _add_rack_units($objId, $units = null)
    {
        $form_dao = new isys_cmdb_dao_category_g_formfactor($this->_database);
        $row = $form_dao->get_data(null, $objId)->get_row();
        if (empty($row)) {
            $units = $units ?? 1;
            return $form_dao->create_data([
                'isys_obj__id' => $objId,
                'rackunits'    => (int) $units,
            ]);
        } else {
            if (isset($units)) {
                return $form_dao->save_data($row['isys_catg_formfactor_list__id'], [
                    'rackunits' => (int) $units,
                ]);
            } else {
                return true;
            }
        }
    }

    /**
     * Add rack position.
     *
     * @param  int $objId
     * @param  array $obj
     * @param  string $key
     * @param  array $locationMapping
     * @return boolean
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     */
    private function _add_rack_position($objId, $obj, $key, $locationMapping)
    {
        if (!array_key_exists($key, $obj)) {
            return null;
        }

        $he = (int)$obj[$key];
        $he = $he / 2;
        $pos = $he;

        $parentLocationObjId = $this->_getOrCreateLocations($obj, $locationMapping);
        if (! empty($parentLocationObjId)) {
            // check slot sorting direction of parent object (rack)
            $enclosure_dao = new isys_cmdb_dao_category_s_enclosure($this->_database);
            $enclosure_row = $enclosure_dao->get_data(null, $parentLocationObjId)->get_row();
            if (! empty($enclosure_row) && $enclosure_row['isys_cats_enclosure_list__slot_sorting'] == 'desc') {
                // calculate descending position
                $formFactor_dao = new isys_cmdb_dao_category_g_formfactor($this->_database);
                $formFactor_row = $formFactor_dao->get_data(null, $parentLocationObjId)->get_row();
                $rackUnits = (int) $formFactor_row['isys_catg_formfactor_list__rackunits'];
                $pos = $rackUnits - $he + 1;
            }
        }

        $location_dao = new isys_cmdb_dao_category_g_location($this->_database);
        $row = $location_dao->get_data(null, $objId)->get_row();
        return $location_dao->save_data($row['isys_catg_location_list__id'], [
            'pos'       => $pos,
            'insertion' => 1,
            'option'    => 3,
        ]);
    }

    /**
     * Set rack sorting.
     *
     * @param  int $objId
     * @param  array $obj
     * @param  string $key
     * @return boolean
     */
    private function _set_rack_sorting($objId, $obj, $key)
    {
        $val = $obj[$key] ? strtolower($obj[$key]) : null;
        $sorting = $val && $val == 'nein' ? isys_cmdb_dao_category_s_enclosure::C__SLOT_SORTING__DESC : isys_cmdb_dao_category_s_enclosure::C__SLOT_SORTING__ASC;
        $dao = new isys_cmdb_dao_category_s_enclosure($this->_database);
        $row = $dao->get_data(null, $objId)->get_row();
        if (! $row) {
            return $dao->create_data([
                'isys_obj__id' => $objId,
                'slot_sorting' => $sorting,
            ]);
        } else {
            return $dao->save_data($row['isys_cats_enclosure_list__id'], [
                'slot_sorting' => $sorting
            ]);
        }
    }

    /**
     * Set half HE.
     *
     * @param  int $objId
     * @param  boolean $hasHalfHe
     * @return boolean
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     */
    private function _set_rack_half_he($objId, $hasHalfHe = true)
    {
        $dao = new isys_cmdb_dao_category_g_smview_patch_formfactor($this->_database);
        $row = $dao->get_data(null, $objId)->get_row();
        $halfHe = $hasHalfHe ? 1 : 0;
        if (! $row) {
            return $dao->create_data([
                'isys_obj__id' => $objId,
                'half_he'      => $halfHe,
            ]);
        } else {
            return $dao->save_data($row['isys_catg_smview_patch_formfactor_list__id'], [
                'half_he' => $halfHe
            ]);
        }
    }

    /**
     * Add inventory number.
     *
     * @param  int $objId
     * @param  array $obj
     * @param  string $key
     * @return boolean|null
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     */
    private function _add_inventory_number($objId, $template_obj_id, $obj, $key)
    {
        $dao = new isys_cmdb_dao_category_g_accounting($this->_database);
        if (!array_key_exists($key, $obj) || empty($obj[$key])) {
            $templateRow = $dao->get_data(null, $template_obj_id)->get_row();
            if (! $templateRow) {
                return null;
            }
            $value = $templateRow['isys_catg_accounting_list__inventory_no'];
        } else {
            $value = $obj[$key];
        }

        $dao = new isys_cmdb_dao_category_g_accounting($this->_database);
        $row = $dao->get_data(null, $objId)->get_row();
        if (! $row) {
            return $dao->create_data([
                'isys_obj__id' => $objId,
                'inventory_no' => $value,
            ]);
        }
        return $dao->save_data($row['isys_catg_accounting_list__id'], [
            'inventory_no' => $value
        ]);
    }

    /**
     * Add IP address.
     *
     * @param  int $objId
     * @param  array $obj
     * @param  string $key
     * @return int|null
     * @throws isys_exception_api
     * @throws isys_exception_api_validation
     */
    private function _add_ip_address($objId, $obj, $key)
    {
        if (!array_key_exists($key, $obj)) {
            return null;
        }

        $dao = new isys_cmdb_dao_category_g_ip($this->_database);
        $row = $dao->get_data(null, $objId)->get_row();
        if (! $row) {
            $catIpId = isys_cmdb_dao_category_g_ip::instance($this->_database)->get_category_id();
            $catIp = new isys_cmdb_dao_category_g_ip($this->_database);
            $modelIp = new isys_api_model_cmdb_category($catIp);
            $address = $obj[$key];

            $ipData = array(
                C__CMDB__GET__OBJECT => $objId,
                C__CMDB__GET__CATG   => $catIpId,
                'data' => array(
                    'description' => $address,
                ),
            );

            if (@inet_pton($address) !== false) {
                $ipData['data']['ipv4_address'] = $address;
            } else {
                $ipData['data']['hostname'] = $address;
            }
            $res = $modelIp->create($ipData);
            return $res['id'];
        }
        return $row['isys_catg_ip_list__id'];
    }

    /**
     * Add module to chassis.
     *
     * @param  int $prototypeObjId
     * @param  int $objId
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     */
    private function _duplicate_chassis_modules($prototypeObjId, $objId)
    {
        // get slots of prototype
        $chassis_dao = isys_cmdb_dao_category_s_chassis::instance($this->_database);
        $relation_dao = isys_cmdb_dao_category_g_relation::instance($this->_database);
        $location_dao = new isys_cmdb_dao_category_g_location($this->_database);
        $slot_dao = isys_cmdb_dao_category_s_chassis_slot::instance($this->_database);
        $slots = $slot_dao->get_data_as_array(null, $prototypeObjId, null, null, defined_or_default('C__RECORD_STATUS__NORMAL'));
        foreach ($slots as $s) {
            // add new chassis
            $chassisId = $chassis_dao->create_data(['isys_obj__id' => $objId]);

            // add new slot
            $slotId = $this->_addNewSlot($objId, null, $s);

            // get assigned chassis of this slot
            $assigned_chassis = $slot_dao->get_assigned_chassis_items_by_cat_id($s['isys_cats_chassis_slot_list__id']);
            foreach ($assigned_chassis as $c) {
                $relation_id = $c['isys_cats_chassis_list__isys_catg_relation_list__id'];
                if (empty($relation_id)) {
                    // this chassis has no device assigned
                    continue;
                }

                // get module of prototype
                $relation = $relation_dao->get_data($relation_id)->get_row();
                $moduleObjId = $relation['isys_catg_relation_list__isys_obj__id__slave'];
                $moduleObj = $location_dao->get_object_by_id($moduleObjId)->get_row();

                // duplicate this module and assign it to duplicated chassis
                $duplicator = new Duplicator();
                $name = $moduleObj['isys_obj__title'];
                if (strpos($name, 'Pro*_') !== false) {
                    $name = str_replace('Pro*_', '', $name);
                }
                $copiedModuleObjId = $duplicator->duplicate($moduleObjId, $name);
                $location_dao->create($copiedModuleObjId, $objId);

                // create new chassis relation and connection
                $chassis_dao->save($chassisId, 2, null, null, $copiedModuleObjId);
            }

            // assign slot to chassis
            $chassis_dao->assign_slot_to_chassis_item($chassisId, $slotId);
        }
    }

    /**
     * Add module to chassis.
     *
     * @param  int $chassisObjId
     * @param  int $moduleObjId
     * @param  string $slotNr
     * @return int
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     */
    private function _add_module_to_chassis($chassisObjId, $moduleObjId, $slotNr)
    {
        $chassis_dao = isys_cmdb_dao_category_s_chassis::instance($this->_database);
        $slot_dao = isys_cmdb_dao_category_s_chassis_slot::instance($this->_database);
        $relation_dao = isys_cmdb_dao_category_g_relation::instance($this->_database);

        // try to find assigned module to chassis
        $chassisRes = $chassis_dao->get_data(null, $chassisObjId);
        $chassisId = null;
        while ($row = $chassisRes->get_row()) {
            $relation_id = $row['isys_cats_chassis_list__isys_catg_relation_list__id'];
            if (! empty($relation_id)) {
                $relation = $relation_dao->get_data($relation_id)->get_row();
                if ($relation['isys_catg_relation_list__isys_obj__id__slave'] == $moduleObjId) {
                    $chassisId = $row['isys_cats_chassis_list__id'];
                }
            }
        }

        // try to get slot by its number
        $slots = $slot_dao->get_data_as_array(null, $chassisObjId, null, null, defined_or_default('C__RECORD_STATUS__NORMAL'));
        $slotId = null;
        foreach ($slots as $s) {
            if ($s['isys_cats_chassis_slot_list__title'] != $slotNr) {
                continue;
            }

            // check if chassis is already assigned to this slot
            $slotId = $s['isys_cats_chassis_slot_list__id'];
            $assigned_chassis = $slot_dao->get_assigned_chassis_items_by_cat_id($s['isys_cats_chassis_slot_list__id']);
            foreach ($assigned_chassis as $c) {
                if ($c['isys_cats_chassis_list__id'] == $chassisId) {
                    // module is already assigned to this chassis in this slot
                    return $slotId;
                }
            }
        }

        if (! $chassisId) {
            // create new chassis and assign this module
            $chassisId = $chassis_dao->create($chassisObjId, C__RECORD_STATUS__NORMAL, null, null, $moduleObjId);
        } else {
            // remove current slot assignments
            $chassis_dao->remove_slot_assignments($chassisId);
        }

        if (! $slotId) {
            // create new slot
            $slotId = $this->_addNewSlot($chassisObjId, $slotNr);
        }

        // assign chassis to slot
        $chassis_dao->assign_slot_to_chassis_item($chassisId, $slotId);
        return $slotId;
    }

    /**
     * Add new slot and update chassis if needed.
     *
     * @param  int $chassisObjId
     * @param  string|null $slotTitle
     * @param  array|null $slot
     * @return int
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     */
    private function _addNewSlot($chassisObjId, $slotTitle = null, $slot = null)
    {
        $chassis_view_dao = isys_cmdb_dao_category_s_chassis_view::instance($this->_database);
        $slot_dao = isys_cmdb_dao_category_s_chassis_slot::instance($this->_database);

        $pos = isset($slotTitle) ? ((int) $slotTitle) - 1 : null;
        $s_id = $slot_dao->create_data([
            'isys_obj__id' => $chassisObjId,
            'title'        => $slotTitle ?? $slot['isys_cats_chassis_slot_list__title'],
            'from_x'       => $pos ?? $slot['isys_cats_chassis_slot_list__x_from'],
            'to_x'         => $pos ?? $slot['isys_cats_chassis_slot_list__x_to'],
            'from_y'       => isset($slot) ? $slot['isys_cats_chassis_slot_list__y_from'] : 0,
            'to_y'         => isset($slot) ? $slot['isys_cats_chassis_slot_list__y_to'] : 0,
            'insertion'    => C__INSERTION__FRONT,
        ]);

        // update horizontal units of chassis
        $chassis_view = $chassis_view_dao->get_data(null, $chassisObjId)->get_row();
        $x = (int) $chassis_view['isys_cats_chassis_view_list__front_width'];
        $slotX = isset($slot) ? ((int) $slot['isys_cats_chassis_slot_list__x_to']) + 1 : (int) $slotTitle;
        if ($x < $slotX) {
            $chassis_view_dao->save_data($chassis_view['isys_cats_chassis_view_list__id'], [
                'front_x' => $slotX,
            ]);
        }
        return $s_id;
    }

    /**
     * Remove purpose if it is "Prototyp".
     *
     * @param  int $obj_id
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     */
    private function _remove_purpose($obj_id)
    {
        $dao = new isys_cmdb_dao_category_g_global($this->_database);
        $row = $dao->get_data(null, $obj_id)->get_row();
        if (! empty($row['isys_catg_global_list__isys_purpose__id'])) {
            if ($row['isys_purpose__title'] == 'Prototyp' || $row['isys_purpose__title'] == 'Template') {
                $dao->save_data($row['isys_catg_global_list__id'], ['purpose' => -1]);
            }
        }
    }

    /**
     * Get connector ID.
     *
     * @param  int $objId
     * @param  string $objName
     * @param  string $connectorTitle
     * @param  int $type
     * @return int|null
     * @throws Exception
     */
    private function _getConnectorId($objId, $objName, $connectorTitle, $type = C__CONNECTOR__INPUT)
    {
        $lang = isys_application::instance()->container->get('language');
        $message = $lang->get('LC__CMDB__CATG__SMVIEW_PATCH__UPLOAD_ERROR_NO_CONNECTOR');

        $connectorId = $this->_getConnectorByName($objId, $connectorTitle, $type);
        if ($connectorId) {
            return $connectorId;
        }

        if (strlen($connectorTitle) > 4) {
            throw new Exception("$message: $objName/$connectorTitle");
        }

        $origConnectorTitle = $connectorTitle;
        while (strlen($connectorTitle) <= 4) {
            $len = strlen($connectorTitle);
            $connectorTitle = str_pad($connectorTitle, $len + 1, '0', STR_PAD_LEFT);
            $connectorId = $this->_getConnectorByName($objId, $connectorTitle, $type);
            if ($connectorId) {
                return $connectorId;
            }
        }
        throw new Exception("$message: $objName/$origConnectorTitle");
    }

    /**
     * Get existing connector or create a new one.
     *
     * @param  int $objId
     * @param  string $objTitle
     * @param  int $type
     * @return int
     */
    private function _getConnectorByName($objId, $objTitle, $type = C__CONNECTOR__INPUT)
    {
        $dao = new isys_cmdb_dao_category_g_connector($this->_database);
        $sql = "AND isys_catg_connector_list.isys_catg_connector_list__title = '$objTitle' "
             . "AND isys_catg_connector_list.isys_catg_connector_list__type=" . $type;
        $port = $dao->get_data(null, $objId, $sql)->get_row();
        return $port ? $port['isys_catg_connector_list__id'] : null;
    }

    /**
     * Update cable properties.
     *
     * @param  int $cableObjId
     * @param  int $cableLength
     * @return boolean
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     */
    private function _updateCableLength($cableObjId, $cableLength)
    {
        $dao = new isys_cmdb_dao_category_g_cable($this->_database);
        $row = $dao->get_data(null, $cableObjId)->get_row();
        $sql = "SELECT * from isys_depth_unit WHERE isys_depth_unit__const = 'C__DEPTH_UNIT__METER'";
        $unitRow = $dao->retrieve($sql)->get_row();

        if (empty($row['isys_catg_cable_list__length'])) {
            $f = (int) $unitRow['isys_depth_unit__factor'];
            $cableLength = $f * $cableLength;
        } else {
            $cableLength = $row['isys_catg_cable_list__length'];
        }

        $data = [
            'length'      => $cableLength,
            'length_unit' => $unitRow['isys_depth_unit__id'],
        ];
        return $dao->save_data($row['isys_catg_cable_list__id'], $data);
    }

    /**
     * Update end device.
     *
     * @param  int $socketObjId
     * @param  string $endDevice
     * @return int|boolean
     * @throws isys_exception_dao
     * @throws isys_exception_dao_cmdb
     */
    private function _updateEndDevice($socketObjId, $endDevice)
    {
        $dao = new isys_cmdb_dao_category_g_smview_patch_end_device($this->_database);
        $row = $dao->get_data(null, $socketObjId)->get_row();
        if (! $row) {
            return $dao->create_data([
                'isys_obj__id' => $socketObjId,
                'name'         => $endDevice,
            ]);
        }
        return $dao->save_data($row['isys_catg_smview_patch_end_device_list__id'], [
            'name' => $endDevice
        ]);
    }
}
